# Copyright 1999-2017. Plesk International GmbH. All rights reserved.
package ArchiveContent::UnixArchiveContent;

use strict;
eval{require warnings;1;};

use Logging;
use AgentConfig;
use ArchiveContent::ArchiveContent;
use Cwd;

use vars qw|@ISA|;

@ISA = qw|ArchiveContent::ArchiveContent|;

sub new {
  my $self = {};
  bless( $self, shift );
  $self->_init(@_);
  return $self;
}

sub _init {
  my ( $self, $storage, $packer ) = @_;
  $self->{storage} = $storage;
  $self->{packer} = $packer;
  Logging::debug("Unix Archive content transport has been initialized");
}


sub hasLink {
  my ($self, $linkpath, $targetpath) = @_;
  Logging::debug("Check link '$linkpath' to '$targetpath'.");
  my $result;
  if (! -l $linkpath) {
    $result =  'false';
  } else {
    $result = (Cwd::realpath($linkpath) eq $targetpath) ? 'true' : 'false';
  }
  Logging::debug("Check link result: $result");
  return $result;
}

sub directoryExists {
  my ($self, $directory) = @_;
  Logging::debug("Check directory existence '$directory'.");
  my $result = -d $directory;
  Logging::debug("Result: $result");
  return $result;
}

### Private routines ###

sub _addTar {
  my $self = shift;
  my $cid_type = shift;
  my $dstFile = shift;
  my %options = @_;
  return undef if $self->_skipContent(%options);

  my $id = $self->{storage}->addTar( $dstFile, @_);
  return undef unless defined $id;
  return $self->_makeCidNode($id, $cid_type, @_);
}


sub _makeCidNode{
  my ($self, $id, $cid_type, %options) = @_;
  my $cid = XmlNode->new( 'cid' );
  $cid->setAttribute( 'type', $cid_type );
  $cid->setAttribute( 'unpacksize', $self->{storage}->getFilesUnpackSizeFromId($id) );
  $cid->setAttribute( 'path', $self->{storage}->getFilePathFromId( $id ) );
  my $files = $self->{storage}->getFilesFromId( $id );
  foreach my $filedata( @{$files} ){
    $cid->addChild( XmlNode->new( 'content-file', 'content' => $filedata->[0], 'attributes' => {'size' => $filedata->[1] } ) );
  }

  if (defined($options{'include'})) {
    my @includes = @{$options{'include'}};
    foreach my $inc (@includes) {
      $cid->addChild(XmlNode->new('include-item', 'content' => $inc));
    }
  }

  if (defined($options{'exclude'})) {
    my @excludes = @{$options{'exclude'}};
    foreach my $exc (@excludes) {
      $cid->addChild(XmlNode->new('exclude-item', 'content' => $exc));
    }
  }
  
  if (defined($options{'indexed'})) {
    $cid->setAttribute( 'indexed', 'true' );
  }

  return $cid;
}

sub _addDb {
  my $self = shift;
  my $cid_type = shift;
  my $dstFile = shift;
  my %options = @_;
  return if $self->_skipContent(%options);
  my $id = $self->{storage}->addDb( $dstFile, @_);
  if ( defined $id ) {
    return $self->_makeCidNode($id, $cid_type, @_);
  }
  return;
}

1;