// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

/* global SetHelpPrefix, SetContext, OpenHelpWindow */

Jsw.namespace('AdminPanel');
Jsw.namespace('AdminPanel.home');

/**
 * @class AdminPanel.home.UpdateProgressNotification
 * @extends Jsw.Component
 */
AdminPanel.home.UpdateProgressNotification = Class.create(Jsw.Component, {

    STATUS_WORKING: 'working',
    STATUS_DONE: 'done',
    STATUS_NOTWORKING: 'notworking',

    updateInterval: 5,

    /**
     * @cfg {String} message
     */

    _initConfiguration: function ($super, config) {
        $super(config);
        this._message = this._getConfigParam('message');
    },

    _addEvents: function () {
        this._processStatus({
            status: this.STATUS_WORKING,
            message: this._message,
        });
    },

    _scheduleUpdateStatus: function () {
        this._statusUpdate.bind(this).delay(this.updateInterval);
    },

    _statusUpdate: function () {
        new Ajax.Request(Jsw.prepareUrl('/update/get-update-data'), {
            method: 'get',
            onSuccess: this._onSuccess.bind(this),
            onFailure: this._onFailure.bind(this),
        });
    },

    _onSuccess: function (transport) {
        try {
            this._processStatus(transport.responseText.evalJSON());
        } catch (e) {
            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();
        }
    },

    _onFailure: function (transport) {
        // Schedule new update - control panel is probably down
        // or othewise unavailable due to updates installation.
        // Have no information to display.

        this._scheduleUpdateStatus();
    },

    _processStatus: function (response) {
        if (!response) {
            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();
        } else if (this.STATUS_WORKING === response.status) {
            this._componentElement.update(
                '<div class="msg-box msg-warning msg-update-running">' +
                '<div><div><div><div><div>' +
                '<div class="msg-content">' +
                    this._message +
                '</div>' +
                '</div></div></div></div></div>' +
                '</div>'
            );

            this._scheduleUpdateStatus();
        } else if (this.STATUS_DONE === response.status || this.STATUS_NOTWORKING === response.status) {
            // Stop asynchronous notification and reload the whole page
            Jsw.redirect('/');
        }
    },

});

/**
 * @class AdminPanel.home.UpdateResultNotification
 * @extends Jsw.Component
 */
AdminPanel.home.UpdateResultNotification = Class.create(Jsw.Component, {

    _initConfiguration: function ($super, config) {
        $super(config);

        this._status = this._getConfigParam('status', 'info');
        this._message = this._getConfigParam('message', '');
    },

    _initComponentElement: function ($super) {
        $super();

        this._updateComponentElement(
            '<div class="msg-box msg-' + this._status + '"><div><div><div><div><div>' +
            '<div class="msg-content">' + this._message + '</div></div></div></div></div></div></div>'
        );
    },

});

/**
 * @class AdminPanel.home.UpdateAvailableNotification
 * @extends Jsw.Component
 */
AdminPanel.home.UpdateAvailableNotification = Class.create(Jsw.Component, {

    _initComponentElement: function ($super) {
        $super();

        var id = this._getConfigParam('type');

        var buttonNow = this._getConfigParam('nowElement');
        var buttonElementNow = new Element('button', {
            type: 'submit',
        }).update(buttonNow.text);

        var buttonShow = this._getConfigParam('showElement');
        var buttonElementShow = new Element('button', {
            type: 'submit',
        }).update(buttonShow.text);

        var linkLater = this._getConfigParam('laterElement');
        var linkElementLater = new Element('a', {
            href: '#',
        }).update('<span>' + linkLater.text + '</span>');

        var noteElement = new Element('div');
        noteElement.addClassName('msg-box');
        noteElement.addClassName('msg-update');

        noteElement.update(
            '<div><div><div><div><div>' +
            '<div class="msg-content">' +
              '<table class="update-container" cellspacing="0" width="100%">' +
                '<tbody>' +
                  '<tr>' +
                    '<td class="update">' +
                      '<span class="update-info">' + this._getConfigParam('message', '') + '</span>' +
                    '</td>' +
                    '<td class="update-links">' +
                    '<span id="control-' + id + '-now" class="btn"></span>' +
                    '<span id="control-' + id + '-show" class="btn"></span>' +
                    '<span id="control-' + id + '-later"></span>' +
                    '</td>' +
                  '</tr>' +
                '</tbody>' +
              '</table>' +
              '</div>' +
              '</div></div></div></div></div>');

        this._updateComponentElement(noteElement);
        $('control-' + id + '-now').insert(buttonElementNow);
        $('control-' + id + '-show').insert(buttonElementShow);
        $('control-' + id + '-later').insert(linkElementLater);

        buttonElementNow.observe('click', function () {
            buttonNow.handler();
            return false;
        });

        buttonElementShow.observe('click', function () {
            buttonShow.handler();
            return false;
        });

        linkElementLater.observe('click', function () {
            linkLater.handler();
            return false;
        });
    },
});

/**
 * @class AdminPanel.home.UpgradeAvailableNotification
 * @extends Jsw.Component
 */
AdminPanel.home.UpgradeAvailableNotification = Class.create(Jsw.Component, {

    _initComponentElement: function ($super) {
        $super();

        var id = this._getConfigParam('type');

        var buttonShow = this._getConfigParam('showElement');
        var buttonElementShow = new Element('button', {
            type: 'submit',
        }).update(buttonShow.text);

        var linkLater = this._getConfigParam('laterElement');
        var linkElementLater = new Element('a', {
            href: '#',
        }).update('<span>' + linkLater.text + '</span>');

        var noteElement = new Element('div');
        noteElement.addClassName('msg-box');
        noteElement.addClassName('msg-update');

        noteElement.update(
            '<div><div><div><div><div>' +
            '<div class="msg-content">' +
              '<table class="update-container" cellspacing="0" width="100%">' +
                '<tbody>' +
                  '<tr>' +
                    '<td class="update">' +
                      '<span class="update-info">' + this._getConfigParam('message', '') + '</span>' +
                    '</td>' +
                    '<td class="update-links">' +
                    '<span id="control-' + id + '-show" class="btn"></span>' +
                    '<span id="control-' + id + '-later"></span>' +
                    '</td>' +
                  '</tr>' +
                '</tbody>' +
              '</table>' +
              '</div>' +
              '</div></div></div></div></div>');

        this._updateComponentElement(noteElement);
        $('control-' + id + '-show').insert(buttonElementShow);
        $('control-' + id + '-later').insert(linkElementLater);

        buttonElementShow.observe('click', function () {
            buttonShow.handler();
            return false;
        });

        linkElementLater.observe('click', function () {
            linkLater.handler();
            return false;
        });
    },
});

/**
 * @class AdminPanel.home.Panel
 * @extends Jsw.Panel
 */
AdminPanel.home.Panel = Class.create(Jsw.Panel, {

    _addTooltips: function () {
    },

    _getSubTitleHtml: function () {
        var descriptionText = this._getConfigParam('description');
        if (!descriptionText) {
            return '';
        }

        return '<div class="description">' +
                '<div class="description-area">' +
                    descriptionText +
                '</div>' +
            '</div>';
    },
});

/**
 * @class AdminPanel.home.PanelItemsGroup
 * @extends Jsw.ListContainer
 */
AdminPanel.home.PanelItemsGroup = Class.create(Jsw.ListContainer, {
});

/**
 * @class AdminPanel.home.PanelItem
 * @extends Jsw.SmallButton
 */
AdminPanel.home.PanelItem = Class.create(Jsw.SmallButton, {
});

/**
 * @class AdminPanel.Help
 * @singleton
 */
AdminPanel.Help = {
    setPrefix: function (helpPrefix) {
        // eslint-disable-next-line new-cap
        SetHelpPrefix(helpPrefix);
    },
    setContext: function (helpContext) {
        // eslint-disable-next-line new-cap
        SetContext(helpContext, '');
    },
    openHelpWindow: function (helpContext, helpPrefix, helpModule) {
        // eslint-disable-next-line new-cap
        OpenHelpWindow(helpContext, helpPrefix, helpModule);
    },
};

/**
 * @class AdminPanel.Subscription
 * @singleton
 */
AdminPanel.Subscription = {
    getControlPanelLoginHTML: function (id, options) {
        options = Object.extend({
            openInNew: false,
            label: '',
            hint: '',
            returnUrl: '',
            isClassicUi: '',
            showAll: false,
        }, options || {});

        if (options.isClassicUi) {
            options.returnUrl = '/subscription/overview/id/' + id;
        }
        var url = Jsw.baseUrl + '/subscription/login/id/' + id;

        if (options.showAll) {
            url += '/all/true';
        }

        if (options.returnUrl) {
            url += '/?returnUrl=' + Jsw.baseUrl + options.returnUrl;
        }

        return '<span>'
            + '<span class="tooltipData">'
                + options.hint
            + '</span>'
            + '<a href="' + url + '"'
                + ' class="s-btn sb-login"'
                + (options.isClassicUi ? '' : ' target="' + (options.openInNew ? '_blank' : '_top') + '"')
            + '>'
                + '<i><i><i><span>'
                    + options.label
                + '</span></i></i></i>'
            + '</a>'
        + '</span>';
    },
};

/**
 * @class AdminPanel.WebserverConfigurationFailure
 * @extends Jsw.Component
 */
AdminPanel.WebserverConfigurationFailure = Class.create(Jsw.Component, {

    _rebuildUrl: '/subscription/webserver-configuration-rebuild-all/',

    /**
     * @cfg {String} message
     */
    /**
     * @cfg {String} messageInProgress
     */
    /**
     * @cfg {String} messageSuccess
     */
    /**
     * @cfg {String} messageFailure
     */

    _initConfiguration: function ($super, config) {
        $super(config);

        this._message = this._getConfigParam('message');
        this._messageInProgress = this._getConfigParam('messageInProgress');
        this._messageSuccess = this._getConfigParam('messageSuccess');
        this._messageFailure = this._getConfigParam('messageFailure');
    },

    _initComponentElement: function ($super) {
        $super();

        Jsw.addStatusMessage('error', this._message);
    },

    _addEvents: function ($super) {
        $super();

        $('webserver-configuration-failure-rebuild-all').observe('click', this._rebuildAll.bind(this));
    },

    _rebuildAll: function () {
        new Ajax.Request(Jsw.prepareUrl(this._rebuildUrl), {
            method: 'post',
            onCreate: this._onCreateExecute.bind(this),
            onSuccess: this._onSuccess.bind(this),
            onException: this._onException.bind(this),
        });
    },

    _onCreateExecute: function () {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('warning', '<span class="ajax-loading">' + this._messageInProgress + '</span>');
    },

    _onSuccess: function (transport) {
        Jsw.clearStatusMessages();
        var response = transport.responseText.evalJSON();
        if ('success' === response.status) {
            Jsw.addStatusMessage('info', this._messageSuccess);
        } else {
            Jsw.addStatusMessage('error', this._messageFailure + response.message);
        }
    },

    _onException: function (transport, exception) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', 'Internal error: ' + exception);
    },
});
