// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

Jsw.namespace('AdminPanel.ConfirmationPopupManager');

AdminPanel.ConfirmationPopupManager.ConvertResellersToCustomers = Class.create(Jsw.ConfirmationPopupManager.PopupForm, {

    _onSuccessPreparePopup: function (transport) {
        this._clearMessages();

        try {
            this._response = transport.responseText.evalJSON();

            if ('success' === this._response.status) {
                $(this._formBoxAreaId).update('');
                $(this._formListAreaId).update(
                    '<table cellspacing="0" width="100%">' +
                        '<tbody id="' + this._formListItemsAreaId + '">' +
                        '</tbody>' +
                    '</table>'
                );

                var hasSubscriptions = false;
                $H(this._response.resellers).each(function (reseller) {
                    reseller = reseller.value;
                    if (reseller.subscriptions.length !== 0) {
                        hasSubscriptions = true;
                    }
                    this._addReseller(reseller);
                }, this);

                if (hasSubscriptions) {
                    $(this._formDescriptionId).insert({ bottom: '&nbsp;' + this.lmsg('descriptionExtended') });
                }
            } else {
                this._addErrorMessage(this._response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    },

    _addReseller: function (reseller) {
        if (reseller.subscriptions.length === 0) {
            this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';
            var itemHtml = '' +
                '<tr class="' + this._itemClass + '">' +
                    '<td><b>' + reseller.name.escapeHTML() + '</b></td>' +
                    '<td>&nbsp;</td>' +
                    '<td>&nbsp;</td>' +
                '</tr>';

            $(this._formListItemsAreaId).insert({ bottom: itemHtml });
        } else {
            var resellerName = reseller.name;
            $H(reseller.subscriptions).each(function (subscription) {
                this._addSubscription(subscription.value, resellerName);
                resellerName = ' ';
            }, this);
        }
    },

    _addSubscription: function (subscription, resellerName) {
        this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';
        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td><b>' + resellerName.escapeHTML() + '</b></td>' +
                '<td>' + subscription.name.escapeHTML() + '</td>' +
                '<td>' + this._getPlanSelectorHtml(subscription) + '</td>' +
            '</tr>';

        $(this._formListItemsAreaId).insert({ bottom: itemHtml });
    },

    _getPlanSelectorHtml: function (subscription) {
        var planSelectorHtml = '<select class="subscriptions" name="subscriptions[' + subscription.id + ']" id="" title="' + subscription.id + '">';
        $H(this._response.plans).each(function (plan) {
            plan = plan.value;
            planSelectorHtml += '<option value="' + plan.id + '">' + plan.name.escapeHTML() + '</option>';
        });
        planSelectorHtml += '</select>';

        return planSelectorHtml;
    },

    _getAdditionalParams: function (params) {
        $$('.subscriptions').each(function (row) {
            params.set('subscriptions[' + row.title + ']', row.value);
        });
        return params;
    },
});

AdminPanel.ConfirmationPopupManager.ConvertCustomersToResellers = Class.create(Jsw.ConfirmationPopupManager.PopupForm, {

    _onSuccessPreparePopup: function (transport) {
        this._clearMessages();

        try {
            this._response = transport.responseText.evalJSON();

            if ('success' === this._response.status) {
                $(this._formBoxAreaId).update('');
                $(this._formListAreaId).update(
                    '<table cellspacing="0" width="100%">' +
                        '<tbody id="' + this._formListItemsAreaId + '">' +
                        '</tbody>' +
                        '</table>'
                );

                $H(this._response.customers).each(function (customer) {
                    this._addCustomer(customer.value);
                }, this);
            } else {
                this._addErrorMessage(this._response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    },

    _addCustomer: function (customer) {
        this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';
        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
            '<td><b>' + customer.name.escapeHTML() + '</b></td>' +
            '<td>' + this._getPlanSelectorHtml(customer) + '</td>' +
            '</tr>';

        $(this._formListItemsAreaId).insert({ bottom: itemHtml });
    },

    _getPlanSelectorHtml: function (customer) {
        var planSelectorHtml = '<select class="customers" name="customers[' + customer.id + ']" id="customers-select-' + customer.id + '" title="' + customer.id + '">';
        $H(this._response.plans).each(function (plan) {
            plan = plan.value;
            planSelectorHtml += '<option value="' + plan.id + '">' + plan.name.escapeHTML() + '</option>';
        });
        planSelectorHtml += '</select>';

        return planSelectorHtml;
    },

    _getAdditionalParams: function (params) {
        $$('.customers').each(function (row) {
            params.set('customers[' + row.title + ']', row.value);
        });
        return params;
    },
});

AdminPanel.ConfirmationPopupManager.ChangeCustomersOwner = Class.create(Jsw.ConfirmationPopupManager.PopupForm, {

    _onSuccessPreparePopup: function (transport) {
        this._clearMessages();
        this._owners = $H();

        try {
            this._response = transport.responseText.evalJSON();

            if ('success' === this._response.status) {
                $(this._formBoxAreaId).update(
                    '<p>' + this.lmsg('ownerHint') + '</p>' +
                     this._getOwnerSelectorHtml()
                );
                $('owners').observe('change', this._onOwnerChange.bind(this));

                $(this._formListAreaId).update(
                    '<table cellspacing="0" width="100%">' +
                        '<tbody id="' + this._formListItemsAreaId + '">' +
                        '</tbody>' +
                    '</table>'
                );

                var hasSubscriptions = false;
                $H(this._response.customers).each(function (customer) {
                    customer = customer.value;
                    if (customer.subscriptions.length !== 0) {
                        hasSubscriptions = true;
                    }
                    this._addCustomer(customer);
                }, this);

                if (!hasSubscriptions) {
                    $(this._formDescriptionId).update('');
                }
            } else {
                this._addErrorMessage(this._response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    },

    _getOwnerSelectorHtml: function () {
        var ownerSelectorHtml = '<select class="owners" name="owners" id="owners">';
        $A(this._response.owners).each(function (owner) {
            ownerSelectorHtml += '<option value="' + owner.id + '">' + owner.name.escapeHTML() + '</option>';
            this._owners.set(owner.id, owner);
        }, this);
        ownerSelectorHtml += '</select>';

        return ownerSelectorHtml;
    },

    _onOwnerChange: function () {
        $H(this._response.customers).each(function (customer) {
            customer = customer.value;
            if (customer.subscriptions.length !== 0) {
                $H(customer.subscriptions).each(function (subscription) {
                    subscription = subscription.value;
                    $('plans-box-' + subscription.id).update(this._getPlanSelectorHtml(subscription));
                    $('ipV4-box-' + subscription.id).update(this._getIpAddressSelectorHtml(subscription, 'ipV4'));
                    $('ipV6-box-' + subscription.id).update(this._getIpAddressSelectorHtml(subscription, 'ipV6'));
                }, this);
            }
        }, this);
    },

    _addCustomer: function (customer) {
        if (customer.subscriptions.length === 0) {
            this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';
            var itemHtml = '' +
                '<tr class="' + this._itemClass + '">' +
                    '<td><b>' + customer.name.escapeHTML() + '</b></td>' +
                    '<td>&nbsp;</td>' +
                    '<td>&nbsp;</td>' +
                    '<td>&nbsp;</td>' +
                    '<td>&nbsp;</td>' +
                '</tr>';

            $(this._formListItemsAreaId).insert({ bottom: itemHtml });
        } else {
            var customerName = customer.name;
            $H(customer.subscriptions).each(function (subscription) {
                this._addSubscription(subscription.value, customerName);
                customerName = ' ';
            }, this);
        }
    },

    _addSubscription: function (subscription, customerName) {
        this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';

        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td><b>' + customerName.escapeHTML() + '</b></td>' +
                '<td>' + subscription.name.escapeHTML() + '</td>' +
                '<td id="plans-box-' + subscription.id + '">' + this._getPlanSelectorHtml(subscription) + '</td>' +
                '<td id="ipV4-box-' + subscription.id + '">' + this._getIpAddressSelectorHtml(subscription, 'ipV4') + '</td>' +
                '<td id="ipV6-box-' + subscription.id + '">' + this._getIpAddressSelectorHtml(subscription, 'ipV6') + '</td>' +
            '</tr>';

        $(this._formListItemsAreaId).insert({ bottom: itemHtml });
    },

    _getPlanSelectorHtml: function (subscription) {
        var selectedOwner = this._getSelectedOwner();
        if (selectedOwner.plans.length === 0) {
            return '';
        }

        var id = subscription.id;
        var planSelectorHtml = '<select class="plans" name="plans[' + id + ']" id="plans-' + id + '">';
        $H(selectedOwner.plans).each(function (plan) {
            plan = plan.value;
            if ('undefined' !== typeof plan.id) {
                planSelectorHtml += '<option value="' + plan.id + '">' + plan.name.escapeHTML() + '</option>';
            }
        });
        planSelectorHtml += '</select>';

        return planSelectorHtml;
    },

    _getIpAddressSelectorHtml: function (subscription, type) {
        var selectedOwner = this._getSelectedOwner();
        if ('undefined' === typeof selectedOwner[type]) {
            return '';
        }

        var id = subscription.id;
        var ipAddressSelectorHtml = '<select class="' + type + '" name="' + type + '[' + id + ']" id="' + type + '-' + id + '">';
        $H(selectedOwner[type]).each(function (ipAddress) {
            ipAddress = ipAddress.value;
            if ('undefined' !== typeof ipAddress.id) {
                var selected = (ipAddress.id == subscription[type].id) ? 'selected' : '';
                ipAddressSelectorHtml += '<option value="' + ipAddress.id + '" ' + selected + '>' + ipAddress.description + '</option>';
            }
        });
        ipAddressSelectorHtml += '</select>';
        if (subscription[type].isDedicated) {
            ipAddressSelectorHtml += '<div class="hint-sub hint-attention">' + this.lmsg('ipAddressHint') + '</div>';
        }

        return ipAddressSelectorHtml;
    },

    _getSelectedOwner: function () {
        return this._owners.get($('owners').value);
    },

    _isValid: function () {
        this._valid = true;
        if (0 == this._getSelectedOwner().id) {
            this._addErrorMessage(this.lmsg('ownerNotSelected'));
            return false;
        }
        $H(this._response.customers).each(function (customer) {
            customer = customer.value;
            if (customer.subscriptions.length !== 0) {
                $H(customer.subscriptions).each(function (subscription) {
                    if (this._valid) {
                        var id = subscription.value.id;
                        var ipV4 = ($('ipV4-' + id)) ? $('ipV4-' + id).value : 0;
                        var ipV6 = ($('ipV6-' + id)) ? $('ipV6-' + id).value : 0;
                        if (!ipV4 && !ipV6) {
                            this._addErrorMessage(this.lmsg('ipAddressNotSelected'));
                            this._valid = false;
                        }
                    }
                }, this);
            }
        }, this);

        return this._valid;
    },

    _getAdditionalParams: function (params) {
        params.set('owner-id', $('owners').value);
        $$('.plans').each(function (row) {
            params.set('plans[' + row.id.replace('plans-', '') + ']', row.value);
        });
        $$('.ipV4').each(function (row) {
            params.set('ipV4[' + row.id.replace('ipV4-', '') + ']', row.value);
        });
        $$('.ipV6').each(function (row) {
            params.set('ipV6[' + row.id.replace('ipV6-', '') + ']', row.value);
        });
        return params;
    },
});
