// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(function () {
    return Class.create(Jsw.PopupForm, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._handlerUrl = this._getConfigParam('handlerUrl', '');
            this._userLogin = this._getConfigParam('userLogin', '');
        },

        setUserLogin: function (userLogin) {
            this._userLogin = userLogin;
        },

        render: function ($super) {
            $super();

            this.setBoxType('form-box');
            this.setTitle(this.lmsg('changeCredentialsTitle'));

            var userNameField = '';
            if ('admin' === this._userLogin) {
                userNameField = '<div class="text-value">' + this._userLogin.escapeHTML() + '</div>';
            } else {
                userNameField = '<input name="userName" id="' + this._id + '-login" value="" class="input-text" type="text">' +
                    '<span class="field-errors" style="display: none;"></span>';
            }

            $(this._contentAreaId).insert(
                '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                    '<div>' +
                        '<div class="form-row">' +
                            '<div class="field-name">' +
                                '<label for="' + this._id + '-login">' +
                                    this.lmsg('userLogin') +
                                '</label>' +
                            '</div>' +
                            '<div class="field-value">' + userNameField + '</div>' +
                        '</div>' +
                        '<div class="form-row">' +
                            '<div class="field-name">' +
                                '<label for="' + this._id + '-password">' +
                                    this.lmsg('userPassword') +
                                '</label>' +
                            '</div>' +
                            '<div class="field-value">' +
                                '<input name="userPassword" id="' + this._id + '-password" ' +
                                    'value="" class="input-text" type="password">' +
                                '<span class="field-errors" style="display: none;"></span>' +
                            '</div>' +
                        '</div>' +
                        '<div class="form-row">' +
                            '<div class="field-name">' +
                                '<label for="' + this._id + '-password-confirmation">' +
                                    this.lmsg('userPasswordConfirmation') +
                                '</label>' +
                            '</div>' +
                            '<div class="field-value">' +
                                '<input name="userPasswordConfirmation" id="' + this._id + '-password-confirmation" ' +
                                    'value="" class="input-text" type="password">' +
                                '<span class="field-errors" style="display: none;"></span>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</form>'
            );

            this.addOkButton();
            this.addCancelButton();

            var form = $(this._id + '-form');
            form._formSubmit = form.submit;
            form.submit = this._onSubmit.bind(this);
            form.observe('submit', this._onSubmitEvent.bind(this));

            this.reset();
        },

        reset: function () {
            if ($(this._id + '-login')) {
                $(this._id + '-login').value = this._userLogin.escapeHTML();
            }
            $(this._id + '-password').value = '';
            $(this._id + '-password-confirmation').value = '';
        },

        addOkButton: function () {
            this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true);
        },

        addCancelButton: function () {
            this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false);
        },

        _onOkClick: function () {
            this._onSubmit();
        },

        _onSubmitEvent: function (event) {
            this._onSubmit();
            Event.stop(event);
            return false;
        },

        _onSubmit: function () {
            var userLogin = $(this._id + '-login') ? $(this._id + '-login').value : '';
            var userPassword = $(this._id + '-password').value;
            var userPasswordConfirmation = $(this._id + '-password-confirmation').value;

            new Ajax.Request(
                this._handlerUrl,
                {
                    method: 'post',
                    parameters: {
                        userLogin: userLogin,
                        userPassword: userPassword,
                        userPasswordConfirmation: userPasswordConfirmation,
                    },
                    onCreate: this._onOkHandlerCreate.bind(this),
                    onSuccess: (function (transport) {
                        this._onOkHandlerSuccess(transport);
                    }).bind(this),
                    onException: this._onOkHandlerException.bind(this),
                    onComplete: this._onOkHandlerComplete.bind(this),
                }
            );
        },

        _onOkHandlerCreate: function (transport) {
            this.disable();
        },

        _onOkHandlerComplete: function (transport) {
            var response = transport.responseText.evalJSON();
            if ('SUCCESS' === response.status) {
                var redirect = function () {
                    this.hide();
                    Jsw.redirect('/file/index');
                };
                redirect.bind(this).delay(5);
            } else {
                this.enable();
            }
        },

        _onOkHandlerSuccess: function (transport) {
            this._clearMessages();

            try {
                var response = transport.responseText.evalJSON();
                if ('SUCCESS' !== response.status) {
                    this._addErrorMessage(response.message);
                }
            } catch (e) {
                this._addErrorMessage(
                    'Internal error: ' + transport.responseText
                );
            }
        },

        _onOkHandlerException: function (transport, exception) {
            this._addErrorMessage('Internal error: ' + exception);
        },

        _onCancelClick: function () {
            this.hide();
        },

        disable: function () {
            var context = this;
            $(this._rightActionButtonsAreaId).select('.btn.action').each(
                function (actionButtonWrapper) {
                    var sendButton = actionButtonWrapper.down('button');
                    context._okButtonTitle = sendButton.innerHTML;
                    sendButton.update(
                        '<span class="wait">' + context.lmsg('waitButtonTitle') + '</span>'
                    );
                }
            );

            $(this._rightActionButtonsAreaId).select('.btn').each(
                function (actionButtonWrapper) {
                    actionButtonWrapper.down('button').disabled = true;
                    actionButtonWrapper.addClassName('disabled');
                }
            );
        },

        enable: function () {
            var context = this;
            $(this._rightActionButtonsAreaId).select('.btn.action').each(
                function (actionButtonWrapper) {
                    var sendButton = actionButtonWrapper.down('button');
                    sendButton.update(context._okButtonTitle);
                }
            );
            $(this._rightActionButtonsAreaId).select('.btn').each(
                function (actionButtonWrapper) {
                    actionButtonWrapper.down('button').disabled = false;
                    actionButtonWrapper.removeClassName('disabled');
                }
            );
        },
    });
});
