// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './select-destination',
], function (SelectDestination) {
    return Class.create(SelectDestination, {

        _onSelectFolder: function ($super) {
            $super();

            this._hideWarning();

            var listControl = Jsw.getComponent('files-list');
            var storageType = this._explorer.getType();
            var folder = this._explorer.getFolder();

            if (('' == folder) && ('private' === storageType) && listControl.isBuiltIn()) {
                this.disable();
                this._showWarning(this.lmsg('forbidToPrivateRoot'));
                return;
            }

            var destinationPath = storageType + '/' + folder;
            var sourcePath = listControl.getStorageType() + '/' + listControl.getCurrentFolder();

            if (sourcePath === destinationPath) {
                this.disable();
                this._showWarning(this.lmsg('forbidToSameFolder'));
                return;
            }

            this._selected.each((function (listItem) {
                if ('folder' !== listItem.type) {
                    return;
                }
                var itemPath = sourcePath + '/' + listItem.name;
                if (destinationPath.startsWith(itemPath)) {
                    this.disable();
                    this._showWarning(this.lmsg('forbidToChild', { folder: listItem.name.escapeHTML() }));
                }
            }).bind(this));
        },

        _onDeselectFolder: function ($super) {
            $super();

            this._hideWarning();
        },
    });
});
