// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './list-popup',
], function (ListPopup) {
    return Class.create(ListPopup, {

        _initConfiguration: function ($super, config) {
            $super(config);

            // :INFO: destination param should be HTML escaped
            this._destination = this._getConfigParam('destination', '');
            this._items = this._getConfigParam('items', []);

            this._onOverwriteHandler = this._getConfigParam('overwriteHandler', function () {});
            this._onSkipHandler = this._getConfigParam('skipHandler', function () {});
            this._onCancelHandler = this._getConfigParam('cancelHandler', function () {});
        },

        render: function ($super) {
            $super();

            this.setBoxType('list-box');
            this.setTitleType('pp-alert');
            this.setTitle(this.lmsg('title'));
            this.setHint(this.lmsg('description'));
            this.setHint1(this.lmsg('hint', { folder: '<b>' + this._destination + '</b>' }));

            this._items.each(function (item) {
                this._addItem(item);
            }, this);

            this.addRightButton(
                this.lmsg('buttonOverwrite'),
                this._onOverwriteHandler,
                false,
                true
            );
            this.addRightButton(
                this.lmsg('buttonSkip'),
                this._onSkipHandler,
                true,
                true
            );
            this.addRightButton(
                this.lmsg('buttonCancel'),
                this._onCancelHandler,
                false,
                false
            );

            this.resizeList();
        },

        _renderContentArea: function ($super) {
            $super();
            $(this._itemsAreaId).insert(
                '<tr>' +
                    '<th class="first">' + this.lmsg('columnName') + '</th>' +
                    '<th>' + this.lmsg('columnNew') + '</th>' +
                    '<th class="last">' + this.lmsg('columnExisting') + '</th>' +
                '</tr>'
            );
        },

        _addItem: function (item) {
            this._itemClass = ('odd' === this._itemClass) ? 'even' : 'odd';

            var itemHtml = '' +
                '<tr class="' + this._itemClass + '">' +
                    '<td class="first">' +
                        '<span class="type-' + item.existItem.type.escapeHTML() + '">' +
                            item.existItem.name.escapeHTML() +
                        '</span>' +
                    '</td>' +
                    '<td class="number">' +
                        '<div>' +
                            item.newItem.formatedSize.escapeHTML() +
                        '</div>' +
                        '<div>' +
                            this.lmsg('modifiedAt', { modificationDate: item.newItem.modificationDate.escapeHTML() }) +
                        '</div>' +
                    '</td>' +
                    '<td class="number last">' +
                        '<div>' +
                            item.existItem.formatedSize.escapeHTML() +
                        '</div>' +
                        '<div>' +
                            this.lmsg('modifiedAt', { modificationDate: item.existItem.modificationDate.escapeHTML() }) +
                        '</div>' +
                    '</td>' +
                '</tr>';

            $(this._itemsAreaId).insert({ bottom: itemHtml });
        },
    });
});
