// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(['./view'], function (view) {
    return Class.create(Jsw.PopupForm, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._handlerUrl = this._getConfigParam('handlerUrl', '');
        },

        render: function ($super) {
            $super();

            this.setBoxType('form-box');
            this.setTitle(this.lmsg('title'));

            $(this._contentAreaId).insert(
                '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                    '<div>' +
                        '<div class="form-row">' +
                            '<div class="field-name">' +
                                '<label for="' + this._id + '-folder-name">' +
                                    this.lmsg('folderName') +
                                '</label>' +
                            '</div>' +
                            '<div class="field-value">' +
                                '<input name="folderName" id="' + this._id + '-folder-name" ' +
                                    'value="" class="f-middle-size input-text" type="text">' +
                                '<span class="field-errors" style="display: none;"></span>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</form>'
            );

            this.addOkButton();
            this.addCancelButton();

            var form = $(this._id + '-form');
            form._formSubmit = form.submit;
            form.submit = this._onSubmit.bind(this);
            form.observe('submit', this._onSubmitEvent.bind(this));

            this.reset();
        },

        reset: function () {
            this.setHint(this.lmsg('hint', { folder: '<b>' + this._getCurrentFolder().escapeHTML() + '</b>' }));

            view.clearStatusMessages();
            this._clearMessages();

            $(this._id + '-folder-name').value = '';

            try {
                $(this._id + '-folder-name').focus();
            } catch (e) {}
        },

        _getCurrentFolder: function () {
            var listControl = Jsw.getComponent('files-list');
            return listControl.getEscapedCurrentPathTitle();
        },

        addOkButton: function () {
            this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true);
        },

        addCancelButton: function () {
            this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false);
        },

        _onOkClick: function () {
            this._onSubmit();
        },

        _onSubmitEvent: function (event) {
            this._onSubmit();
            Event.stop(event);
            return false;
        },

        _onSubmit: function () {
            var folderName = $(this._id + '-folder-name').value;
            new Ajax.Request(
                this._handlerUrl,
                {
                    method: 'post',
                    parameters: {
                        type: Smb.FileSharing.ListInstance.getStorageType(),
                        folder: Smb.FileSharing.ListInstance.getCurrentFolder(),
                        newFolderName: folderName,
                    },
                    onCreate: this._onOkHandlerCreate.bind(this),
                    onSuccess: (function (transport) {
                        this._onOkHandlerSuccess(transport, folderName);
                    }).bind(this),
                    onException: this._onOkHandlerException.bind(this),
                    onComplete: this._onOkHandlerComplete.bind(this),
                }
            );
        },

        _onOkHandlerCreate: function (transport) {
            this.disable();
        },

        _onOkHandlerComplete: function (transport) {
            this.enable();
        },

        _onOkHandlerSuccess: function (transport, folderName) {
            this._clearMessages();
            try {
                var response = transport.responseText.evalJSON();
                if ('SUCCESS' === response.status) {
                    Smb.FileSharing.ListInstance.reload();
                    var message = response.message
                        .replace(
                            '%%folder%%',
                            '<a href="#" id="' + this._id + '-success-message-link">' +
                                folderName.escapeHTML() +
                            '</a>'
                        );
                    Jsw.addStatusMessage('info', message);
                    $(this._id + '-success-message-link').observe(
                        'click',
                        function (event) {
                            Event.stop(event);
                            Smb.FileSharing.ListInstance.changeFolder(
                                Smb.FileSharing.ListInstance.makeRelativePath(folderName)
                            );
                        }
                    );
                    this.hide();
                } else {
                    this._addErrorMessage(response.message);
                }
            } catch (e) {
                this._addErrorMessage(
                    'Internal error: ' + transport.responseText
                );
            }
        },

        _onOkHandlerException: function (transport, exception) {
            this._addErrorMessage('Internal error: ' + exception);
        },

        _onCancelClick: function () {
            this.hide();
        },

        disable: function () {
            var context = this;
            $(this._rightActionButtonsAreaId).select('.btn.action').each(
                function (actionButtonWrapper) {
                    var sendButton = actionButtonWrapper.down('button');
                    context._okButtonTitle = sendButton.innerHTML;
                    sendButton.update(
                        '<span class="wait">' + context.lmsg('waitButtonTitle') + '</span>'
                    );
                }
            );

            $(this._rightActionButtonsAreaId).select('.btn').each(
                function (actionButtonWrapper) {
                    actionButtonWrapper.down('button').disabled = true;
                    actionButtonWrapper.addClassName('disabled');
                }
            );
        },

        enable: function () {
            var context = this;
            $(this._rightActionButtonsAreaId).select('.btn.action').each(
                function (actionButtonWrapper) {
                    var sendButton = actionButtonWrapper.down('button');
                    sendButton.update(context._okButtonTitle);
                }
            );
            $(this._rightActionButtonsAreaId).select('.btn').each(
                function (actionButtonWrapper) {
                    actionButtonWrapper.down('button').disabled = false;
                    actionButtonWrapper.removeClassName('disabled');
                }
            );
        },
    });
});
