// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './list-popup',
], function (ListPopup) {
    return Class.create(ListPopup, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._cls = this._getConfigParam('cls', 'popup-panel');
            this._titleCls = this._getConfigParam('titleCls', 'pp-error');

            this._errorMessage = this._getConfigParam('errorMessage', '');
            this._message = this._getConfigParam('message', null);
            this._items = this._getConfigParam('items', $A());

            this._title = this._getConfigParam('title', this.lmsg('title'));
        },

        render: function ($super) {
            $super();

            this.setBoxType('list-box');
            this.setTitle(this._title);

            this._addErrorMessage(this._errorMessage);
            if (null === this._message) {
                $(this._popupContentAreaId).select('.msg-error').forEach(function (messageElement) {
                    messageElement.addClassName('list-error');
                });
            } else {
                this.setHint(this._message);
            }

            this._items.each(function (item) {
                this._addItem(item);
            }, this);

            this.addRightButton(
                this.lmsg('buttonClose'),
                this._onClose.bind(this),
                true,
                true
            );

            this.resizeList();
        },

        _onClose: function () {
            this.hide();
        },

        _addItem: function (item) {
            this._itemClass = ('even' === this._itemClass) ? 'odd' : 'even';

            var statusClass = (item.status)
                ? ('ERROR' === item.status ? 'failed' : 'ok')
                : 'not-started';

            var statusHml = '' +
                '<span class="hint-' + statusClass + '">' +
                    this.lmsg(item.status ? ('ERROR' === item.status ? 'statusError' : 'statusSuccess') : 'statusNotStarted') +
                '</span>';

            var itemHtml = '' +
                '<tr class="' + this._itemClass + '">' +
                    '<td class="first">' +
                        '<span class="type-' + item.type.escapeHTML() + '">' +
                            item.name.escapeHTML() +
                        '</span>' +
                    '</td>' +
                    '<td class="last">' +
                        statusHml +
                    '</td>' +
                '</tr>';

            $(this._itemsAreaId).insert({ bottom: itemHtml });
        },
    });
});
