// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './main',
    './tree-box',
    './types-list',
], function (fileSharing, TreeBox, TypesList) {
    return Class.create(Jsw.Component, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._typesAreaId = this._id + '-types-area';
            this._foldersAreaId = this._id + '-folders-area';

            this._cls = this._getConfigParam('cls', 'explorer-block');
            this._type = this._getConfigParam('type', null);
            this._storageTabs = this._getConfigParam('storageTabs', []);
            this._storageTypes = $H(this._getConfigParam('storageTypes', {}));
            this._folder = this._getConfigParam('folder');
            this._onSelectFolder = this._getConfigParam('onSelectFolder', null);
            this._onDeselectFolder = this._getConfigParam('onDeselectFolder', null);

            this._typeControl = null;
            this._typeTrees = $H();
            this.reset();
        },

        reset: function () {
            if (this._typeControl) {
                this._typeControl.reset();
            }

            this._typeTrees.each(
                function (pair) {
                    pair.value.hide();
                }
            );

            if ($(this._foldersAreaId)) {
                $(this._foldersAreaId).update('<div class="tree-box"></div>');
            }
            this._typeTrees = $H();

            this._onDeselectFolderWrapper();
        },

        _initComponentElement: function ($super) {
            $super();

            this._componentElement.update(
                '<div id="' + this._typesAreaId + '" class="folders-list"></div>' +
                '<div id="' + this._foldersAreaId + '"><div class="tree-box"></div></div>'
            );
        },

        render: function ($super) {
            $super();

            this._initTypeControl();
        },

        _initTypeControl: function () {
            this._typeControl = new TypesList({
                renderTo: this._typesAreaId,
                items: this._storageTabs,
                value: this._type,
                onClick: this._select.bind(this),
            });

            if (null != this._folder) {
                this._select(this.getType(), this._folder);
            }
        },

        _select: function (type, folder) {
            this._typeTrees.each(
                function (pair) {
                    pair.value.hide();
                }
            );

            if (1 > this._typeTrees.keys().length) {
                $(this._foldersAreaId).update('');
            }

            var tree = this._typeTrees.get(type);
            if (!tree) {
                this._onDeselectFolderWrapper();
                tree = this._makeTree(type, folder);
                this._typeTrees.set(type, tree);
            } else {
                tree.show();
                if (Object.isFunction(this._onSelectFolder)) {
                    this._onSelectFolder();
                }
            }
        },

        _onDeselectFolderWrapper: function () {
            if (Object.isFunction(this._onDeselectFolder)) {
                this._onDeselectFolder();
            }
        },

        _makeTree: function (type, folder) {
            return new TreeBox({
                renderTo: this._foldersAreaId,
                fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
                itemsSource: Jsw.prepareUrl('/file/tree-data/type/' + encodeURIComponent(type)),
                locale: this.getLocale(),
                onLoad: this._onSelectFolder,
                onSelectFolder: this._onSelectFolder,
                value: folder,
            });
        },

        getType: function () {
            return this._typeControl.getValue();
        },

        setType: function (type) {
            this._typeControl.setValue(type);
        },

        getTypeTitle: function () {
            var listControl = Jsw.getComponent('files-list');
            return listControl.getStorageTitle(this.getType());
        },

        getEscapedFolderTitle: function () {
            var tree = this._typeTrees.get(this.getType());
            if (!tree) {
                return null;
            }

            var value = tree.getValue();
            if ('' == value) {
                return this.getTypeTitle();
            }

            return $A(value.split('/')).last().escapeHTML();
        },

        getEscapedPathTitle: function () {
            var tree = this._typeTrees.get(this.getType());
            if (!tree) {
                return null;
            }

            var value = tree.getValue();
            if ('' == value) {
                return this.getTypeTitle();
            }

            return this.getTypeTitle() + '/' + value.escapeHTML();
        },

        getFileUrl: function (file) {
            return fileSharing.getFileUrl(
                this._storageTypes.get(this.getType()).url,
                this.getFolder(),
                file
            );
        },

        makeRelativePath: function (file) {
            var currentFolder = this.getFolder();
            return ('' == currentFolder)
                ? file
                : fileSharing.trimLastSlash(currentFolder) + '/' + file;
        },

        getFolder: function () {
            var tree = this._typeTrees.get(this.getType());
            if (!tree) {
                return null;
            }

            return tree.getValue();
        },
    });
});
