// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './popup-area',
    './change-credentials',
], function (PopupArea, ChangeCredentials) {
    return Class.create(PopupArea, {
        _initConfiguration: function ($super, config) {
            $super(config);
            this._webDavUrl = this._getConfigParam('webDavUrl', '');
            this._httpUrl = this._getConfigParam('httpUrl', '');
            this._webDavHowToUrl = Jsw.prepareUrl(this._getConfigParam('webDavHowToLink', ''));
            this._userLogin = this._getConfigParam('userLogin', '');
            this._userPassword = this._getConfigParam('userPassword', '');
        },

        _getTitle: function () {
            return this.lmsg('linkToFolderAreaTitle');
        },

        _initComponentElement: function ($super) {
            $super();
            this._componentElement.addClassName('left');
            this._componentElement.down('.heading-area').insert({ top: '<span class="arrow"></span>' });
        },

        _getContent: function () {
            var content = '<p>' + this.lmsg('linkToWebDavFolderDescription') +
                '<br/>' +
                '<a href="' + this._webDavHowToUrl + '" target="_blank">' +
                    this.lmsg('webDavHowToLink') +
                '</a>' +
            '</p>' +
            '<p><input class="input-text" value="' + this._webDavUrl + '" readonly="readonly" type="text"></p>';

            if (this._httpUrl) {
                content += '<hr>' +
                '<p>' + this.lmsg('linkToHttpFolderDescription') + '</p>' +
                '<p><input class="input-text" value="' + this._httpUrl + '" readonly="readonly" type="text"></p>';
            }

            content += '<table cellspacing="0" class="access-info">'
                + '<tbody><tr><th>' + this.lmsg('userLogin') + ':</th><td>' + this._userLogin.escapeHTML() + '</td></tr>';

            if (this._userPassword) {
                content += '<tr><th>' + this.lmsg('userPassword') + ':</th><td>' + this._userPassword.escapeHTML() + '</td></tr>';
            }

            content += '</tbody></table>';

            content += '<div class="popup-action">'
                + '<a id="' + this._id + '-changeCredentials" href="#">'
                + this.lmsg('changeCredentialsButton')
                + '</a></div>';

            return content;
        },

        _addEvents: function ($super) {
            $super();
            Event.observe(this._id + '-changeCredentials', 'click', this._onChangeCredentialsClick.bind(this));
        },

        _onChangeCredentialsClick: function (event) {
            this.toggle(event);

            var component = Jsw.getComponent('files-list-change-credentials');
            if (component) {
                component.reset();
                component.setUserLogin(this._userLogin);
                component.show();
            } else {
                component = new ChangeCredentials({
                    id: 'files-list-change-credentials',
                    renderTo: 'files-list-modal-dialog-box-change-credentials',
                    cls: 'popup-panel',
                    handlerUrl: Jsw.prepareUrl('/file/change-credentials/'),
                    locale: this.getLocale(),
                    userLogin: this._userLogin,
                    titleCls: 'pp-access',
                });
            }
        },

        render: function ($super) {
            $super();
            this._tooltip = Jsw.Tooltip.init(this._toggleElementId, { text: this.lmsg('linkToFolderHint') });
        },
    });
});
