// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './main',
    './view',
    './list-pathbar',
], function (fileSharing, view, ListPathbar) {
    return Class.create(Jsw.List, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._storageType = this._getConfigParam('storageType', '');
            this._storageUrl = this._getConfigParam('storageUrl', '');
            var storageTabs = this._getConfigParam('storageTabs', {});
            this._storageTabs = Object.keys(storageTabs).map(function (id) {
                storageTabs[id].id = id;
                return storageTabs[id];
            });
            this._storageTypes = $H(this._getConfigParam('storageTypes'));
            this._users = $H(this._getConfigParam('users'));
            this._userContactName = this._getConfigParam('userContactName');
            this._userLogin = this._getConfigParam('userLogin');
            this._isBuiltIn = this._getConfigParam('isBuiltIn');
            this._loadingTitle = this._getConfigParam('loadingTitle', '');
            this._currentFolder = this._getConfigParam('currentFolder', '');
            this._previousFolder = this._currentFolder;
            this._previousRequestParams = '';
            this._onReloadFolderComplete = null;
            this._footer = this._getConfigParam('footer', null);
            this._clearStatusOnReload = false;

            Event.observe(
                window,
                'resize',
                (function () {
                    this.resizeList();
                }).bind(this),
                false
            );
        },

        getData: function () {
            return this._data;
        },

        getUsers: function () {
            return this._users;
        },

        getUserContactName: function () {
            return this._userContactName;
        },

        containerView: function ($super) {
            var view = $super();

            var children = view.children.splice(1);
            view.children.push(Jsw.createElement('.objects-content',
                Jsw.createElement('#' + this._id + '-objects-content-area.objects-content-area',
                    Jsw.createElement('.list-box',
                        Jsw.createElement('.box-area',
                            Jsw.createElement('.content',
                                Jsw.createElement('.content-area', children)
                            )
                        )
                    )
                ),
                this._footer ? this._getFooterHtml() : ''
            ));

            return view;
        },

        _getFooterHtml: function () {
            return '' +
                '<div id="' + this._id + '-objects-content-footer" class="objects-details-footer">' +
                    '<table cellspacing="0" class="footer-container"><tbody><tr>' +
                        '<td>' +
                            '<span class="' + this._footer.cls + '">' +
                                this._footer.description +
                            '</span>' +
                        '</td>' +
                        '<td id="' + this._id + '-footer-actions-area" class="footer-action">' +
                        '</td>' +
                    '</tr></tbody></table>' +
                '</div>';
        },

        makeRelativePath: function (item) {
            return ('' == this._currentFolder)
                ? item
                : fileSharing.trimLastSlash(this._currentFolder) + '/' + item;
        },

        changeFolder: function (folder, onComplete) {
            this._previousFolder = this._currentFolder;
            this._currentFolder = folder;
            this._onReloadFolderComplete = function () {
                view.clearStatusMessages();
                if (Object.isFunction(onComplete)) {
                    onComplete();
                }
            };

            this.reload(this._previousRequestParams);
        },

        getCurrentFolder: function () {
            return this._currentFolder;
        },

        getFileUrl: function (file) {
            var storageUrl = (this._isBuiltIn || ('private' !== this._storageType))
                ? this._storageUrl
                : (this._storageUrl + this._userLogin + '/');

            return fileSharing.getFileUrl(storageUrl, this._currentFolder, file);
        },

        getStorageType: function () {
            return this._storageType;
        },

        getStorageTitle: function (storageType) {
            var storageInfo = this._storageTypes.get(storageType);
            return storageInfo.title;
        },

        getCurrentStorageTitle: function () {
            return this.getStorageTitle(
                this.getStorageType()
            );
        },

        getEscapedCurrentPathTitle: function () {
            var folder = this.getCurrentFolder();
            if ('' != folder) {
                folder = '/' + folder;
            }
            return this.getCurrentStorageTitle() + folder;
        },

        isBuiltIn: function () {
            return this._isBuiltIn;
        },

        getStorageTabs: function () {
            return this._storageTabs;
        },

        getStorageTypes: function () {
            return this._storageTypes;
        },

        updateFolderLinkAction: function (link, storageType, folder) {
            if (this.getStorageType() === storageType) {
                $(link).observe(
                    'click',
                    (function (event) {
                        Event.stop(event);
                        this.changeFolder(folder);
                    }).bind(this)
                );
            } else {
                var uri = '';

                $A(folder.split('/')).each(
                    function (item) {
                        if ('' == item) {
                            return;
                        }
                        uri += item + '/';
                    }
                );
                if ('' != uri) {
                    uri = '/folder/' + encodeURIComponent(uri);
                }
                $(link).href = Jsw.baseUrl +
                    '/file/index/type/' + encodeURIComponent(storageType) +
                    uri;
            }
        },

        cleanSelection: function () {
            this._componentElement.select('input[name="listCheckbox[]"]').each(function (checkbox) {
                checkbox.up('tr').removeClassName('selected');
                checkbox.checked = false;
            });

            var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();
            globalCheckbox.checked = false;
        },

        disable: function () {
            var element = $(this._id + '-pathbar');
            if (element) {
                element.hide();
            }
            element = $(this._id + '-objects-content-area');
            if (element) {
                element.hide();
            }
            element = $(this._id + '-objects-content-footer');
            if (element) {
                element.hide();
            }
            element = $(this._id + '-loading-container');
            if (element) {
                element.show();
            } else {
                this._componentElement.insert(
                    '<div class="ajax-loading" id="' + this._id + '-loading-container">' +
                        this._loadingTitle +
                    '</div>'
                );
            }
            this._disablerOverlay.show();
            this._disablerOverlay.clonePosition($('files-list-container').down('.actions-box'));
        },

        enable: function () {
            $(this._id + '-loading-container').hide();

            var element = $(this._id + '-pathbar');
            if (element) {
                element.show();
            }

            element = $(this._id + '-objects-content-area');
            if (element) {
                element.show();
            }
            element = $(this._id + '-objects-content-footer');
            if (element) {
                element.show();
            }
            this._disablerOverlay.hide();
        },

        render: function ($super) {
            $super();

            this.reload();

            this._disablerOverlay.style.zIndex = 850;
        },

        _addFooter: function () {
            this._footer.actions.each(function (config) {
                Jsw.render($(this._id + '-footer-actions-area'), Jsw.createComponent(config));
            }, this);
        },

        redraw: function ($super) {
            view.hideWarning();

            if (this._operations && 'private' === this._storageType && this._isBuiltIn) {
                if ('' == this._currentFolder) {
                    this._switchOperations(false);
                } else {
                    this._switchOperations(true);
                }
            }

            $super();

            if ((0 == this._pager.totalItemCount) && (this._operations)) {
                // TODO: check the reason
                // this._addOperations();
            }
            if (this._footer) {
                this._addFooter();
            }

            this.resizeList();
        },

        _switchOperations: function (enabled) {
            this._operations.each(function (config) {
                config.disabled = !enabled;
            });
        },

        reload: function ($super, params) {
            view.collapseExpandedWidgets();
            if (params) {
                this._previousRequestParams = params;
            } else {
                params = '';
            }
            params += '/type/' + encodeURIComponent(this._storageType);
            params += '/folder/' + encodeURIComponent(this._currentFolder);

            return $super(params);
        },

        _onReloadFailure: function ($super, transport) {
            $super(transport);

            this.processReloadError(null);
        },

        _onReloadException: function ($super, transport, exception) {
            $super(transport, exception);

            this.processReloadError(null);
        },

        _onReloadComplete: function ($super) {
            $super();

            if (Object.isFunction(this._onReloadFolderComplete)) {
                this._onReloadFolderComplete();
                this._onReloadFolderComplete = null;
            }
        },

        resizeList: function () {
            var height = $('objects-details').getHeight();

            height -= $('files-list-container').down('.actions-box').getHeight();
            if ($('files-list-pathbar')) {
                height -= $('files-list-pathbar').getHeight();
            }
            // Border size:
            height -= 2;

            var footer = $(this._id + '-objects-content-footer');
            if (footer) {
                height -= footer.getHeight();
            }

            if (height > 0) {
                $(this._id + '-objects-content-area').style.height = height + 'px';
            }
        },

        _showItemsNotSelectedWarning: function () {
            view.showWarning(this.lmsg('itemsNotSelected'));
        },

        _hideItemsNotSelectedWarning: function () {
            view.hideWarning();
        },

        processReloadError: function ($super, data) {
            if (data && data.status && data.message) {
                Jsw.clearStatusMessages();
                switch (data.status) {
                    case 'ERROR':
                        Jsw.addStatusMessage('error', 'Internal error: ' + data.message.escapeHTML(), []);
                        break;
                    case 'WARNING':
                        Jsw.addStatusMessage('warning', data.message.escapeHTML(), []);
                        break;
                }
            }
            this._onReloadFolderComplete = (
                function () {
                    var element = $(this._id + '-pathbar');
                    if (element) {
                        element.hide();
                    }
                }
            ).bind(this);

            $(this._id + '-container').down('.content-area').replace(
                '<div class="content-area"><div id="' + this._id + '-errorMessage" class="msg-box msg-warning">' +
                    '<div><div><div><div><div>' +
                        '<div class="msg-content">' +
                            this.lmsg('loadingFailed') +
                            ' <a onclick="Smb.FileSharing.ListInstance.reload(); return false;" href="#retry">' +
                                this.lmsg('loadingRetry') +
                            '</a>' +
                        '</div>' +
                    '</div></div></div></div></div>' +
                '</div></div>'
            );
        },

        _addResponsiveHtml: function () {
        },
    });
});
