// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './explorer',
    './tree-box',
], function (Explorer, TreeBox) {
    return Class.create(Explorer, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._cls = this._getConfigParam('cls', 'form-row');
            this._title = this._getConfigParam('title', '');
        },

        reset: function ($super) {
            $super();

            if ($(this._foldersAreaId)) {
                this._componentElement.down('.fs-select-block').addClassName('hide');
                this._select(this.getType(), this._folder);
            }
        },

        getEscapedPathTitle: function ($super) {
            var result = $super();

            if (null == result) {
                result = this.getTypeTitle();
            }
            return result;
        },

        _initComponentElement: function ($super) {
            $super();

            this._componentElement.update(
                '<div id="' + this._typesAreaId + '" class="field-name"></div>' +
                '<div class="field-value">' +
                    '<div class="fs-select-block hide">' +
                        '<div class="path-to-folder">' +
                            '<span>' +
                                this.getEscapedPathTitle() +
                            '</span>' +
                        '</div>' +
                        '<div class="select-folder">' +
                            '<span>' +
                                this.lmsg('selectFolder') +
                            '</span>' +
                        '</div>' +
                        '<div id="' + this._foldersAreaId + '">' +
                            '<div class="tree-box"></div>' +
                        '</div>' +
                    '</div>' +
                '</div>'
            );

            this._componentElement.down('.fs-select-block .select-folder span')
                .observe('click', this._onSelectFolderClick.bindAsEventListener(this));
        },

        _onSelectFolderClick: function (event) {
            Event.stop(event);
            this._componentElement.down('.fs-select-block').toggleClassName('hide');
        },

        render: function ($super) {
            $super();

            $(this._typesAreaId).update(this._title);
        },

        _initTypeControl: function () {
            this._select(this.getType(), this._folder);
        },

        _onDeselectFolderWrapper: function () {
            // :INFO: deselect never executes here, folder is always known
        },

        _makeTree: function ($super, type, folder) {
            return new TreeBox({
                renderTo: this._foldersAreaId,
                fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
                itemsSource: Jsw.prepareUrl('/file/tree-data/type/' + encodeURIComponent(type)),
                locale: this.getLocale(),
                onLoad: this._onSelectFolderWrapper.bind(this),
                onSelectFolder: this._onSelectFolderWrapper.bind(this),
                value: folder,
            });
        },

        _onSelectFolderWrapper: function () {
            this._componentElement.down('.fs-select-block .path-to-folder span').update(
                this.getEscapedPathTitle()
            );

            this._onSelectFolder();
        },

        getType: function () {
            return this._type;
        },

        setType: function ($super, type) {
            this._type = type;
            this._select(type);
            this._onSelectFolderWrapper();
        },
    });
});
