// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(function () {
    return Class.create(Jsw.Container, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._value = this._getConfigParam('value', '');
            this._treeNodesAreaId = this._id + '-tree-nodes-area';
            this._fileTypesIconsPathTemplate = this._getConfigParam('fileTypesIconsPathTemplate', null);
            this._onSelectFolder = this._getConfigParam('onSelectFolder', {});
            this._onLoad = this._getConfigParam('onLoad', {});
        },

        _initComponentElement: function ($super) {
            $super();

            this._updateComponentElement(
                '<div class="tree-box">' +
                    '<div class="box-area"><ul id="' + this._treeNodesAreaId + '">' +
                    '</ul></div>' +
                '</div>'
            );
        },

        _addEvents: function ($super) {
            $super();

            this._componentElement.select('.tree-box').each(function (element) {
                element.observe('click', this.setValue.bind(this, ''));
            }, this);
        },

        render: function ($super) {
            $super();
            this._loadItems();
        },

        _loadItems: function () {
            var source = this._getConfigParam('itemsSource');
            if (!source) {
                return;
            }

            new Ajax.Request(source, {
                onCreate: this._renderLoadingItems.bind(this),
                onSuccess: this._onItemsSuccess.bind(this),
                onException: this._onItemsException.bind(this),
            });
        },

        _onItemsSuccess: function (transport) {
            var response = transport.responseText.evalJSON();
            switch (response.status) {
                case 'SUCCESS':
                    this._initItems(response.data);
                    this._renderItems();

                    if (Object.isFunction(this._onLoad)) {
                        this._onLoad();
                    }
                    break;
                case 'ERROR':
                    this._reportError(response.message);
                    break;
                default:
                    this._reportError('Bad folders list response status: ' + response.status);
                    break;
            }
        },

        _onItemsException: function (transport, exception) {
            this._reportError(exception);
        },

        _reportError: function () {
            var retryElement = new Element('a', { href: '#"' }).update(this.lmsg('loadingRetry'));
            $(this._treeNodesAreaId).update(
                '<span class="hint-error">' + this.lmsg('loadingFailed') + '</span> '
            );
            $(this._treeNodesAreaId).insert(retryElement);
            retryElement.observe('click', this._loadItems.bind(this));
        },

        _renderItems: function () {
            if (!this._items.length) {
                this._renderNoItems();
            } else {
                $(this._treeNodesAreaId).update('');
                var parent = $(this._treeNodesAreaId);
                this._items.each(function (item) {
                    this._renderFolder(item, parent);
                }, this);
            }
        },

        _renderLoadingItems: function () {
            $(this._treeNodesAreaId).update(
                '<span class="hint-wait">' + this.lmsg('loadingFolders') + '</span>'
            );
        },

        _renderNoItems: function () {
            $(this._treeNodesAreaId).update(
                '<span class="hint">' + this.lmsg('noFolders') + '</span>'
            );
        },

        _renderItem: function (item) {
            return this._renderFolder(item, $(this._treeNodesAreaId));
        },

        setValue: function (value) {
            var folder = this._getFolderByValue(value, this._items);
            this._changeFolder(folder);
        },

        _getFolderByValue: function (value, folders) {
            if (!folders) {
                return null;
            }
            var ret = null;
            folders.each(function (folder) {
                if (value == folder.value) {
                    return folder;
                }
                ret = this._getFolderByValue(value, folder.sub);
                if (ret) {
                    throw $break;
                }
            }, this);

            return ret;
        },

        _changeFolder: function (folder) {
            this._componentElement.select('DIV.treeBoxNode').each(function (element) {
                element.removeClassName('selected');
            });
            if (folder) {
                folder.element.addClassName('selected');
                this._value = folder.value;
            } else {
                this._value = '';
            }
            if (Object.isFunction(this._onSelectFolder)) {
                this._onSelectFolder();
            }
        },

        _onClick: function (event, folder) {
            Event.stop(event);
            this._changeFolder(folder);
        },

        _renderFolder: function (folder, parent) {
            var listElement = new Element('li');
            var handlerElement = new Element('span').update(this._renderFileTypeIcon('folder') + '<span>' + folder.name.escapeHTML() + '</span>');
            folder.element = new Element('div', { class: 'treeBoxNode' }).update(handlerElement);

            if (('' == this._value) && folder.isDefault) {
                this._value = folder.value;
            }
            if (folder.value === this._value) {
                folder.element.addClassName('selected');
            }

            listElement.insert(folder.element);
            parent.insert(listElement);

            handlerElement.observe('click', this._onClick.bindAsEventListener(this, folder));

            if (null != folder.sub) {
                var subElements = new Element('ul');
                listElement.insert(subElements);
                folder.sub.each(function (item) {
                    this._renderFolder(item, subElements);
                }, this);
            }
        },

        _renderFileTypeIcon: function (type) {
            if (!this._fileTypesIconsPathTemplate) {
                return '';
            }
            return '<img alt="" src="' + this._fileTypesIconsPathTemplate.replace('%%type%%', type) + '"/>';
        },

        getValue: function () {
            return this._value;
        },
    });
});
