// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(function () {
    return Class.create(Jsw.Container, {

        _tag: 'ul',

        _initConfiguration: function ($super, config) {
            $super(config);

            this._value = this._getConfigParam('value', null);
            this._onClickHandler = this._getConfigParam('onClick', null);

            this._treeNodes = $H();
        },

        reset: function () {
            this._treeNodes.each(function (pair) {
                pair.value.removeClassName('active');
            });
            this._value = this._getConfigParam('value', null);
        },

        _renderItems: function () {
            var parent = this._componentElement;
            this._items.forEach(function (item) {
                this._renderItem(item, parent);
            }, this);
        },

        _renderItem: function (item, parent) {
            var listElement = new Element('li', {
                class: 'typeNode' +
                         ' files-' + item.id +
                         (this._value === item.id ? ' active' : ''),
            });
            var nodeElement = new Element('div').update('<span>' + item.title + '</span>');
            parent.insert(listElement.update(nodeElement));
            nodeElement.observe('click', this._onClick.bindAsEventListener(this, item));
            if (item['sub-items']) {
                var subElements = new Element('ul');
                listElement.insert(subElements);
                Object.keys(item['sub-items']).forEach(function (id) {
                    item['sub-items'][id] = id;
                    this._renderSubItem(item, subElements);
                }, this);
            }
            this._treeNodes.set(item.id, listElement);
        },

        _renderSubItem: function (item, parent) {
            var listElement = new Element('li', {
                class: 'typeNode' +
                         (this._value === item.id ? ' active' : ''),
            });
            var nodeElement = new Element('div').update('<span>' + item.title + '</span>');
            parent.insert(listElement.update(nodeElement));
            nodeElement.observe('click', this._onClick.bindAsEventListener(this, item));
            this._treeNodes.set(item.id, listElement);
        },

        _onClick: function (event, item) {
            var treeNode = Event.findElement(event, 'LI.typeNode');
            if (!treeNode) {
                return;
            }
            Event.stop(event);
            this.setValue(item.id);
        },

        getValue: function () {
            return this._value;
        },

        setValue: function (value) {
            this.reset();

            this._treeNodes.get(value).addClassName('active');

            this._value = value;
            if (this._onClickHandler) {
                this._onClickHandler(value);
            }
        },
    });
});
