// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './list-popup',
], function (ListPopup) {
    return Class.create(ListPopup, {
        _initConfiguration: function ($super, config) {
            $super(config);

            this._cls = this._getConfigParam('cls', 'popup-panel');
            this._titleCls = this._getConfigParam('titleCls', 'pp-upload');

            this._items = this._getConfigParam('items', $A());

            this._title = this._getConfigParam('title', this.lmsg('title'));

            this._uploadProgressDialogDescriptionId = this._id + '-description';
            this._uploadProgressDialogItemsAreaId = this._id + '-items';

            this._onCancelClickExternal = this._getConfigParam('onCancelClick', null);
        },

        _renderContentArea: function ($super) {
            $super();
            if (1 === this._items.length) {
                return;
            }
            $(this._contentAreaId).insert({
                top: '<div class="paging">' +
                    '<div class="paging-area">' +
                        '<span class="paging-info" id="' + this._uploadProgressDialogDescriptionId + '"> </span>' +
                     '</div>' +
                '</div>',
            });
        },

        render: function ($super) {
            $super();

            this.setBoxType('list-box');
            this.setTitleType('pp-upload');

            this.setTitle(this.lmsg('titleUploadProgress'));
            this.setHint(this.lmsg('hintUploadProgress'));

            this._items.each(function (item) {
                this._addItem(item);
            }, this);

            if (this._items.length > 1) {
                this._cancelButton = this.addRightButton(
                    this.lmsg('buttonCancelUpload'),
                    this._onCancelClick,
                    true,
                    false
                );
            }

            this.resizeList();
        },

        _onCancelClick: function () {
            this._updateButton(this._cancelButton, {
                title: this.lmsg('buttonCancelUploadClicked'),
                disabled: true,
            });
            if (this._onCancelClickExternal) {
                this._onCancelClickExternal();
            }
        },

        _addItem: function (queueItem) {
            this._itemClass = ('even' === this._itemClass) ? 'odd' : 'even';

            var itemHtml = '<tr class="' + this._uploadProgressItemClass + '" id="' + queueItem.progressId + '">' +
                '<td class="first">' +
                    '<span class="type-' + queueItem.type.escapeHTML() + '">' +
                        queueItem.savedFileName.escapeHTML() +
                    '</span>' +
                '</td>' +
                '<td class="last">' +
                    '<span class="hint-pending">' +
                        this.lmsg('statusPending') +
                    '</span>' +
                '</td>' +
            '</tr>';

            $(this._itemsAreaId).insert({ bottom: itemHtml });
        },

        updateListDescription: function (html) {
            $(this._uploadProgressDialogDescriptionId).update(html);
        },

        updateItem: function (queueItem) {
            var statusIndicator = '';

            switch (queueItem.status) {
                case 'UPLOADING':
                    statusIndicator = '<span class="hint-wait">' + this.lmsg('statusUploading') + '</span>';
                    break;
                case 'ERROR':
                    statusIndicator = '<span class="hint-failed">' + this.lmsg('statusFailed') + '</span>';
                    break;
                case 'SUCCESS':
                    statusIndicator = '<span class="hint-ok">' + this.lmsg('statusCompleted') + '</span>';
                    break;
            }

            $(queueItem.progressId).down('.last').update(statusIndicator);
        },
    });
});
