// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './explorer',
    './upload-progress-dialog',
    './upload-confirmation-dialog',
    './error-dialog',
    './notification',
    './view',
], function (Explorer, UploadProgressDialog, UploadConfirmationDialog, ErrorDialog, Notification, view) {
    return Class.create(Jsw.PopupForm, {

        _initConfiguration: function ($super, config) {
            $super(config);

            this._selectFilesDialogId = this._id + '-select-files-dialog';
            this._selectFilesWarningBlockId = this._selectFilesDialogId + '-warning-block';
            this._explorerBlockId = this._id + '-explorer-block';
            this._selectFilesDialogItemsAreaId = this._selectFilesDialogId + '-items';
            this._specifyEmailDialogId = this._id + '-specify-email-dialog';

            this._notificationDialogLocale = this._getConfigParam('notificationDialogLocale', {});
            this._confirmationDialoglocale = this._getConfigParam('confirmationDialogLocale', {});
            this._errorDialogLocale = this._getConfigParam('errorDialogLocale', {});

            this._explorer = null;
            this._confirmDialog = null;
        },

        _getUploadIFrameElement: function () {
            if (!this._uploadFormElement) {
                this._uploadFormElement = new Element('iframe', {
                    id: this._id + '-upload-iframe',
                    name: '_uploadIFrame',
                    style: 'display: none;',
                    src: this._getConfigParam('uploadHandler', '') + '?init=true',
                });
                $(this._contentAreaId).insert(this._uploadFormElement);
                this._uploadFormElement.observe('load', this._onUploadFrameInitialized.bindAsEventListener(this));
            }

            return this._uploadFormElement;
        },

        _onUploadFrameInitialized: function () {
            if (this._uploadFrameInitialized) {
                return;
            }
            this._uploadFrameInitialized = true;
            this._uploadFormElement.observe('load', this._onNextQueueUploadFinished.bindAsEventListener(this));
            if (this._scheduleStartUpload) {
                this._scheduleStartUpload = null;
                this.startNextQueueUpload();
            }
        },

        _addEvents: function ($super) {
            $super();
            this._cleanupDialog();
            this.showSelectFilesDialog();
        },

        showSelectFilesDialog: function () {
            this.setBoxType('form-box');
            this.setTitle(this.lmsg('titleSelectFiles'));
            this.setHint(this.lmsg('hintSelectFiles'));

            this.addSelectFilesDialogButtons();

            if (!$(this._selectFilesDialogId)) {
                this.renderSelectFilesDialog();
            } else {
                $(this._selectFilesDialogId).show();
            }
        },

        addSelectFilesDialogButtons: function (disabled) {
            var title;
            var updateHandler;
            if (this._sendEmailNotification) {
                title = this.lmsg('buttonNext');
                updateHandler = function () {
                    if (!this._checkUploadQueue()) {
                        return;
                    }
                    this.hide();
                    this.hideSelectFilesDialog();
                    this.showSpecifyEmailDialog();
                };
            } else {
                title = null;
                updateHandler = function () {
                    if (!this._checkUploadQueue()) {
                        return;
                    }
                    this.hideSelectFilesDialog();
                    this.showCheckUploadDialog();
                };
            }
            this._selectUpdateButtonElement = this.addUpdateButton(updateHandler, true, title);
            this._updateButton(this._selectUpdateButtonElement, { disabled: disabled });
            this.addCancelButton();
        },

        _checkUploadQueue: function () {
            this._clearMessages();
            this._prepareUploadQueue();
            if (!this._uploadQueue.length) {
                this._addErrorMessage(this.lmsg('noFilesSelectedForUpload'));
                return false;
            }
            return true;
        },

        renderSelectFilesDialog: function () {
            var sendEmailNotificationOptionId = this._id + '-send-email-notification-option';

            $(this._contentAreaId).insert('<div id="' + this._selectFilesDialogId + '">' +
                    '<div id="' + this._selectFilesDialogItemsAreaId + '"></div>' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<a href="#" class="s-btn sb-item-add"><i><i><i><span>' + this.lmsg('addFileItem') + '</span></i></i></i></a>' +
                        '</div>' +
                    '</div>' +
                    '<hr class="hr"/>' +
                    '<div class="form-row" id="' + this._explorerBlockId + '">' +
                        '<p>' + this.lmsg('uploadTo') + '</p>' +
                    '</div>' +
                    '<div class="form-row" style="display: none;" id="' + this._selectFilesWarningBlockId + '">' +
                        '<div class="single-row">' +
                            '<div class="hint-attention">' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="checkbox" class="checkbox" value="1" id="' + sendEmailNotificationOptionId + '" name="general[account][isActive]"/>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + sendEmailNotificationOptionId + '">' + this.lmsg('sendEmailNotificationOption') + '</label>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>'
            );
            this._explorer = new Explorer({
                id: this._id + 'Explorer',
                renderTo: this._explorerBlockId,
                type: this._getConfigParam('type'),
                storageTabs: Smb.FileSharing.ListInstance.getStorageTabs(),
                storageTypes: Smb.FileSharing.ListInstance.getStorageTypes(),
                fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
                folder: this._getConfigParam('folder'),
                locale: this.getLocale(),
                onSelectFolder: (function () {
                    if ('private' !== this._explorer.getType() || this._explorer.getFolder() || !Smb.FileSharing.ListInstance.isBuiltIn()) {
                        this.enable();
                        $(this._selectFilesWarningBlockId).hide();
                    } else {
                        this.disable();
                        $(this._selectFilesWarningBlockId).down('.hint-attention').update(this.lmsg('hintInvalidDestination'));
                        $(this._selectFilesWarningBlockId).show();
                    }
                }).bind(this),
                onDeselectFolder: this.disable.bind(this),
            });

            this.addSelectFilesItem();

            $(this._selectFilesDialogId).select('.sb-item-add').each(function (element) {
                element.observe('click', this._onAddSelectFilesItemClick.bindAsEventListener(this));
            }, this);
            $(sendEmailNotificationOptionId).observe('click', this._onSendEmailNotificationOptionClick.bindAsEventListener(this));
        },

        enable: function () {
            if (this._selectUpdateButtonElement) {
                this._updateButton(this._selectUpdateButtonElement, { disabled: false });
            }
        },

        disable: function () {
            if (this._selectUpdateButtonElement) {
                this._updateButton(this._selectUpdateButtonElement, { disabled: true });
            }
        },

        _onAddSelectFilesItemClick: function (event) {
            this.addSelectFilesItem();
            Event.stop(event);
        },

        _onSendEmailNotificationOptionClick: function (event) {
            var element = Event.element(event);
            this._sendEmailNotification = element.checked;
            var disabled =
                this._selectUpdateButtonElement &&
                this._selectUpdateButtonElement.down('button') &&
                this._selectUpdateButtonElement.down('button').disabled;

            this.removeRightButtons();

            this.addSelectFilesDialogButtons(disabled);
        },

        addSelectFilesItem: function () {
            var itemsArea = $(this._selectFilesDialogItemsAreaId);
            var itemsCount = itemsArea.select('form').length;

            var itemId = this._selectFilesDialogId + '-item-' + itemsCount;
            var item = '<form id="' + itemId + '" action="' + this._getConfigParam('uploadHandler', '') + '" method="post" enctype="multipart/form-data" target="_uploadIFrame">' +
                    '<input name="type" type="hidden"/>' +
                    '<input name="folder" type="hidden"/>' +
                    '<input name="fileName" type="hidden"/>' +
                    ($('forgery_protection_token') ? '<input name="forgery_protection_token" type="hidden" value="' + $('forgery_protection_token').content + '"/>' : '') +
                    '<div class="form-row">' +
                        '<input name="file" type="file"/>' +
                        (itemsCount ? ' <a href="#" class="s-btn sb-item-remove"><i><i><i><span> ' + this.lmsg('removeFileItem') + '</span></i></i></i></a>' : '') +
                    '</div>' +
                '</form>';

            itemsArea.insert({ bottom: item });

            if (itemsCount) {
                $(itemId).select('.sb-item-remove').each(function (element) {
                    element.observe('click', this._onRemoveSelectFilesItemClick.bindAsEventListener(this));
                }, this);
            }
        },

        _onRemoveSelectFilesItemClick: function (event) {
            var item = Event.element(event).up('form');
            if (item) {
                item.remove();
            }
            Event.stop(event);
        },

        hideSelectFilesDialog: function () {
            this._cleanupDialog();
            if ($(this._selectFilesDialogId)) {
                $(this._selectFilesDialogId).hide();
            }
        },

        showSpecifyEmailDialog: function () {
            if (this._notificationDialog) {
                this._notificationDialog.show();
            } else {
                this._notificationDialog = new Notification({
                    id: this._id + '-notification',
                    renderTo: this.getRenderTarget().id + '-notification',
                    cls: 'popup-panel',
                    locale: this._notificationDialogLocale,
                    titleCls: this._titleCls,
                    title: this._notificationDialogLocale.specifyEmailParameters,
                    onBack: this._onNotificationBackClick.bind(this),
                    onOk: this._onNotificationOkClick.bind(this),
                    onCancel: this._onNotificationCancelClick.bind(this),
                    getFilesHandler: this.getFilesForNotificationDialog.bind(this),
                });
            }

            this._notificationDialog.setTargetType(this._explorer.getType());
            this._notificationDialog.setTargetFolder(this._explorer.getFolder());
            this._notificationDialog.reset();
        },

        getFilesForNotificationDialog: function () {
            var files = $A();
            this._uploadQueue.each(function (queueItem) {
                files.push(queueItem.originalFileName);
            });
            return files;
        },

        _onNotificationBackClick: function () {
            this._notificationDialog.hide();
            this.show();
            this.showSelectFilesDialog();
        },

        _onNotificationOkClick: function () {
            this._notificationDialog.hide();
            this.show();
            this.showCheckUploadDialog();
        },

        _onNotificationCancelClick: function () {
            this._notificationDialog.hide();
            this._onCancelClick();
        },

        hideSpecifyEmailDialog: function () {
            this._cleanupDialog();
            if ($(this._specifyEmailDialogId)) {
                $(this._specifyEmailDialogId).hide();
            }
        },

        _prepareUploadQueue: function () {
            this._uploadQueue = $A();
            var items = $(this._selectFilesDialogItemsAreaId).select('form');
            items.each(function (formElement) {
                var fileName = formElement.down('input[type="file"]').value;
                if (!fileName) {
                    return;
                }

                // IE saves full path to file, but we needed in file name only
                fileName = fileName.replace(/\\/g, '/').replace(/.*\//, '');

                var queueItem = {
                    inputId: formElement.id,
                    progressId: formElement.id + '-progress',
                    formElement: formElement,
                    originalFileName: fileName,
                    savedFileName: fileName,
                    status: 'PENDING',
                    type: 'unknown',
                };
                this._uploadQueue.push(queueItem);
            }, this);
        },

        showCheckUploadDialog: function () {
            this.setBoxType('list-box');
            this.setTitle(this.lmsg('titleCheckUpload'));
            this.setHint('<span class="ajax-loading">' + this.lmsg('hintCheckUpload') + '</span>');

            this._skipSelected = false;
            var checkQueue = $H({
                type: this._explorer.getType(),
                folder: this._explorer.getFolder(),
            });

            this._uploadQueue.each(function (queueItem) {
                checkQueue.set('queue[' + queueItem.inputId + '][id]', queueItem.inputId);
                checkQueue.set('queue[' + queueItem.inputId + '][originalFileName]', queueItem.originalFileName);
            }, this);

            new Ajax.Request(Jsw.baseUrl + '/file/check-upload/', {
                method: 'post',
                parameters: checkQueue,
                onSuccess: this._onSuccessCheckUploads.bind(this),
                onException: (function (transport, exception) {
                    view.clearStatusMessages();
                    Jsw.addStatusMessage(
                        'error',
                        'Internal error: ' + exception + '\n' +
                        transport.responseText
                    );
                    this.hide();
                }).bind(this),
            });
        },

        _onSuccessCheckUploads: function (transport) {
            var errorsFound = false;

            var result = transport.responseText.evalJSON();
            switch (result.status) {
                case 'SUCCESS':
                    break;
                case 'ERROR':
                    this._reportError(result.message);
                    return;
                default:
                    this._reportError('Bad upload check response status: ' + result.status);
                    return;
            }
            $H(result.data).each(function (pair) {
                var id = pair.key;
                var checkItem = pair.value;
                var queueItem = null;
                this._uploadQueue.each(function (item) {
                    if (item.inputId == id) {
                        queueItem = item;
                        throw $break;
                    }
                }, this);

                if (!queueItem) {
                    return;
                }

                if (checkItem.exists) {
                    queueItem.exists = checkItem.exists;
                    errorsFound = true;
                }
                if (checkItem.savedFileName) {
                    queueItem.savedFileName = checkItem.savedFileName;
                }
                if (checkItem.type) {
                    queueItem.type = checkItem.type;
                }
            }, this);

            this.hideCheckUploadDialog();
            if (errorsFound) {
                this.showConfirmReplaceDialog();
            } else {
                this.showUploadProgressDialog();
                this.startNextQueueUpload();
            }
        },

        hideCheckUploadDialog: function () {
            this._cleanupDialog();
        },

        showConfirmReplaceDialog: function () {
            var context = this;

            this.hide();
            var items = $A();
            this._uploadQueue.each(function (queueItem) {
                if (!queueItem.exists) {
                    return;
                }
                items.push(queueItem.exists);
            });

            this._confirmDialog = new UploadConfirmationDialog({
                id: this._id + '-confirmation',
                renderTo: this._id + '-confirmation-dialog-box',
                cls: 'popup-panel',
                locale: this._confirmationDialoglocale,
                destination: context._explorer.getEscapedPathTitle(),
                items: items,
                overwriteHandler: function () {
                    context.hideConfirmReplaceDialog();
                    context.showUploadProgressDialog();
                    context.startNextQueueUpload();
                },
                skipHandler: function () {
                    context._skipSelected = true;
                    context._uploadQueue.each(function (item) {
                        if (item.exists) {
                            context._uploadQueue = context._uploadQueue.without(item);
                        }
                    }, context);
                    context.hideConfirmReplaceDialog();
                    if (context._uploadQueue.length > 0) {
                        context.showUploadProgressDialog();
                        context.startNextQueueUpload();
                    } else {
                        context.reportUploadFinished();
                    }
                },
                cancelHandler: function () {
                    context.close();
                },
            });
        },

        hideConfirmReplaceDialog: function () {
            this._cleanupDialog();
            this._confirmDialog.remove();
            this._confirmDialog = null;
        },

        showUploadProgressDialog: function () {
            this.hide();
            this._uploadProgressDialog = new UploadProgressDialog({
                id: this._id + '-progress',
                renderTo: this._id + '-progress-dialog-box',
                items: this._uploadQueue,
                locale: this.getLocale(),
                onCancelClick: this._onCancelUploadClick.bind(this),
            });
        },

        _onCancelUploadClick: function () {
            this._uploadCanceled = true;
        },

        startNextQueueUpload: function () {
            this._getUploadIFrameElement();
            if (!this._uploadFrameInitialized) {
                this._scheduleStartUpload = true;
                return;
            }

            if (this._uploadCurrent) {
                return;
            }

            if (this._uploadCanceled) {
                this._onAllQueueUploadFinished();
                return;
            }

            var started = false;

            this._uploadQueue.each(function (queueItem) {
                if ('PENDING' !== queueItem.status) {
                    return;
                }
                started = true;
                this.updateUploadProgressItemStatus(queueItem, 'UPLOADING');
                this._uploadCurrent = queueItem;
                queueItem.formElement.select('input[name="type"]').each(function (element) {
                    element.value = this._explorer.getType();
                }, this);
                queueItem.formElement.select('input[name="folder"]').each(function (element) {
                    element.value = this._explorer.getFolder();
                }, this);
                queueItem.formElement.select('input[name="fileName"]').each(function (element) {
                    element.value = queueItem.savedFileName;
                }, this);
                queueItem.formElement.submit();
                throw $break;
            }, this);

            if (!started) {
                this._onAllQueueUploadFinished();
                return;
            }

            if (1 === this._uploadQueue.length) {
                return;
            }

            var status = this.getQueueStatus();

            this._uploadProgressDialog.updateListDescription(
                this.lmsg('descriptionUploadProgress', {
                    total: status.total,
                    pending: status.pending,
                    uploading: status.uploading,
                    completed: status.completed,
                    failed: status.failed,
                    folder: '<b>' + this._explorer.getEscapedPathTitle() + '</b>',
                })
            );
        },

        _onNextQueueUploadFinished: function () {
            if (!this._uploadCurrent) {
                return;
            }

            var queueItem = this._uploadCurrent;
            this._uploadCurrent = null;

            var status = this._getNextQueueUploadResult();
            switch (status.status) {
                case 'SUCCESS':
                    this.updateUploadProgressItemStatus(queueItem, 'SUCCESS');
                    this.startNextQueueUpload();
                    break;
                case 'ERROR':
                default:
                    this.updateUploadProgressItemStatus(queueItem, 'ERROR');
                    this._onQueueUploadFailed(status);
                    break;
            }
        },

        _getNextQueueUploadResult: function () {
            var statusFrame = this._getUploadIFrameElement();
            var statusDocument = statusFrame.contentDocument;
            if (statusDocument === undefined || statusDocument === null) {
                statusDocument = statusFrame.contentWindow.document;
            }
            var status = null;
            try {
                status = statusDocument.body.innerHTML.evalJSON();
            } catch (e) {
                status = {
                    status: 'ERROR',
                    message: 'No upload response',
                    data: null,
                };
            }
            return status;
        },

        _onAllQueueUploadFinished: function () {
            Smb.FileSharing.ListInstance.reload();
            this.reportUploadFinished.bind(this).delay(0.5);
        },

        _onQueueUploadFailed: function (status) {
            var errorItems = $A();
            this._uploadQueue.each(function (item) {
                errorItems.push({
                    name: item.savedFileName,
                    type: item.type,
                    status: item.status,
                });
            });

            this.close();

            new ErrorDialog({
                errorMessage: status.message,
                message: 1 < errorItems.length ? this.lmsg('failed') : '',
                locale: this._errorDialogLocale,
                items: errorItems,
            });
        },

        reportUploadFinished: function () {
            var messageType = null;
            var message = null;

            view.clearStatusMessages();

            this.close();

            if (!this._uploadQueue.length) {
                return;
            }

            var status = this.getQueueStatus();

            var sendLinks = false;

            if (this._uploadCanceled) {
                message = this.lmsg('uploadCanceled');
                messageType = 'warning';
            } else if (1 === this._uploadQueue.length) {
                var queueItem = this._uploadQueue.first();
                if ('SUCCESS' === queueItem.status) {
                    message = this.lmsg('uploadFinishedSingleFile', {
                        file: '<a href="' + this._explorer.getFileUrl(queueItem.savedFileName) + '" target="_blank">' +
                            queueItem.savedFileName.escapeHTML() + '</a>',
                    });
                    messageType = 'info';
                    sendLinks = true;
                } else {
                    message = this.lmsg('uploadFailedSingleFile', {
                        file: '<b>' + queueItem.savedFileName.escapeHTML() + '</b>',
                    });
                    messageType = 'error';
                }
            } else if (status.completed > 0) {
                message = this.lmsg('uploadFinished');
                messageType = 'info';
                sendLinks = true;
            } else {
                message = this.lmsg('uploadFinishedNoFiles');
                messageType = 'warning';
            }

            Jsw.addStatusMessage(messageType, message
                .replace('%%total%%', status.total)
                .replace('%%pending%%', status.pending)
                .replace('%%uploading%%', status.uploading)
                .replace('%%completed%%', status.completed)
                .replace('%%failed%%', status.failed)
                .replace('%%folder%%',
                    '<a href="' +
                        Jsw.prepareUrl(
                            '/file/index/type/' +
                            encodeURIComponent(this._explorer.getType()) +
                            '/?folder=' +
                            encodeURIComponent(this._explorer.getFolder())) +
                        '">' + this._explorer.getEscapedFolderTitle() + '</a>')
            );

            if (this._sendEmailNotification && sendLinks && !this._skipSelected) {
                this._notificationDialog._execute();
            }
        },

        _reportError: function (message) {
            view.clearStatusMessages();
            Jsw.addStatusMessage('error', message);

            this.close();
        },

        updateUploadProgressItemStatus: function (queueItem, status) {
            queueItem.status = status;
            this._uploadProgressDialog.updateItem(queueItem);
        },

        addBackButton: function (handler, isDefault, title) {
            return this.addLeftButton(
                title || this.lmsg('buttonBack'),
                handler,
                isDefault,
                false
            );
        },

        addUpdateButton: function (handler, isDefault, title) {
            var button = this.addRightButton(
                title || this.lmsg('buttonOk'),
                handler,
                isDefault,
                true
            );
            $(this._actionButtonsId).show();
            return button;
        },

        addCancelButton: function (title, isDefault) {
            var button = this.addRightButton(
                title || this.lmsg('buttonCancel'),
                this._onCancelClick,
                isDefault,
                false
            );
            $(this._actionButtonsId).show();
            return button;
        },

        _onCancelClick: function () {
            this.hide();
        },

        _cleanupDialog: function () {
            this.setTitle('');
            this.setHint('');
            this.setHint1('');
            this.removeRightButtons();
            this.removeLeftButtons();

            $(this._actionButtonsId).hide();
        },

        getQueueStatus: function () {
            var status = {
                completed: 0,
                pending: 0,
                failed: 0,
                uploading: 0,
                total: this._uploadQueue.length,
            };

            this._uploadQueue.each(function (queueItem) {
                switch (queueItem.status) {
                    case 'PENDING':
                        status.pending++;
                        break;
                    case 'UPLOADING':
                        status.uploading++;
                        break;
                    case 'ERROR':
                        status.failed++;
                        break;
                    case 'SUCCESS':
                        status.completed++;
                        break;
                }
            }, this);

            return status;
        },

        close: function () {
            if (this._confirmDialog) {
                this._confirmDialog.remove();
                this._confirmDialog = null;
            }

            if (this._uploadProgressDialog) {
                this._uploadProgressDialog.remove();
                this._uploadProgressDialog = null;
            }
            this.hide();
        },
    });
});
