// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(function () {
    return {
        resizeBlocks: function (noTimeOut) {
            if (this.resizeBlocksInProgress) {
                return;
            }
            this.resizeBlocksInProgress = true;
            if (this.resizeBlocksTimeout) {
                window.clearTimeout(this.resizeBlocksTimeout);
                this.resizeBlocksTimeout = null;
            }
            try {
                var minBlockHeight = 200;
                var bodyHeight = (Prototype.Browser.IE)
                    ? $(document.documentElement).getHeight()
                    : $(document.body).getHeight();

                var block = $$('.objects-management-box')[0];
                var blockLayout = block.getLayout();

                var newBlockHeight = bodyHeight -
                    block.cumulativeOffset()[1] -
                    blockLayout.get('border-top') -
                    blockLayout.get('border-bottom') -
                    parseInt($$('.page-content')[0].getStyle('paddingBottom'));

                if ($$('.page-footer-wrapper').first()) {
                    newBlockHeight -= $$('.page-footer-wrapper').first().getHeight();
                }
                if (newBlockHeight < minBlockHeight) {
                    newBlockHeight = minBlockHeight;
                }

                var listTop =
                    $('objects-list-content-area').cumulativeOffset()[1] -
                    $('objects-list').cumulativeOffset()[1];

                var contentHeight = newBlockHeight - listTop;

                Element.setStyle($('objects-details'), {
                    height: newBlockHeight + 'px',
                });
                Element.setStyle($('objects-list-content-area').up(), {
                    height: contentHeight + 'px',
                });

                var listHeight = $('objects-list-content-area').down('div.objects-nav-container').getHeight();
                var helpHeight = $('objects-list-content-area').down('.hint-help').getHeight();
                // borders + padding
                var helpDelta = 12;
                var listContentHeight = listHeight + helpHeight + helpDelta;

                var withScroll = listContentHeight > contentHeight;

                Element.setStyle($('objects-list-content-area'), {
                    height: withScroll
                        ? listHeight + helpDelta + 'px'
                        : contentHeight - helpHeight - helpDelta + 'px',
                    paddingBottom: helpHeight + 'px',
                });

                if (!noTimeOut) {
                    this.resizeBlocksTimeout = this.resizeBlocks.delay(0.5, true);
                }
            } catch (e) {
            }
            this.resizeBlocksInProgress = null;
        },

        showWarning: function (message) {
            $('objects-warnings').show();
            var element = $('objects-warnings').down('.list-actions-msg');
            if (element) {
                element.update('<span>' + message + '</span>').show();
            } else {
                $('objects-warnings')
                    .insert('<span class="list-actions-msg"><span>' + message + '</span></span>');
            }
        },

        hideWarning: function () {
            $('objects-warnings').hide();
            var element = $('objects-warnings').down('.list-actions-msg');
            if (element) {
                element.hide();
            }
        },

        clearStatusMessages: function () {
            this.hideWarning();
            Jsw.clearStatusMessages();
        },

        responsiveItems: function () {
            var items = $$('#objects-list .objects-nav-item');
            items.each(function (el) {
                el.observe('click', function (e) {
                    var isCollapsed = false;
                    items.each(function (el) {
                        if (el.getStyle('display') === 'none') {
                            isCollapsed = true;
                            throw $break;
                        }
                    });
                    if (isCollapsed) {
                        el.up('.objects-nav-container').toggleClassName('responsive-items-visible');
                        e.stop();
                    }
                });
            });
        },

        collapseExpandedWidgets: function (skip) {
            var widgets = {
                'files-list-additional-buttons': 'collapse',
                'files-list-pathbar-component-popup-box': 'hide',
                'files-list-access-info-popup': 'hide',
            };

            Object.keys(widgets).forEach(function (widgetId) {
                var widget = Jsw.getComponent(widgetId);
                if (widget && widget.getId() !== skip) {
                    widget[widgets[widgetId]]();
                }
            });
        },
    };
});
