// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

/* global ActiveXObject */

define(function () {
    var ce = Jsw.createElement;
    var locale = Jsw.Locale.getSection('controllers.help.tutorials');

    var checkFlash = function () {
        var flashPlug = 'application/x-shockwave-flash';
        var flashActiveX = 'ShockwaveFlash.ShockwaveFlash';

        if (window.navigator && window.navigator.plugins && window.navigator.plugins.length) {
            if (window.navigator.mimeTypes && window.navigator.mimeTypes[flashPlug]) {
                return true;
            }
        }

        try {
            new ActiveXObject(flashActiveX);
        } catch (err) {
            return false;
        }
        return true;
    };

    var embedSWF = function (src, width, height) {
        var flashvars = {
            file: src,
            'http.startparam': 'ec_seek',
            provider: 'http',
            skin: '/javascript/externals/swfobject/skin.swf',
            autostart: 'true',
            stretching: 'fill',
            menu: 'false',
        };
        var params = {
            wmode: 'opaque',
            allowscriptaccess: 'always',
            allowfullscreen: 'true',
        };

        return ce('', {
            onrender: function (element) {
                require(['externals/swfobject/swfobject'], function (swfobject) {
                    swfobject.embedSWF('/javascript/externals/swfobject/player.swf', element.identify(),
                        width || '500', height || '320', '9.0.0', '/javascript/externals/swfobject/expressInstall.swf',
                        flashvars, params, {});
                });
            },
        });
    };

    var noFlash = function (urls) {
        return ce('.no-flash-info', ce('no-flash-info-area', [
            ce('img', { src: urls['no-flash-logo'] }),
            ce('br'),
            locale.lmsg('flashPlayerIsNotInstalled'),
            ce('br'),
            ce('a', { href: urls['get-flash-player'], target: '_blank' }, ce('b', locale.lmsg('downloadFlashPlayer'))),
        ]));
    };

    var embedHTML5 = function (src, width, height) {
        var video = document.createElement('video');
        var vtype;
        try {
            if (video.canPlayType && video.canPlayType('video/ogg; codecs="theora, vorbis"').replace(/no/, '')) {
                vtype = 'video/ogg; codecs=\'theora, vorbis\'';
            }
            if (video.canPlayType && video.canPlayType('video/mp4; codecs="avc1.42E01E, mp4a.40.2"').replace(/no/, '')) {
                vtype = 'video/mp4; codecs="avc1.42E01E, mp4a.40.2"';
            }
            if (video.canPlayType && video.canPlayType('video/webm; codecs="vp8, vorbis"').replace(/no/, '')) {
                vtype = 'video/webm; codecs="vp8, vorbis"';
            }
        } catch (e) {
        }

        if (!vtype) {
            return ce('.no-flash-info');
        }
        return ce('video', {
            width: width,
            height: height,
            controls: 'true',
            autoplay: 'true',
            preload: 'none',
            src: src,
        });
    };

    var embedIFrame = function (src) {
        return ce('iframe', { src: src });
    };

    var embedContent = function (urls, tutorial) {
        if (tutorial.iframe) {
            return embedIFrame(tutorial.iframe);
        }
        if (tutorial.webm) {
            return embedHTML5(tutorial.webm, tutorial.width, tutorial.height);
        }
        if (tutorial.flv) {
            if (checkFlash()) {
                return embedSWF(tutorial.flv, tutorial.width, tutorial.height);
            }
            return noFlash(urls);
        }
    };

    var openVideoWindow = function (urls, tutorial) {
        new Jsw.Popup({
            title: tutorial.title,
            popupCls: 'popup-panel popup-panel--video-tutorial',
            closeButtonEnabled: true,
            hideOnEscape: true,
            onShow: function () {
                Jsw.render($(this._popupContentAreaId), embedContent(urls, tutorial));
            },
            onHide: function () {
                $(this._popupContentAreaId).update('');
            },
        });
    };

    return Class.create(Jsw.ListContainer, {
        _initConfiguration: function ($super, config) {
            var tutorials = config.tutorials || [];
            var urls = config.urls || {};
            config = Object.extend({
                cls: 'tools-list tools-list-video-tutorial',
                itemClass: 'tools-item',
                items: tutorials.map(function (tutorial) {
                    return new Jsw.BigButton({
                        title: tutorial.title,
                        comment: tutorial.description,
                        href: '#' + tutorial.anchor,
                        handler: function () {
                            openVideoWindow(urls, tutorial);
                        },
                    });
                }),
            }, config || {});
            $super(config);

            var anchor = document.location.href.split('#')[1];
            if (anchor) {
                var tutorial = tutorials.filter(function (tutorial) {
                    return anchor === tutorial.anchor;
                })[0];

                if (tutorial) {
                    openVideoWindow(urls, tutorial);
                }
            }
        },
    });
});
