// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(['./sources'], function (Sources) {
    return Class.create(Jsw.ScrollableList, {
        onSourceSearchFilterChange: function () {
            var selector = Jsw.getComponent('log-browser-files');
            var list = this;
            this._sourcesSelected = selector.getSelected();
            selector.updateTextValue();
            var realTimeEnabled = list.realTimeEnabled;
            if (realTimeEnabled) {
                this.disableRealTime();
            }
            list._dataUrl = this.getDataUrl(true);
            var onReloadSuccess = list._onReloadSuccess;
            list._onReloadSuccess = function () {
                var isOpened = selector.isOpened();
                list._onReloadSuccess = onReloadSuccess;
                onReloadSuccess.apply(list, arguments);
                selector = Jsw.getComponent('log-browser-files');
                selector.setOpened(isOpened);
                if (realTimeEnabled) {
                    list.toggleControls(true);
                    list.enableRealTime();
                }
            };
            list.reload();
        },

        getDataUrl: function (withSources) {
            var dataUrl = this._defaultDataUrl;
            var value = this._sourcesSelected;
            if (this._logFile && !withSources) {
                dataUrl += '?file=' + encodeURIComponent(this._logFile) + (this._isCustomLog ? '&custom=true' : '');
            } else if (value) {
                dataUrl += '?sources=' + encodeURIComponent(JSON.stringify(value));
            }
            return dataUrl;
        },

        toggleControls: function (realTimeEnabled) {
            if ('undefined' === typeof realTimeEnabled) {
                realTimeEnabled = Jsw.getComponent('log-browser').realTimeEnabled;
            }
            var enabledHidden = realTimeEnabled ? 'hide' : 'show';
            var enabledShown = realTimeEnabled ? 'show' : 'hide';
            Jsw.getComponent('log-browser-start-realtime')[enabledHidden]();
            Jsw.getComponent('log-browser-stop-realtime')[enabledShown]();
            Jsw.getComponent('log-browser-refresh')[enabledHidden]();
            Jsw.getComponent('log-browser-clear')[enabledShown]();

            Jsw.getComponent('log-browser-add-custom')[this._isCustomLog ? 'show' : 'hide']();
            Jsw.getComponent('log-browser-files')[this._isCustomLog ? 'hide' : 'show']();
        },

        realTimeUpdate: function () {
            if (this.realTimeInProgress) {
                return;
            }
            var onReloadCreate = this._onReloadCreate;
            this._onReloadCreate = function () {
                this.realTimeInProgress = true;
            };
            var onReloadComplete = this._onReloadComplete;
            this._onReloadComplete = function () {
                this.realTimeRequest = null;
                this.realTimeInProgress = false;
                this._onReloadCreate = onReloadCreate;
                this._onReloadComplete = onReloadComplete;
            };

            var params = '?showAfter=' + encodeURIComponent(JSON.stringify(this._additionalData.updateAfter))
                + '&short=true';
            this.realTimeRequest = this.reload(params, 'after');
        },

        enableRealTime: function () {
            this.realTimeInterval = setInterval(this.realTimeUpdate.bind(this), 1000);
            this.realTimeEnabled = true;
        },

        disableRealTime: function () {
            clearInterval(this.realTimeInterval);
            this.realTimeEnabled = false;
            this.realTimeInProgress = false;
            if (this.realTimeRequest) {
                this.realTimeRequest.abort();
            }
        },

        makeLinks: function (item, returnOnlyPath) {
            var context = this;
            var text = item.message.escapeHTML();
            var linkRe = /(^|.)(\/|[a-zA-Z]?:?\\)([^\?\s]+)\.([^\s?'/:\.\(\)="]+)($|.)/gm;

            return text.replace(linkRe, function (match, trailingStart, pathStart, path, extension, trailingEnd) {
                var sTrailingStart = trailingStart.strip();
                var sTrailingEnd = trailingEnd.strip();
                if (
                    (sTrailingStart !== '' && !/^['\(\["]$/.test(sTrailingStart))
                    || (sTrailingEnd !== '' && !/^['\(\)\[\]\?=:"]$/.test(sTrailingEnd))
                ) {
                    return match;
                }
                var splittedPath = path.split(/[\\/]/);
                var filename = splittedPath.pop() + '.' + extension;
                var fullPath = pathStart + path + '.' + extension;
                var dirname = '/' + context._domainRootDir;
                var homeDir = context._domainHomeDir;

                path = pathStart + splittedPath.join('/');

                if (fullPath.indexOf(homeDir) > -1) {
                    var tempFullpath = fullPath.replace(homeDir, '');
                    if (/^[\\/]$/.test(tempFullpath.strip()) || tempFullpath.strip() === '') {
                        return match;
                    }

                    var splittedFull = tempFullpath.split(/[\\/]/);
                    splittedFull.pop();
                    dirname = splittedFull.join('/');
                } else {
                    dirname += path;
                }

                if (returnOnlyPath) {
                    return fullPath.strip();
                }

                var linkUrl = context._urls.codeEditor +
                    '?currentDir=' + encodeURIComponent(dirname) +
                    '&file=' + encodeURIComponent(filename) +
                    '&redirect=' + encodeURIComponent(context._urls.returnUrl);
                return trailingStart +
                    '<a href="' + linkUrl + '" title="' + context.lmsg('linkInMessageToViewFile') + '">' + fullPath + '</a>' +
                    trailingEnd;
            });
        },

        getRowClass: function (item) {
            switch (item.type) {
                case 'error':
                    return 'state-error';
                case 'warning':
                    return 'state-warning';
                default:
                    return '';
            }
        },

        _initConfiguration: function ($super, config) {
            config = Object.extend({
                id: 'log-browser',
                cls: 'log-browser',
                pageable: false,
                getRowClass: this.getRowClass,
                onRedraw: this.onRedraw,
            }, config || {});
            $super(config);

            this._logFile = this._getConfigParam('logFile', null);
            this._isCustomLog = this._getConfigParam('isCustomLog', false);
            this._sources = this._getConfigParam('sources', {});
            this._sourcesSelected = this._getConfigParam('sourcesSelected', []);
            this._domainRootDir = this._getConfigParam('domainRootDir', '');
            this._domainHomeDir = this._getConfigParam('domainHomeDir', '');
            this._webspaceId = this._getConfigParam('webspaceId', '');
            this._urls = this._getConfigParam('urls', {});

            this._defaultDataUrl = this._dataUrl;
            this._dataUrl = this.getDataUrl();

            this._initColumns();
            this._initOperations();
        },

        _isColumnVisible: function (index) {
            return !!this._getConfigParam('isColumnAvailable', {})[index];
        },

        _initColumns: function () {
            this._columns.push({
                header: this.lmsg('date'),
                dataIndex: 'date',
                filter: {
                    type: 'date',
                    placeholder: this.lmsg('dateFilterPlaceholder'),
                },
                cls: 'number state-default',
            });
            this._columns.push({
                header: this.lmsg('type'),
                dataIndex: 'type',
                filter: {
                    type: 'text',
                    multi: true,
                    placeholder: this.lmsg('any'),
                    options: [{
                        type: 'value',
                        value: 'access',
                        label: 'Access',
                    }, {
                        type: 'value',
                        value: 'warning',
                        label: 'Warning',
                    }, {
                        type: 'value',
                        value: 'error',
                        label: 'Error',
                    }],
                },
                cls: 'number',
                renderer: function (item) {
                    if (!item.type) {
                        return '';
                    }
                    return item.type[0].toUpperCase() + item.type.substr(1);
                },
                isVisible: this._isColumnVisible.bind(this, 'type'),
            });
            this._columns.push({
                header: this.lmsg('ip'),
                dataIndex: 'ip',
                filter: {
                    type: 'text',
                    live: true,
                    placeholder: this.lmsg('ipFilterPlaceholder'),
                },
                cls: 'number',
                isVisible: this._isColumnVisible.bind(this, 'ip'),
            });
            this._columns.push({
                header: this.lmsg('code'),
                dataIndex: 'code',
                filter: {
                    type: 'text',
                    live: true,
                    startsWith: true,
                    placeholder: this.lmsg('codeFilterPlaceholder'),
                },
                cls: 'number',
                isVisible: this._isColumnVisible.bind(this, 'code'),
            });
            this._columns.push({
                header: this.lmsg('message'),
                dataIndex: 'message',
                filter: {
                    type: 'text',
                    live: true,
                    placeholder: this.lmsg('messageFilterPlaceholder'),
                },
                renderer: function (item) {
                    var text = this.makeLinks(item);

                    var rows = text.split('\n');
                    var visibleNumber = 10;
                    if (rows.length <= visibleNumber) {
                        return text.replace(/\n/g, '<br/>');
                    }
                    var firstRows = rows.slice(0, visibleNumber);
                    var lastRows = rows.slice(visibleNumber);
                    var more = '<a class="toggler js-more-link" href="#"><span>' + this.lmsg('more') + '&gt;&gt;</span></a>';
                    var less = '<a class="toggler js-more-link" href="#" style="display: none;"><span>&lt;&lt;' + this.lmsg('less') + '</span></a>';
                    return firstRows.join('<br/>') + '<div class="js-more" style="display: none;">' + lastRows.join('<br/>') + '</div>' +
                        '<div>' + more + less + '</div>';
                }.bind(this),
            });
            this._columns.push({
                header: this.lmsg('referer') +
                '<span class="tooltipData">' + this.lmsg('refererHint') + '</span>',
                dataIndex: 'referer',
                cls: 'min',
                renderer: function (item) {
                    if (item.referer === '-' || item.referer === '') {
                        return '';
                    }
                    var viewTitle = item.referer.escapeHTML();
                    var viewButton = '<span>' +
                        '<span class="tooltipData">' + viewTitle + '</span>' +
                        '<img src="' + this._urls.iconReferer + '"' +
                        ' alt="' + viewTitle.replace(/"/g, '&quot;') + '" title="" /></span>';
                    return viewButton;
                }.bind(this),
                isVisible: this._isColumnVisible.bind(this, 'referer'),
            });
            this._columns.push({
                header: this.lmsg('agent') +
                '<span class="tooltipData">' + this.lmsg('agentHint') + '</span>',
                dataIndex: 'agent',
                cls: 'min',
                renderer: function (item) {
                    var icon;
                    var viewTitle = item.agent.escapeHTML();
                    if (item.agent === '') {
                        return '';
                    } else if (item.agent === '-') {
                        icon = this._urls.iconUnknown;
                        viewTitle = this.lmsg('unknown');
                    } else if (item.agent.match(/bot/i)) {
                        icon = this._urls.iconRobot;
                    } else {
                        icon = this._urls.iconClient;
                    }

                    var viewButton = '<span>' +
                        '<span class="tooltipData">' + viewTitle + '</span>' +
                        '<img src="' + icon + '"' +
                        ' alt="' + viewTitle.replace(/"/g, '&quot;') + '" title="" /></span>';
                    return viewButton;
                }.bind(this),
                isVisible: this._isColumnVisible.bind(this, 'agent'),
            });
            this._columns.push({
                header: this.lmsg('size'),
                dataIndex: 'size',
                cls: 'state-default nowrap',
                isVisible: this._isColumnVisible.bind(this, 'size'),
            });
            this._columns.push({
                header: this.lmsg('source'),
                dataIndex: 'source',
                renderer: function (item) {
                    return this._sources[item.source];
                }.bind(this),
                cls: 'min nowrap state-default',
                isVisible: function () {
                    return this._sourcesSelected.length > 1;
                }.bind(this),
            });
        },

        _initOperations: function () {
            this._operations = [
                {
                    id: 'log-browser-start-realtime',
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('buttonStartRealtime'),
                    description: this.lmsg('buttonStartRealtimeHint'),
                    addCls: 'sb-start',
                    handler: function () {
                        this._componentElement.select('tr.divider').invoke('removeClassName', 'divider');
                        var lastRow = $('log-browser').select('tr.odd,tr.even').last();
                        if (lastRow) {
                            lastRow.addClassName('divider');
                        }
                        this.scrollToBottom();
                        this.toggleControls(true);
                        this.enableRealTime();
                    }.bind(this),
                }, {
                    id: 'log-browser-stop-realtime',
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('buttonStopRealtime'),
                    description: this.lmsg('buttonStopRealtimeHint'),
                    addCls: 'sb-stop',
                    hidden: true,
                    handler: function () {
                        this.toggleControls(false);
                        this.disableRealTime();
                    }.bind(this),
                }, {
                    id: 'log-browser-refresh',
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('buttonRefresh'),
                    description: this.lmsg('buttonRefreshHint'),
                    addCls: 'sb-refresh',
                    handler: function () {
                        this.reload();
                    }.bind(this),
                }, {
                    id: 'log-browser-clear',
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('buttonClear'),
                    description: this.lmsg('buttonClearHint'),
                    addCls: 'sb-clean',
                    hidden: true,
                    handler: function () {
                        this._additionalData.showMoreBefore = null;
                        this._data = [];
                        this.redraw();
                    }.bind(this),
                }, {
                    componentType: 'Jsw.bar.Separator',
                }, {
                    id: 'log-browser-add-custom',
                    componentType: 'Jsw.SmallButton',
                    title: this.lmsg('buttonAddCustom'),
                    description: this.lmsg('buttonAddCustomHint'),
                    addCls: 'sb-add-new',
                    hidden: !this._isCustomLog,
                    handler: function () {
                        this._isCustomLog = false;
                        new Ajax.Request(this._urls.addCustomLog, {
                            method: 'post',
                            parameters: { file: this._logFile },
                            onSuccess: function (transport) {
                                Jsw.clearStatusMessages();

                                var response = transport.responseText.evalJSON();
                                if (response.isUnsupportedLog) {
                                    Jsw.addStatusMessage('warning', this.lmsg('invalidLogFormat'));
                                }
                            }.bind(this),
                        });

                        Jsw.getComponent('log-browser-add-custom').hide();
                        Jsw.getComponent('log-browser-files').show();
                    }.bind(this),
                }, new Sources({
                    logBrowser: this,
                    hidden: this._isCustomLog,
                    webspaceId: this._webspaceId,
                    urls: this._urls,
                    locale: this.getLocale(),
                    id: 'log-browser-files',
                }),
            ];
        },

        onRedraw: function () {
            this.toggleControls();
            var table = $(this._tableId);
            if (!table) {
                return;
            }
            table.select('.alert-container').each(Element.remove);

            if (!this._additionalData.showMoreBefore) {
                return;
            }
            var moreRowHtml = '<tr class="alert-container">' +
                    '<td colspan="' + this._columns.length + '">' +
                        '<div class="alert-wrapper">' +
                            '<div class="alert">' +
                                this.lmsg('loadMoreOrUseFilters', {
                                    loadMore: '<a href="#" class="log-browser-load-more">' + this.lmsg('loadMore') + '</a>',
                                }) +
                            '</div>' +
                        '</div>' +
                    '</td>' +
                '</tr>';
            table.down('thead').insert({ after: moreRowHtml });
            var context = this;
            table.down('.log-browser-load-more').observe('click', function (event) {
                event.preventDefault();
                this.up('.alert-wrapper').update('<div class="ajax-loading">' + context.lmsg('loadingTitle') + '</div>');

                var params = '?showBefore=' + encodeURIComponent(JSON.stringify(context._additionalData.showMoreBefore))
                    + '&short=true';
                context.reload(params, 'before');
            });
        },

        _addEvents: function ($super) {
            $super();
            this._componentElement.observe('click', function (event) {
                var element = event.findElement('a.js-more-link');
                if (!element) {
                    return;
                }
                event.preventDefault();
                element.up('td').down('.js-more').toggle();
                element.up('div').select('a').invoke('toggle');
            });
        },
    });
});
