// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define(function () {
    return Class.create(Jsw.Component, {
        _initConfiguration: function ($super, config) {
            config = Object.extend({
                cls: 'log-browser-options',
            }, config || {});
            $super(config);

            this._logBrowser = this._getConfigParam('logBrowser');
            this._urls = this._getConfigParam('urls', {});
            this._webspaceId = this._getConfigParam('webspaceId');
        },

        isOpened: function () {
            return this._componentElement.down('.dropdown').hasClassName('open');
        },

        setOpened: function (isOpened) {
            this._componentElement.down('.dropdown').toggleClassName('open', isOpened);
        },

        getSelected: function () {
            return this._componentElement.down('.dropdown-menu').select('input[type="checkbox"]:checked').map(function (element) {
                return element.value;
            });
        },

        getTextValue: function () {
            var filterValue = this._logBrowser._sourcesSelected;
            if (!Object.keys(this._logBrowser._sources).length || (filterValue && !filterValue.length)) {
                return this.lmsg('sourceFilterNoLogs');
            }
            if (filterValue && filterValue.length === 1) {
                return this._logBrowser._sources[filterValue[0]];
            }
            if (!filterValue || filterValue.length === Object.keys(this._logBrowser._sources).length) {
                return this.lmsg('sourceFilterAllLogs');
            }
            return this.lmsg('sourceFilterFiles', { count: filterValue.length });
        },

        updateTextValue: function () {
            this._componentElement.select('input[type=text]').first().setValue(this.getTextValue());
        },

        addCustomLog: function (path) {
            new Ajax.Request(this._urls.addCustomLog, {
                method: 'post',
                parameters: {
                    file: path,
                },
                onSuccess: function (transport) {
                    this._logBrowser._sources[path] = path.split('/').pop();
                    this._logBrowser._sourcesSelected.push(path);
                    this._initComponentElement();
                    this._logBrowser.onSourceSearchFilterChange();

                    var response = transport.responseText.evalJSON();
                    if (response.isUnsupportedLog) {
                        Jsw.addStatusMessage('warning', this.lmsg('invalidLogFormat'));
                    }
                }.bind(this),
            });
        },

        unlinkCustomLog: function (path) {
            new Ajax.Request(this._urls.unlinkCustomLog, {
                method: 'post',
                parameters: {
                    file: path,
                },
                onSuccess: function () {
                    delete this._logBrowser._sources[path];
                    this._initComponentElement();
                    this._logBrowser.onSourceSearchFilterChange();
                }.bind(this),
            });
        },

        _initComponentElement: function ($super) {
            $super();

            var context = this;
            var filterValue = this._logBrowser._sourcesSelected;
            var dropdownElement = new Element('div', { class: 'dropdown' });
            dropdownElement.insert(
                '<div class="input-group">' +
                    '<input type="text" readonly class="form-control" value="' + this.getTextValue() + '">' +
                    '<span class="input-group-btn"><span class="btn dropdown-toggle">' +
                    '<button type="button"><span class="caret"></span></button></span></span>' +
                '</div>'
            );
            var dropdownMenu = new Element('ul', { class: 'dropdown-menu dropdown-menu-flip' });
            dropdownMenu.observe('click', function (event) {
                event.stopPropagation();
            });
            dropdownElement.insert(dropdownMenu);

            var regularFiles = {};
            var customFiles = {};
            Object.keys(this._logBrowser._sources).forEach(function (name) {
                if (name.startsWith('/')) {
                    customFiles[name] = context._logBrowser._sources[name];
                } else {
                    regularFiles[name] = context._logBrowser._sources[name];
                }
            });

            var dropdownMenuAddItem = function () {
                Jsw.render(dropdownMenu, Jsw.createElement.bind(null, 'li').apply(null, arguments));
            };

            var renderGroup = function (group, withTooltip) {
                var items = Object.keys(group).map(function (name) {
                    var item = Jsw.createElement('label', [
                        Jsw.createElement('input', {
                            type: 'checkbox',
                            checked: !filterValue || filterValue.indexOf(name) !== -1 ? 'checked' : null,
                            value: name.escapeHTML(),
                            onclick: function (event) {
                                event.stopPropagation();
                                context._logBrowser.onSourceSearchFilterChange();
                            },
                        }),
                        withTooltip ? Jsw.createElement('span.tooltipData', name.escapeHTML()) : '',
                        group[name].escapeHTML(),
                    ]);

                    if (name.startsWith('/')) {
                        item = Jsw.createElement('.b-indent.b-indent-flip', [
                            Jsw.createElement('span.b-indent-icon', [
                                Jsw.createElement('a', {
                                    onclick: function (event) {
                                        event.preventDefault();
                                        context.setOpened(false);
                                        Jsw.messageBox.show({
                                            subtype: 'unlink',
                                            text: context.lmsg('buttonUnlinkConfirmationDescription'),
                                            type: Jsw.messageBox.TYPE_YESNO,
                                            buttonTitles: {
                                                yes: context.lmsg('buttonUnlinkConfirmationYes'),
                                                no: context.lmsg('buttonUnlinkConfirmationNo'),
                                            },
                                            onYesClick: function () {
                                                context.unlinkCustomLog(name);
                                            },
                                        });
                                    },
                                }, [
                                    Jsw.createElement('span.tooltipData', context.lmsg('buttonUnlinkHint')),
                                    Jsw.createElement('i.icon-unlink'),
                                ]),
                            ]),
                            item,
                        ]);
                    }

                    return item;
                });

                return Jsw.createElement('.dropdown-menu-content.checkbox-group', items);
            };

            if (Object.keys(regularFiles).length) {
                dropdownMenuAddItem(renderGroup(regularFiles));
                dropdownMenuAddItem({ class: 'divider' });
            }
            if (Object.keys(customFiles).length) {
                dropdownMenuAddItem(renderGroup(customFiles, true));
                dropdownMenuAddItem({ class: 'divider' });
            }

            dropdownMenuAddItem(
                Jsw.createElement('a.s-btn.sb-folder', { href: this._urls.manageLogs },
                    Jsw.createElement('span', this.lmsg('manageLogFiles'))
                )
            );

            dropdownMenuAddItem(
                Jsw.createElement('a.s-btn.sb-add-new', {
                    onclick: function (event) {
                        event.preventDefault();
                        context.setOpened(false);
                        new Jsw.DirectoryBrowser({
                            subscriptionId: context._webspaceId,
                            showFiles: true,
                            treeConfig: {
                                filterNodes: function (item) {
                                    return item.isDirectory || 'text' === item.type || 'unknown_text' === item.type;
                                },
                            },
                            locale: {
                                title: this.lmsg('customLogPopupTitle'),
                                buttonOk: this.lmsg('buttonOk'),
                                buttonCancel: this.lmsg('buttonCancel'),
                            },
                            onSubmit: function (path) {
                                this.addCustomLog('/' + path);
                            }.bind(this),
                        });
                    }.bind(this),
                }, Jsw.createElement('span', this.lmsg('addCustomLogFile')))
            );

            this._componentElement.update(dropdownElement);
        },
    });
});
