// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

define([
    './popup-packages-base',
], function (PopupPackagesBase) {
    return Class.create(PopupPackagesBase, {
        _showLockStatus: true,

        render: function ($super) {
            $super();

            this.setBoxType('form-box');
            this.setTitle(this.lmsg('popupUpdateTitle'));
            this._updateButton(this._buttonOk, { disabled: true });

            $(this._contentAreaId).update('<div class="ajax-loading">' + this.lmsg('loading') + '</div>');

            new Ajax.Request(
                Jsw.prepareUrl(this._urls.packageUpdateInfo),
                {
                    method: 'post',
                    parameters: {
                        'ids[]': this._getPackagesIds(),
                    },
                    onSuccess: function (transport) {
                        var response = transport.responseText.evalJSON();

                        if (!response.data) {
                            $(this._contentAreaId).update('');
                            if (response.statusMessages) {
                                $A(response.statusMessages).each(function (message) {
                                    this._addStatusMessage(message.status, message.content);
                                }.bind(this));
                            }
                            return;
                        }

                        if (!response.data.length) {
                            $(this._contentAreaId).update('');
                            this._addStatusMessage('info', this.lmsg('selectedPackagesUpToDate'));
                            this._buttonOk.hide();
                            this._updateButton(this._buttonCancel, { title: this.lmsg('buttonOk') });
                            return;
                        }

                        $(this._contentAreaId).update(
                            '<p>' + this.lmsg('popupUpdatePackagesListTitle') + '</p>' +
                            '<ul class="list-unordered">' + this._packages.map(this._getPackageHtml.bind(this)).join('') + '</ul>'
                        );

                        var packagesIds = this._getPackagesIds();
                        var dependencies = response.data.filter(function (pkg) {
                            return packagesIds.indexOf(pkg.name) === -1;
                        });

                        if (dependencies.length) {
                            $(this._contentAreaId).insert(
                                '<p><a href="#" class="toggler" onclick="$(this).up(\'.content-area\').select(\'ul.js-dependencies\').first().toggle()">' +
                                this.lmsg('popupUpdateDependencies', { count: dependencies.length }) +
                                '</a></p>' +
                                '<ul class="list-unordered js-dependencies" style="display:none;">' +
                                dependencies.map(this._getPackageHtml.bind(this)).join('') +
                                '</ul>'
                            );
                        }

                        var lockedPackages = response.data.filter(function (pkg) {
                            return pkg.locked;
                        });
                        if (lockedPackages.length) {
                            $(this._contentAreaId).insert(
                                '<hr>' +
                                '<div class="b-indent">' +
                                '<span class="b-indent-icon"><i class="icon-warning"></i></span>' +
                                '<p>' + this.lmsg('popupUpdateLockedPackagesListTitle') + '</p>' +
                                '<ul class="list-unordered">' +
                                lockedPackages.map(this._getPackageHtml.bind(this)).join('') +
                                '</ul>' +
                                '</div>' +
                                '<div class="form-row">' +
                                '<div class="single-row">' +
                                '<div class="indent-box">' +
                                '<input id="fid-unlock-packages" type="checkbox" class="checkbox" >' +
                                '<div class="indent-box-content">' +
                                '<label for="fid-unlock-packages">' + this.lmsg('unlockPackagesForUpdateOnly') + '</label>' +
                                '</div>' +
                                '</div>' +
                                '</div>' +
                                '</div>'
                            );
                            $('fid-unlock-packages').observe('click', function (event) {
                                this._updateButton(this._buttonOk, { disabled: !event.element().checked });
                            }.bind(this));
                        } else {
                            this._updateButton(this._buttonOk, { disabled: false });
                        }
                    }.bind(this),
                }
            );
        },

        _onOkClick: function (event) {
            var updateUrl = this._urls.updatePackages;
            var progressBar = Jsw.getComponent('asyncProgressBarWrapper');
            var itemId = progressBar.fly(event.target.cumulativeOffset(),
                this.lmsg('taskUpdatingPackages', {
                    count: this._packages.length,
                }),
                function () {
                    new Ajax.Request(Jsw.prepareUrl(updateUrl), {
                        method: 'post',
                        parameters: {
                            'ids[]': this._getPackagesIds(),
                        },
                        onSuccess: function () {
                            progressBar.removePreparingItem(itemId);
                            progressBar.update();
                        },
                    });
                }.bind(this)
            );
            this.remove();
            this.hide();
        },
    });
});
