// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

/* global apsCatalog */

Smb.FormApsCatalog = Class.create(Jsw.FormAjax, {

    _isSearchResultsShown: false,
    _catalogType: 'apscatalog',
    _isCategoriesLoaded: false,

    _initConfiguration: function ($super, config) {
        $super(config);
        Jsw.baseUrl = this._getConfigParam('baseUrl', '');
        this._catalogType = $('catalogType').value;

        this._resizeCallback = this._getConfigParam('resizeCallback', null);
    },

    _initComponentElement: function ($super) {
        $super();

        var ce = Jsw.createElement;

        Jsw.render(this._componentElement,
            new Jsw.SmallTools({
                id: 'catalog-actions-box',
                locale: this.getLocale(),
                searchCollapsed: 'false' === Jsw.Cookie.get(this._id + '-search-show'),
                searchFilters: {
                    name: {
                    },
                },
                quickSearchInputId: 'quick-search-name',
            })
        );

        Jsw.render($('catalog-actions-box').down('.objects-toolbar'),
            ce('span#current-action.list-actions',
                ce('span#current-action-navigation'),
                ce('span#current-action-buttons',
                    ce('span.link-back-to-category',
                        ce('a', { href: '#tolist' }, this.lmsg('returnToCategories'))
                    )
                )
            ),
            'top'
        );

        Jsw.render($('catalog-actions-box').down('.quick-search-box'),
            ce('span.search-control',
                ce('a.s-btn.sb-search-show', this.lmsg('showSearch')),
                ce('a.s-btn.sb-search-hide', this.lmsg('hideSearch'))
            )
        );

        Jsw.render($('catalog-actions-box'),
            ce('#search-box.search-box',
                ce('ul',
                    ce('li',
                        '<div><span>' + this.lmsg('searchName') + '</span><input id="search-name" type="text" value=""/></div>'
                    ),
                    ce('input#search-vendor', { type: 'hidden', value: '' }),
                    ce('li',
                        '<div><span>' + this.lmsg('searchPackager') + '</span><input id="search-packager" type="text" value=""/></div>'
                    ),
                    ce('li',
                        ce('',
                            ce('span', this.lmsg('searchLicenseType')),
                            ce('select#search-license-type',
                                ce('option', { value: 'any', selected: 'selected' }, this.lmsg('searchLicenseTypeAny')),
                                ce('option', { value: 'freeware' }, this.lmsg('searchLicenseTypeFreeware')),
                                ce('option', { value: 'commercial' }, this.lmsg('searchLicenseTypeCommercial'))
                            )
                        )
                    )
                ),
                ce('a.s-btn.sb-search', { href: '#' }, this.lmsg('buttonSearch')),
                ce('a.s-btn.sb-show-all', { href: '#' }, this.lmsg('buttonResetSearch'))
            )
        );

        Jsw.render(this._componentElement,
            ce('.objects-content',
                ce('#overview-catalog.objects-content-area'),
                ce('#category-info.objects-content-area'),
                ce('#details-info.objects-content-area')
            )
        );
    },

    _executeResizeCallback: function () {
        if (this._resizeCallback) {
            this._resizeCallback();
        }
    },

    render: function ($super) {
        $super();
        this._addSearchHandlers();
        this._initBlocksResize();

        if ('apscatalog' === this._catalogType && document.location.hash !== '#show-all') {
            this.showCategories();
        } else {
            this.showSearchResults(false);
        }

        var category = this._getConfigParam('category');
        if (category) {
            this.showCategory(category);
        }

        var details = this._getConfigParam('details');
        if (details) {
            this.showDetails(details);
        }

        this._executeResizeCallback();
    },

    _showPanel: function (panel) {
        if (panel === 'overview-catalog') {
            $('overview-catalog').show();
        } else {
            $('overview-catalog').hide();
        }

        if (panel === 'category-info') {
            $('category-info').show();
        } else {
            $('category-info').hide();
        }

        if (panel === 'details-info') {
            $('details-info').show();
            $('current-action-navigation').hide();
            $('current-action-buttons').show();
            $('catalog-actions-box').down('.quick-search-box').hide();
            $('search-box').hide();
        } else {
            $('details-info').hide();
            $('current-action-navigation').show();
            $('current-action-buttons').hide();
            $('catalog-actions-box').down('.quick-search-box').show();
            $('search-box').show();
        }

        if (('marketplace' === this._catalogType) && ('category-info' === panel)) {
            $('catalog-actions-box').setStyle({ display: 'none' });
        } else {
            $('catalog-actions-box').setStyle({ display: '' });
        }

        this._executeResizeCallback();
    },

    returnToHome: function () {
        this.showCategories();

        return false;
    },

    returnToCategory: function () {
        if ($('category-info').empty()) {
            this.showCategories();
        } else {
            this._showPanel('category-info');
        }
        return false;
    },

    showCategories: function () {
        this._showPanel('overview-catalog');

        $('current-action-navigation').update(this.lmsg('showCategories'));

        this._isSearchResultsShown = false;
        if (this._isCategoriesLoaded) {
            return false;
        }

        Element.update($('overview-catalog'), '<div class="ajax-loading">' + this.lmsg('loading') + '</div>');

        new Ajax.Updater('overview-catalog', Jsw.baseUrl + '/app/categories-data/catalogId/' + $('catalogId').value, {
            method: 'get',
        });

        this._isCategoriesLoaded = true;

        return false;
    },

    showDetails: function (downloadItem) {
        Element.update($('details-info'), '<div class="ajax-loading">' + this.lmsg('loading') + '</div>');

        this._showPanel('details-info');

        var applicationDetails = $H(downloadItem);
        new Ajax.Updater({ success: 'details-info' }, Jsw.baseUrl + '/app/details-data/catalogId/' + $('catalogId').value, {
            method: 'post',
            parameters: { applicationDetails: Object.toJSON(applicationDetails) },
            evalScripts: true,
            onFailure: this.onErrorShowDetails.bind(this, downloadItem),
        });
        return false;
    },

    onErrorShowDetails: function (downloadItem) {
        $('details-info').update(this.lmsg('connectionFailed', { retryLink: '<a href="#">' + this.lmsg('retry') + '</a>' }));
        $('details-info').select('a').each(function (element) {
            element.observe('click', this.showDetailsRetry.bind(this, downloadItem));
        }, this);
    },

    showDetailsRetry: function (downloadItem) {
        this.showDetails(downloadItem);
    },

    showCategory: function (catExternal) {
        this.resetSearchResults(true);

        this._showPanel('category-info');

        Element.update($('category-info'), '<div class="ajax-loading">' + this.lmsg('loading') + '</div>');

        var context = this;

        new Ajax.Request(Jsw.baseUrl + '/app/applications-list-data/catalogId/' + $('catalogId').value, {
            method: 'get',
            parameters: { category: catExternal, cert: 'any' },
            onSuccess: function (transport) {
                try {
                    var result = transport.responseText.evalJSON();

                    if (result.list) {
                        Element.update($('category-info'), result.list);
                    }
                    if (result.navigation) {
                        Element.update($('current-action-navigation'), result.navigation);
                    }

                    context._executeResizeCallback();
                } catch (e) {

                }
            },
        });

        this._isSearchResultsShown = false;
        return false;
    },

    showCategoryRetry: function (catExternal) {
        this.showCategory(catExternal);
    },

    install: function (item) {
        $('name').value = item.name;
        var params = ['version', 'release', 'packager', 'vendor', 'advancedInstall'];
        for (var i = 0, length = params.length; i < length; i++) {
            if ($(params[i]).value === '') {
                $(params[i]).value = item[params[i]];
            }
        }
        this._componentElement.action = Jsw.baseUrl + '/app/download/';
        this._componentElement.submit();
        return false;
    },

    renderVersionsList: function (submenu, item, versions) {
        var versionKeys = Object.keys(versions);
        var showMore = new Element('li').update('<a href="#">' + this.lmsg('moreVersions') + '</a>').observe('click', function (e) {
            Event.stop(e);
            this.up().select('li').invoke('show');
            this.hide();
        });
        submenu.update();
        for (var i = 0; i < versionKeys.length; i++) {
            (function () {
                var versionData = versions[versionKeys[i]];
                var a = new Element('a', { href: '#' }).update(versionData.version).observe('click', function () {
                    var params = ['version', 'release', 'packager', 'vendor'];
                    for (var i = 0, length = params.length; i < length; i++) {
                        $(params[i]).value = versionData[params[i]];
                    }
                    item.advancedInstall = 1;
                    apsCatalog.install(item);
                    return false;
                });
                var li = new Element('li').insert(a);
                if (i > 9) {
                    li.hide();
                }
                submenu.insert(li);
            }());
        }
        if (versionKeys.length > 9) {
            submenu.insert(showMore);
        }
    },

    showAppVersions: function (item, submenu) {
        if (Object.keys(item.versions).length) {
            apsCatalog.renderVersionsList(submenu, item, item.versions);
            return;
        }

        submenu.update('<div class="ajax-loading">' + this.lmsg('loading') + '</div>');
        new Ajax.Request(Jsw.baseUrl + '/app/versions-data/catalogId/' + $('catalogId').value, {
            method: 'post',
            parameters: {
                applicationDetails: JSON.stringify(item),
            },
            onSuccess: function (transport) {
                var result = transport.responseText.evalJSON();
                apsCatalog.renderVersionsList(submenu, item, result.versions);
            },
        });
    },

    uninstall: function (packageId) {
        $('packageId').value = packageId;
        this._componentElement.action = Jsw.baseUrl + '/app/remove/';
        this._componentElement.submit();
        return false;
    },

    _addSearchHandlers: function () {
        var context = this;

        this._componentElement.select('.search-control .s-btn').each(function (element) {
            element.observe('click', function () {
                this.up('.actions-box').toggleClassName('actions-box-search-show');
                Jsw.Cookie.set(context._id + '-search-show', this.up('.actions-box').hasClassName('actions-box-search-show'));
                context._executeResizeCallback();
                return false;
            });
        });

        var quickSearchLink = this._componentElement.select('.quick-search-box').first().select('em')
            .first();
        quickSearchLink.observe('click', function () {
            context.showSearchResults(true);
        });

        var searchLink = this._componentElement.select('.sb-search').first();
        searchLink.observe('click', function () {
            context.showSearchResults(false);
        });

        var resetSearchLink = this._componentElement.select('.sb-show-all').first();
        resetSearchLink.observe('click', function () {
            context.resetSearchResults();
        });

        var searchFields = $w('quick-search-name search-name search-vendor search-packager search-license-type');
        searchFields.each(function (searchFieldName) {
            $(searchFieldName).observe('keydown', function (event) {
                // search by Enter keydown
                if (13 === event.keyCode) {
                    var isQuick = false;
                    if ('quick-search-name' === searchFieldName) {
                        isQuick = true;
                    }
                    context.showSearchResults(isQuick);
                    Event.stop(event);
                    return false;
                }
            });
        });

        this._componentElement.select('.link-back-to-category a').each(function (element) {
            element.observe('click', this.returnToCategory.bind(this));
        }, this);
    },

    showSearchResults: function (isQuick) {
        if (isQuick) {
            $('search-packager').value = '';
            $('search-license-type').value = 'any';
            $('search-name').value = $('quick-search-name').value;
        } else {
            $('quick-search-name').value = $('search-name').value;
        }
        this._showPanel('category-info');

        var context = this;
        var showCategoriesLink = new Element('a', { href: '#' });
        showCategoriesLink.update(this.lmsg('gotoHome'));
        showCategoriesLink.observe('click', function () {
            context.resetSearchResults(true);
            context.showCategories();
        });
        $('current-action-navigation').update(showCategoriesLink).insert(' > ' + this.lmsg('searchResults'));

        $('category-info').update('<div class="ajax-loading">' + this.lmsg('loading') + '</div>');

        new Ajax.Updater('category-info', Jsw.baseUrl + '/app/search-data/catalogId/' + $('catalogId').value, {
            method: 'get',
            evalScripts: true,
            parameters: {
                name: $('search-name').value,
                vendor: $('search-vendor').value,
                packager: $('search-packager').value,
                cert: 'any',
                licenseType: $('search-license-type').value,
            },

            onComplete: function () {
                Smb.Views.Apps.fixContentWidth();
                Smb.Views.Apps.fixMarketplaceContentHeight();
            },
        });

        this._isSearchResultsShown = true;

        return false;
    },

    resetSearchResults: function (searchFieldsOnly) {
        $('quick-search-name').value = '';
        $('search-name').value = '';
        $('search-vendor').value = '';
        $('search-packager').value = '';
        $('search-license-type').value = 'any';

        if (!this._isSearchResultsShown || searchFieldsOnly) {
            return;
        }

        this._isSearchResultsShown = false;

        this.showSearchResults(false);
    },

    // resizable height for div elements
    _initBlocksResize: function () {
        try {
            // this needs to be executed twice for proper work on Firefox
            this._resizeBlocks();
            this._resizeBlocks();
            var context = this;
            Event.observe(window, 'resize', function () {
                context._resizeBlocks();
                context._resizeBlocks();
            }, false);
        } catch (e) {
            // alert(e.description);
        }
    },

    _resizeBlocks: function () {
        try {
            var bodyHeight = document.body.clientHeight - 12;
            if (Prototype.Browser.IE) {
                bodyHeight = document.documentElement.clientHeight - 12;
            }
            var blockTop = Element.cumulativeOffset($('application-info'))[1];
            var blockBottom = Element.getHeight('footer-wrapper');
            var newBlockHeight = Math.max(bodyHeight - blockTop - blockBottom, 200);

            $('application-info').style.height = newBlockHeight + 'px';
            $('category-select').style.height = newBlockHeight + 'px';
        } catch (e) {
        }
    },
});

Smb.ApsCatalogProgressBar = Class.create(Jsw.Component, {
    STATUS_STARTED: 0,
    STATUS_NOT_STARTED: 1,
    STATUS_DONE: 2,
    STATUS_ERROR: 3,

    _initConfiguration: function ($super, config) {
        $super(config);
        var context = this;
        window.setTimeout(function () {
            context.updateProgress();
        }, 1000);
    },

    _initComponentElement: function () {
        this._progressTextElement = new Element('div', {
            class: 'progress-label',
        });
        this._progressBarElement = new Element('div', {
            class: 'progress',
        }).update('<div class="progress-bar"></div>');
        this._componentElement = new Element('div', {
            class: 'progress-box app-progress-panel',
        });
        this._componentElement.appendChild(this._progressTextElement);
        this._componentElement.appendChild(this._progressBarElement);
        this._update(0);
    },

    _update: function (percents) {
        percents = Math.max(percents, 1);
        this._progressTextElement.update(this.lmsg('progress', { percents: percents }));
        this._progressBarElement.select('div.progress-bar').each(function (element) {
            $(element).setStyle({
                width: percents + '%',
            });
        });
    },

    updateProgress: function () {
        var context = this;
        new Ajax.Request(context._getConfigParam('requestUrl'), {
            method: 'get',
            onSuccess: function (transport) {
                try {
                    var result = transport.responseText.evalJSON();
                    if (result.progress) {
                        context._update(result.progress);
                    }
                    if (result.status === context.STATUS_STARTED || result.status === context.STATUS_NOT_STARTED) {
                        setTimeout(function () {
                            context.updateProgress();
                        }, context._getConfigParam('timeout', 1000));
                        return;
                    }
                    result.url = result.url
                        ? result.url
                        : context._getConfigParam('redirectUrl').replace('%packageId%', result.packageId).replace('%resourceId%', result.resourceId);
                    result.method = context._getConfigParam('redirectMethod', 'get');
                    Jsw.redirect(result);
                } catch (e) {
                }
            },
        });
    },
});
