// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

Jsw.namespace('Smb.Views');

Smb.Views.Email = {
    isAttachmentsSupported: true,

    updateRedirectControlsState: function () {
        if ($('redirect-redirectSection-enabled').checked) {
            $('redirect-redirectSection-subscribers').enable();
        } else {
            $('redirect-redirectSection-subscribers').disable();
        }
    },

    updateAutoresponderControlsState: function () {
        var elements = $A([
            'autoResponder-autoResponderSection-contentType-textplain',
            'autoResponder-autoResponderSection-contentType-texthtml',
            'autoResponder-autoResponderSection-codepage',
            'autoResponder-autoResponderSection-subject',
            'autoResponder-autoResponderSection-message',
            'autoResponder-autoResponderSection-replyTo',
            'autoResponder-autoResponderSection-forwardAddress',
            'autoResponder-autoResponderSection-replyLimit',
            'autoResponder-autoResponderSection-endDateEnabled',
        ]);
        if ($('autoResponder-autoResponderSection-enabled').checked) {
            elements.each(function (item) {
                if ($(item)) {
                    $(item).enable();
                }
            });
            if ($('attachments')) {
                $('attachments').show();
            }
        } else {
            elements.each(function (item) {
                if ($(item)) {
                    $(item).disable();
                }
            });
            if ($('attachments')) {
                $('attachments').hide();
            }
        }
        Smb.Views.Email.updateEndDateControlsState();
    },

    updateEndDateControlsState: function () {
        var elements = [
            $('autoResponder-autoResponderSection-endDate-year'),
            $('autoResponder-autoResponderSection-endDate-month'),
            $('autoResponder-autoResponderSection-endDate-day'),
        ];
        var endDateEnabledElement = $('autoResponder-autoResponderSection-endDateEnabled');

        elements.invoke(!endDateEnabledElement.disabled && endDateEnabledElement.checked ? 'enable' : 'disable');
    },

    updateAntivirusControlsState: function () {
        if ($('antivirus-antivirusSection-enabled').checked) {
            if ($('antivirus-antivirusSection-antivirusSettings-incoming')) {
                $('antivirus-antivirusSection-antivirusSettings-incoming').enable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-outgoing')) {
                $('antivirus-antivirusSection-antivirusSettings-outgoing').enable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-both')) {
                $('antivirus-antivirusSection-antivirusSettings-both').enable();
            }
        } else {
            if ($('antivirus-antivirusSection-antivirusSettings-incoming')) {
                $('antivirus-antivirusSection-antivirusSettings-incoming').disable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-outgoing')) {
                $('antivirus-antivirusSection-antivirusSettings-outgoing').disable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-both')) {
                $('antivirus-antivirusSection-antivirusSettings-both').disable();
            }
        }
    },

    updateAntispamControlsState: function () {
        var antispamEnabled = $('antispam-mainSection-enabled').checked;


        Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-mark'), antispamEnabled);
        var markEnabled = $('antispam-mainSection-antispamSettings-mark').checked;
        Smb.Views.Email.enableControl($('antispam-mainSection-subjText'), antispamEnabled && markEnabled);
        Smb.Views.Email.enableControl($('antispam-mainSection-reportText'), antispamEnabled && markEnabled);
        Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-delete'), antispamEnabled);
        Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-move'), antispamEnabled);
        Smb.Views.Email.enableControl($('antispam-advancedSection-filterScore-score'), antispamEnabled);
        Smb.Views.Email.enableControl($('antispam-advancedSection-whiteList-list'), antispamEnabled);
        Smb.Views.Email.enableControl($('antispam-advancedSection-blackList-list'), antispamEnabled);
        Smb.Views.Email.enableControl($('antispam-advancedSection-trustedNetworks-networks'), antispamEnabled);
    },

    enableControl: function (ctrl, value) {
        if (!ctrl) {
            return;
        }
        if (value) {
            ctrl.enable();
        } else {
            ctrl.disable();
        }
    },
};

Smb.Views.EmailFormAjax = Class.create(Jsw.FormAjax, {
    _initComponentElement: function ($super) {
        $super();
        this._attachmentsCount = this._getAttachments().length;
    },

    _getAttachments: function () {
        return $('attachments')
            ? $('attachments').select('div.dynamicSubFormContent input').filter(function (item) {
                return item.value !== '';
            })
            : [];
    },

    disable: function ($super) {
        this._getAttachments().invoke('disable');
        $super();
    },

    enable: function ($super) {
        this._getAttachments().invoke('enable');
        $super();
    },

    _processForm: function ($super, response) {
        if (response.emailId) {
            var attachments = this._getAttachments();
            var newAttachmentsCount = attachments.filter(function (item) {
                return item.type === 'file';
            }).length;
            var removedAttachmentsCount = this._attachmentsCount - (attachments.length - newAttachmentsCount);
            if (newAttachmentsCount || removedAttachmentsCount) {
                this._processAttachments($super, response);
                return;
            }
        }
        $super(response);
    },

    _processAttachments: function (continueCallback, response) {
        var form = new Element('form', {
            action: Jsw.baseUrl + '/email-address/responder-attachments/id/' + response.emailId,
            method: 'post',
            enctype: 'multipart/form-data',
            encoding: 'multipart/form-data',
            style: 'display:none',
        });
        if ($('forgery_protection_token')) {
            form.insert(new Element('input', {
                type: 'hidden',
                name: 'forgery_protection_token',
                value: $('forgery_protection_token').content,
            }));
        }
        this._componentElement.insert({ after: form });
        this._getAttachments().each(function (input) {
            input.enable();
            input.name = 'file[]';
            var inputCopy = input.clone();
            input.hide();
            input.insert({ after: inputCopy });
            form.insert(input);
        });
        form.submit();
    },
});

Jsw.onReady(function () {
    Smb.Views.Email.updateRedirectControlsState();
    $('redirect-redirectSection-enabled').observe('click', Smb.Views.Email.updateRedirectControlsState);

    if ($('autoResponder-autoResponderSection-enabled')) {
        Smb.Views.Email.updateAutoresponderControlsState();
        $('autoResponder-autoResponderSection-enabled').observe('click', Smb.Views.Email.updateAutoresponderControlsState);
        $('autoResponder-autoResponderSection-endDateEnabled').observe('click', Smb.Views.Email.updateEndDateControlsState);
    }

    if ($('antivirus-antivirusSection-enabled')) {
        Smb.Views.Email.updateAntivirusControlsState();
        $('antivirus-antivirusSection-enabled').observe('click', Smb.Views.Email.updateAntivirusControlsState);
    }

    if ($('antispam-mainSection-enabled')) {
        Smb.Views.Email.updateAntispamControlsState();
        $('antispam-mainSection-enabled').observe('click', Smb.Views.Email.updateAntispamControlsState);
        $('antispam-mainSection-antispamSettings-mark').observe('click', Smb.Views.Email.updateAntispamControlsState);
        $('antispam-mainSection-antispamSettings-delete').observe('click', Smb.Views.Email.updateAntispamControlsState);
        if ($('antispam-mainSection-antispamSettings-move')) {
            $('antispam-mainSection-antispamSettings-move').observe('click', Smb.Views.Email.updateAntispamControlsState);
        }
    }
});
