// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

Jsw.namespace('Smb.Database');

Smb.Database.ListOperations = Class.create(Jsw.Component, {
    _initConfiguration: function ($super, config) {
        $super(config);
        this._dumpPathMaxLength = this._getConfigParam('dumpPathMaxLength');
        this._invalidPathChars = this._getConfigParam('invalidPathChars', '');
    },

    openWebAdmin: function (item) {
        var params = {};

        if (item.askPasswordForWebadmin) {
            // eslint-disable-next-line no-alert
            var dbPassword = prompt(this.lmsg('promptPassword').replace('%%login%%', item.webadminUserLogin));
            if (!dbPassword) {
                return;
            }
            params.dbPassword = dbPassword;
        }
        Jsw.redirect({
            url: '/database/webadmin/id/' + item.id,
            method: params.dbPassword ? 'post' : 'get',
            data: params,
            target: 'dbedit',
        });
    },

    connectionInfo: function (item) {
        new Smb.Database.ConnectionInfoPopup({
            locale: this.getLocale(),
            serverAddress: item.serverHost + (item.serverPort > 0 ? ':' + item.serverPort : ''),
            databaseName: item.name,
            userLogin: item.webadminUserLogin || this.lmsg('connectionInfoNoUserLogin'),
            userPassword: (item.webadminUserLogin && (item.webadminUserPassword || '******')) || '',
        });
    },

    downloadDump: function (item) {
        new Smb.Database.DownloadPopup({
            locale: this.getLocale(),
            databaseId: item.id,
            databaseName: item.name,
            databaseType: item.type,
            dumpPathMaxLength: this._dumpPathMaxLength,
            isWebspace: item.isWebspace,
            invalidPathChars: this._invalidPathChars,
        });
    },

    uploadDump: function (item) {
        new Smb.Database.UploadPopup({
            locale: this.getLocale(),
            databaseId: item.id,
            databaseName: item.name,
            allowRecreateDatabase: item.allowRecreateDatabase,
            isWebspace: item.isWebspace,
        });
    },

    assignToSubscription: function (item) {
        new Smb.Database.AssignToSubscription({
            locale: this.getLocale(),
            ids: [item.id],
        });
    },

    checkDatabase: function (item, event) {
        var beginOffset = event.target.cumulativeOffset();
        var progressBar = Jsw.getComponent('asyncProgressBarWrapper');
        var itemId = progressBar.fly(beginOffset,
            this.lmsg('taskCheckDatabase'),
            function () {
                new Ajax.Request(Jsw.prepareUrl('/database/check/id/' + item.id), {
                    method: 'post',
                    onSuccess: function () {
                        progressBar.removePreparingItem(itemId);
                        progressBar.update();
                    },
                });
            }
        );
    },

    deleteConfirmation: function (item) {
        Jsw.messageBox.show({
            type: Jsw.messageBox.TYPE_YESNO,
            buttonTitles: {
                yes: this.lmsg('buttonDeleteConfirmationYes'),
                no: this.lmsg('buttonDeleteConfirmationNo'),
            },
            text: this.lmsg('buttonDeleteConfirmationDescription'),
            subtype: 'delete',
            onYesClick: function () {
                Jsw.redirectPost('/database/delete', { ids: { 0: item.id }, redirect: true });
            },
        });
    },
});

Smb.Database.AssignToSubscription = Class.create(Jsw.PopupForm, {
    _initConfiguration: function ($super, config) {
        $super(config);
        this._ids = this._getConfigParam('ids', []);
        this._handlerUrl = this._getConfigParam('handlerUrl', '/database/assign-to-subscription');
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
    },

    render: function ($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this.lmsg('assignToSubscriptionPopupTitle'));

        $(this._contentAreaId).update(
            '<p>' + this.lmsg('assignToSubscriptionPopupDesc') + '</p>' +
            '<form method="post" id="' + this._id + '-form">' +
                '<div class="form-row">' +
                '<div class="field-name"><label for="' + this._id + '-domain-name">' + this.lmsg('assignToSubscription') + '</label></div>' +
                '<div class="field-value" id="' + this._id + '-container">' +
                '</div>' +
                '</form>'
        );

        this._initSearch();
        this._initOnSubmitEvent();
        this.addOkButton();
        this.addCancelButton();
    },

    _initSearch: function () {
        this._searchComponent = new Jsw.LookUp({
            id: this._id + '-domain-name',
            name: this._id + '-domain-name',
            renderTo: this._id + '-container',
            locale: this.getLocale(),
            dataUrl: '/database/subscription-list/',
            copyTitleValue: true,
        });
    },

    _onSubmitEvent: function (event) {
        this._handler();
        Event.stop(event);
        return false;
    },

    _initOnSubmitEvent: function () {
        var form = $(this._id + '-form');
        form._formSubmit = form.submit;
        form.submit = this._onOkClick.bind(this);
        form.observe('submit', this._onSubmitEvent.bind(this));
    },

    addOkButton: function () {
        this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true, { id: this._sendButtonId });
    },

    addCancelButton: function () {
        this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false, { id: this._cancelButtonId });
    },

    _onOkClick: function () {
        this._handler();
    },

    _onCancelClick: function () {
        this.hide();
    },

    disable: function () {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = true;
            sendButtonWrapper.addClassName('disabled');
            if (!this._sendButtonTitle) {
                this._sendButtonTitle = sendButton.innerHTML;
            }
            sendButton.update('<span class="wait">' + this.lmsg('loading') + '</span>');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = true;
            cancelButtonWrapper.addClassName('disabled');
        }
    },

    enable: function () {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = false;
            sendButton.update(this._sendButtonTitle);
            sendButtonWrapper.removeClassName('disabled');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = false;
            cancelButtonWrapper.removeClassName('disabled');
        }
    },

    _onException: function (transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
        this.enable();
    },

    _onSuccess: function (request) {
        Jsw.clearStatusMessages();
        try {
            var response = request.responseText.evalJSON();
            this._processForm(response);
        } catch (e) {
            // show error, if request wasn't aborted by user
            if (0 != request.status) {
                Jsw.showInternalError(request.responseText);
                this.hide();
            }
        }
    },

    _processForm: function (response) {
        if (response.status && 'error' === response.status) {
            $A(response.statusMessages).each(function (value) {
                Jsw.addStatusMessage(value.status, value.content, { title: value.title });
            });

            this.hide();
        } else {
            document.location.reload();
        }
    },

    _handler: function () {
        var domainName = this._searchComponent.getDisplayValue();
        if (!domainName) {
            return;
        }

        new Ajax.Request(Jsw.baseUrl + this._handlerUrl, {
            method: 'post',
            parameters: {
                domainName: domainName,
                'ids[]': this._ids,
            },
            onException: this._onException.bind(this),
            onSuccess: this._onSuccess.bind(this),
        });
    },
});

Smb.Database.DownloadPopup = Class.create(Jsw.PopupForm, {
    _initConfiguration: function ($super, config) {
        $super(config);
        this._databaseId = this._getConfigParam('databaseId');
        this._databaseName = this._getConfigParam('databaseName');
        this._isWebspace = this._getConfigParam('isWebspace', false);
        this._dumpExt = 'mssql' === this._getConfigParam('databaseType') ? '.zip' : '.sql.zip';
        this._dumpPathMaxLength = this._getConfigParam('dumpPathMaxLength');
        this._invalidPathChars = this._getConfigParam('invalidPathChars', '');
    },

    _getDateString: function () {
        var padString = function (input) {
            return ('0' + input).slice(-2);
        };

        var dateObj = new Date();
        return dateObj.getFullYear() + '-' + padString(dateObj.getMonth() + 1) + '-' + padString(dateObj.getDate()) + '_' +
            padString(dateObj.getHours()) + '-' + padString(dateObj.getMinutes()) + '-' + padString(dateObj.getSeconds());
    },

    render: function ($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this.lmsg('downloadPopupTitle'));
        this.setTitleType(this._getConfigParam('titleCls', 'pp-download'));

        var dumpName = this._databaseName.escapeHTML() + '_' + this._getDateString();

        var relativePath = this._isWebspace ?
            '<div class="form-row">' +
                '<div class="field-name"><label>' + this.lmsg('downloadRelativePath') + '</label></div>' +
                '<div class="field-value">' +
                    '<div class="scrollable fm-scrollable">' +
                        '<div class="scrollable-wrap">' +
                            '<div id="' + this._id + '-files-tree" class="tree"></div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>'
            : '';
        $(this._contentAreaId).update(
            '<form method="post" id="' + this._id + '-form">' +
                '<p>' + this.lmsg('downloadHint').replace('%%database%%', '<b>' + this._databaseName.escapeHTML() + '</b>') + '</p>' +
                relativePath +
                '<div class="form-row">' +
                    '<div class="field-name"><label for="' + this._id + '-dump-name">' + this.lmsg('downloadDumpName') + '</label></div>' +
                    '<div class="field-value">' +
                        '<input id="' + this._id + '-dump-name" type="text" class="input-text f-middle-size" value="' + dumpName + '">' + this._dumpExt +
                        '<span class="field-errors" style="display: none;"></span>' +
                    '</div>' +
                '</div>' +
                '<div class="form-row">' +
                    '<div class="single-row">' +
                        '<div class="indent-box">' +
                            '<input type="checkbox" id="' + this._id + '-download-dump-automatically" class="checkbox">' +
                            '<div class="indent-box-content">' +
                                '<label for="' + this._id + '-download-dump-automatically">' + this.lmsg('downloadDumpAutomatically') + '</label>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        if (this._isWebspace) {
            this._tree = new Jsw.FileManager.Tree({
                applyTo: this._id + '-files-tree',
                data: null,
                dataUrl: '/file-manager/tree-data',
                rootNodeTitle: this.lmsg('rootDirectory'),
                waitMessage: this.lmsg('waitMessage'),
                onReload: function () {
                    this.setDirectory('/');
                },
            });
        }

        $(this._id + '-form').observe('submit', this._onOkClick.bindAsEventListener(this));
        this._buttonOk = this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true);
        this._buttonCancel = this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false);
        $(this._id + '-dump-name').focus();
    },

    _onOkClick: function (event) {
        if (event) {
            event.preventDefault();
        }
        if (this._buttonOk.down('button').disabled) {
            return;
        }

        this._clearFieldErrors();

        var dumpName = $(this._id + '-dump-name').getValue();
        if ('' === dumpName) {
            this._addFieldError(this.lmsg('errorDumpNameEmpty'));
            return;
        }

        if (this._invalidPathChars.length && dumpName.match(new RegExp('[' + this._invalidPathChars + ']'))) {
            this._addFieldError(this.lmsg('errorDumpNameInvalid').replace('%%chars%%', this._invalidPathChars));
            return;
        }

        var context = this;
        var params = {
            relativePath: this._isWebspace ? this._tree.getSelectedItemData().path : '',
            dumpName: dumpName + this._dumpExt,
        };

        if ((params.relativePath + '/' + params.dumpName).length > this._dumpPathMaxLength) {
            this._addFieldError(this.lmsg('errorDumpNameTooLong').replace('%%maxLength%%', this._dumpPathMaxLength));
            return;
        }

        var downloadDumpAutomatically = $(this._id + '-download-dump-automatically').checked;

        this._updateButton(this._buttonOk, { disabled: true });
        this._updateButton(this._buttonCancel, { disabled: true });
        var beginOffset = this._buttonOk.cumulativeOffset();
        var progressBar = Jsw.getComponent('asyncProgressBarWrapper');
        var itemId = progressBar.fly(beginOffset,
            this.lmsg('downloadTask').replace('%%database%%', this._databaseName.escapeHTML()),
            function () {
                context.hide();
                new Ajax.Request(Jsw.prepareUrl('/database/download-dump/id/' + context._databaseId), {
                    method: 'post',
                    parameters: params,
                    onSuccess: function (response) {
                        response = response.responseText.evalJSON();
                        if (response.redirect) {
                            Jsw.redirect(response.redirect);
                            return;
                        }
                        Jsw.Observer.append(function (item) {
                            if (downloadDumpAutomatically && 'database-download' === item.type && 'done' === item.status) {
                                var downloadLink = new Element('div').insert(item.progressTitleHtml).select('.download-link')
                                    .first();
                                var downloadParams = downloadLink ? downloadLink.href.toQueryParams() : {};
                                if (downloadParams.file === params.dumpName && downloadParams.currentDir === params.relativePath) {
                                    Jsw.redirect(downloadLink.href);
                                }
                            }
                        }, 'plesk:taskComplete');
                        progressBar.removePreparingItem(itemId);
                        progressBar.update();
                    },
                });
            });
    },

    _onCancelClick: function () {
        this.hide();
    },

    _addFieldError: function (message) {
        var errors = $(this._id + '-dump-name').next('.field-errors');
        errors.up('.form-row').addClassName('error');
        errors.update('<span class="error-hint">' + message.escapeHTML() + '</span>');
        errors.show();
    },

    _clearFieldErrors: function () {
        var errors = $(this._id + '-dump-name').next('.field-errors');
        errors.up('.form-row').removeClassName('error');
        errors.hide();
    },
});

Smb.Database.UploadPopup = Class.create(Jsw.PopupForm, {
    _initConfiguration: function ($super, config) {
        $super(config);
        this._databaseId = this._getConfigParam('databaseId');
        this._databaseName = this._getConfigParam('databaseName');
        this._isWebspace = this._getConfigParam('isWebspace');
        this._allowRecreateDatabase = this._getConfigParam('allowRecreateDatabase', false);
    },

    render: function ($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this.lmsg('uploadPopupTitle'));
        this.setTitleType(this._getConfigParam('titleCls', 'pp-upload'));
        var recreateDatabaseContent = this._allowRecreateDatabase ?
            '<div class="form-row">' +
                '<div class="single-row">' +
                    '<div class="indent-box">' +
                        '<input type="checkbox" value="1" name="recreateDatabase" id="' + this._id + '-recreate-database" class="checkbox" checked="checked">' +
                        '<div class="indent-box-content">' +
                            '<label for="' + this._id + '-recreate-database">' + this.lmsg('recreateDatabase') + '</label>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>' : '';

        $(this._contentAreaId).update(
            '<form method="post" action="' +
                Jsw.prepareUrl('/database/upload-dump/id/' + this._databaseId) + '" enctype="multipart/form-data" id="' + this._id + '-form" target="_uploadIFrame"' +
            '>' +
                '<input name="forgery_protection_token" type="hidden" value="' + $('forgery_protection_token').content + '"/>' +
                '<p>' + this.lmsg('uploadHint').replace('%%database%%', '<b>' + this._databaseName.escapeHTML() + '</b>') + '</p>' +
                (!this._isWebspace ? (
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<input id="' + this._id + '-dump-file" name="file[]" type="file" class="input-file f-max-size">' +
                        '</div>' +
                    '</div>'
                ) : (
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="radio" value="file" id="' + this._id + '-upload-type-file" class="radio" name="' + this._id + '-upload-type" checked>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-upload-type-file">' + this.lmsg('uploadFromFile') + '</label>' +
                                    '<div class="form-row">' +
                                        '<div class="single-row">' +
                                            '<input id="' + this._id + '-dump-file" name="file[]" type="file" class="input-file f-max-size">' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                            '<div class="indent-box">' +
                                '<input type="radio" value="webspace" id="' + this._id + '-upload-type-webspace" class="radio" name="' + this._id + '-upload-type">' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-upload-type-webspace">' + this.lmsg('uploadFromWebspace') + '</label>' +
                                    '<div class="form-row">' +
                                        '<div class="single-row">' +
                                            '<div class="scrollable fm-scrollable">' +
                                                '<div class="scrollable-wrap">' +
                                                    '<div id="' + this._id + '-files-tree" class="tree"></div>' +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>'
                )) +
                recreateDatabaseContent +
            '</form>'
        );

        if (this._isWebspace) {
            this._tree = new Jsw.FileManager.Tree({
                applyTo: this._id + '-files-tree',
                data: null,
                dataUrl: '/file-manager/tree-data',
                showFiles: true,
                rootNodeTitle: this.lmsg('rootDirectory'),
                waitMessage: this.lmsg('waitMessage'),
                onNodeClick: this._onTreeNodeClick.bind(this),
                onReload: function () {
                    this.setDirectory('/');
                },
            });
        }

        this._fileInputElement = $(this._id + '-dump-file');
        this._fileInputElement.observe('click', this._onFileInputClick.bind(this));
        this._fileInputElement.observe('change', this._updateOkStatus.bind(this));

        if (this._isWebspace) {
            this._uploadTypeFile = $(this._id + '-upload-type-file');
            this._uploadTypeFile.observe('change', this._updateOkStatus.bind(this));

            this._uploadTypeWebspace = $(this._id + '-upload-type-webspace');
            this._uploadTypeWebspace.observe('change', this._updateOkStatus.bind(this));
        }

        this._uploadAdapter = new Jsw.FileManager.UploadAdapterIframe({
            contentAreaId: this._contentAreaId,
            handlerUrl: this._fileInputElement.up('form').action,
            startUploadHandler: function () {},
            finishUploadHandler: this._onFinishUpload.bind(this),
            progressUploadHandler: function () {},
        });

        this._buttonOk = this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true, { disabled: true });
        this._updateButton(this._buttonOk, { disabled: true });
        this._buttonCancel = this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false);
    },

    _onTreeNodeClick: function () {
        this._uploadTypeWebspace.checked = true;
        this._updateOkStatus();
    },

    _onFileInputClick: function () {
        this._uploadTypeFile.checked = true;
        this._updateOkStatus();
    },

    _updateOkStatus: function () {
        var checkStatus = this._fileInputElement.value;
        if (this._isWebspace) {
            checkStatus = (checkStatus && this._uploadTypeFile.checked) || (this._uploadTypeWebspace.checked && !this._tree.getSelectedItemData().isDirectory);
        }
        this._updateButton(this._buttonOk, {
            disabled: !(checkStatus),
        });
    },

    _onOkClick: function () {
        this._updateButton(this._buttonOk, {
            title: '<span class="wait">' + this.lmsg('uploading') + '</span>',
            disabled: true,
        });

        this._updateButton(this._buttonCancel, { disabled: true });
        var recreateDatabase = this._allowRecreateDatabase ? $(this._id + '-recreate-database').checked : false;

        var uploadTypeFile = this._isWebspace ? this._uploadTypeFile.checked : true;
        if (uploadTypeFile) {
            this._fileInputElement.up('form').submit();
            return;
        }

        var context = this;
        var beginOffset = this._buttonOk.cumulativeOffset();
        var progressBar = Jsw.getComponent('asyncProgressBarWrapper');
        var itemId = progressBar.fly(beginOffset,
            this.lmsg('uploadTask').replace('%%database%%', this._databaseName.escapeHTML()),
            function () {
                context.hide();
                new Ajax.Request(Jsw.prepareUrl('/database/upload-dump/id/' + context._databaseId), {
                    method: 'post',
                    parameters: {
                        recreateDatabase: recreateDatabase,
                        dumpPath: context._isWebspace ? context._tree.getSelectedItemData().path : '',
                    },
                    onSuccess: function () {
                        progressBar.removePreparingItem(itemId);
                        progressBar.update();
                    },
                });
            });
    },

    _onFinishUpload: function () {
        var context = this;

        var beginOffset = this._buttonOk.cumulativeOffset();
        var progressBar = Jsw.getComponent('asyncProgressBarWrapper');
        var itemId = progressBar.fly(beginOffset,
            this.lmsg('uploadTask').replace('%%database%%', this._databaseName.escapeHTML()),
            function () {
                context.hide();
                progressBar.removePreparingItem(itemId);
                progressBar.update();
            });
    },

    _onCancelClick: function () {
        this.hide();
    },
});

Smb.Database.ConnectionInfoPopup = Class.create(Jsw.PopupForm, {
    _initConfiguration: function ($super, config) {
        $super(config);
        this._serverAddress = this._getConfigParam('serverAddress');
        this._databaseName = this._getConfigParam('databaseName');
        this._userLogin = this._getConfigParam('userLogin');
        this._userPassword = this._getConfigParam('userPassword');
    },

    render: function ($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this.lmsg('connectionInfoTitle').replace('%%database%%', this._databaseName.escapeHTML()));
        this.setHint1(this.lmsg('connectionInfoDesc'));

        $(this._contentAreaId).update(
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('connectionInfoServerAddress') + '</div>' +
                '<div class="field-value"><div class="text-value"><b>' + this._serverAddress.escapeHTML() + '</b></div></div>' +
            '</div>' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('connectionInfoDatabaseName') + '</div>' +
                '<div class="field-value"><div class="text-value"><b>' + this._databaseName.escapeHTML() + '</b></div></div>' +
            '</div>' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('connectionInfoUserLogin') + '</div>' +
                '<div class="field-value"><div class="text-value"><b>' + this._userLogin.escapeHTML() + '</b></div></div>' +
            '</div>' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('connectionInfoUserPassword') + '</div>' +
                '<div class="field-value"><div class="text-value"><b>' + this._userPassword.escapeHTML() + '</b></div></div>' +
            '</div>'
        );

        this.addRightButton(this.lmsg('buttonClose'), function () {
            this.hide();
        }.bind(this));
    },
});
