// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

Jsw.namespace('Smb.MailSettingsPopupManager');

Smb.MailSettingsPopupManager.MailSettingsPopupForm = Class.create(Jsw.PopupForm, {

    _initConfiguration: function ($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
        this._handlerUrl = this._getConfigParam('handlerUrl', '');
        this._listId = this._getConfigParam('listId', 'mail-settings-list');
        this._isOutgoingMailControlAllowed = this._getConfigParam('isOutgoingMailControlAllowed', false);
    },

    render: function ($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this.lmsg(this._id + 'Title'));

        this.addOkButton();
        this.addCancelButton();
    },

    _initComponentElement: function ($super) {
        $super();

        this._componentElement.down('div.content-area').update(
            '<div>' + this.lmsg(this._id + 'Hint') + '</div>' +
            '<hr>' +
            this._getFormContent() +
            '<hr>' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('selectedDomainsHint') + '</div>' +
                '<div class="field-value">' +
                    '<div class="text-value">' + this._getDomainsList() + '</div>' +
                '</div>' +
            '</div>'
        );
    },

    addOkButton: function () {
        this.addRightButton(this.lmsg('buttonOk'), this._onOkClick, true, true, { id: this._sendButtonId });
    },

    addCancelButton: function () {
        this.addRightButton(this.lmsg('buttonCancel'), this._onCancelClick, false, false, { id: this._cancelButtonId });
    },

    _onOkClick: function () {
        this._handler();
    },

    _onCancelClick: function () {
        this.hide();
    },

    disable: function () {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = true;
            sendButtonWrapper.addClassName('disabled');
            if (!this._sendButtonTitle) {
                this._sendButtonTitle = sendButton.innerHTML;
            }
            sendButton.update('<span class="wait">' + this.lmsg('loading') + '</span>');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = true;
            cancelButtonWrapper.addClassName('disabled');
        }
    },

    enable: function () {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = false;
            sendButton.update(this._sendButtonTitle);
            sendButtonWrapper.removeClassName('disabled');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = false;
            cancelButtonWrapper.removeClassName('disabled');
        }
    },

    _onException: function (transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
        this.enable();
    },

    _onSuccess: function (request) {
        Jsw.clearStatusMessages();

        try {
            var response = request.responseText.evalJSON();
            this._processForm(response);
        } catch (e) {
            // show error, if request wasn't aborted by user
            if (0 !== request.status) {
                Jsw.showInternalError(request.responseText);
                this.hide();
            }
        }
    },

    _processForm: function (response) {
        if (response.redirect) {
            if (response.postData) {
                Jsw.redirectPost(response.redirect, response.postData, response.target);
            } else {
                Jsw.redirect(response.redirect);
            }
        } else if (response.status && 'error' === response.status) {
            $A(response.statusMessages).each(function (value) {
                Jsw.addStatusMessage(value.status, value.content, { title: value.title });
            });

            this.hide();
        } else {
            document.location.reload();
        }
    },

    _getDomainsList: function () {
        var domainsListHtml = '';
        Jsw.getComponent(this._listId).getSelectedItems().each(function (item) {
            domainsListHtml += '<li>' + item.name + '</li>';
        });
        return '<ul>' + domainsListHtml + '</ul>';
    },

    _handler: function () {
        this.disable();

        var params = {};

        var formElements = $$('#' + this._id + '-content-area :input');
        formElements.each(function (element) {
            if ('radio' === element.type) {
                if (element.checked) {
                    params[element.name] = element.value;
                }
            } else {
                params[element.id] = element.value;
            }
        });

        params['ids[]'] = Jsw.getComponent(this._listId).getSelectedItemsIds();

        new Ajax.Request(Jsw.baseUrl + this._handlerUrl, {
            method: 'post',
            parameters: params,
            onException: this._onException.bind(this),
            onSuccess: this._onSuccess.bind(this),
        });
    },
});

Smb.MailSettingsPopupManager.SwitchServicePopupForm = Class.create(Smb.MailSettingsPopupManager.MailSettingsPopupForm, {

    _initConfiguration: function ($super, config) {
        $super(config);
        this._isMailSettingsManagementAllowed = this._getConfigParam('isMailSettingsManagementAllowed', false);
        this._isMailListsManagementAllowed = this._getConfigParam('isMailListsManagementAllowed', false);
        this._isGreylistingManagementAllowed = this._getConfigParam('isGreylistingManagementAllowed', false);
        this._isSignOutgoingManagementAllowed = this._getConfigParam('isSignOutgoingManagementAllowed', false);
    },

    _getFormContent: function () {
        var optionsHtml = '';
        ['intact', 'on', 'off'].each(function (item) {
            optionsHtml += '<option value="' + item + '">' + this.lmsg(item + 'Service') + '</option>';
        }, this);

        var mailSettingsContent = (this._isMailSettingsManagementAllowed)
            ? '<div class="form-row">' +
                    '<div class="field-name">' + this.lmsg('fieldMailService') + '</div>' +
                    '<div class="field-value">' +
                    '<select id="mail-settings-mail-service">' + optionsHtml + '</select>' +
                    '</div>' +
                '</div>'
            : '';
        var mailListingContent = (this._isMailListsManagementAllowed)
            ? '<div class="form-row">' +
                    '<div class="field-name">' + this.lmsg('fieldMailingLists') + '</div>' +
                    '<div class="field-value">' +
                    '<select id="mail-settings-mailing-lists">' + optionsHtml + '</select>' +
                    '</div>' +
                '</div>'
            : '';
        var SpamFilterAndMailSettingsContent = '';
        if (this._isSignOutgoingManagementAllowed) {
            SpamFilterAndMailSettingsContent = SpamFilterAndMailSettingsContent +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('fieldDomainKeys') + '</div>' +
                '<div class="field-value">' +
                '<select id="mail-settings-domain-keys">' + optionsHtml + '</select>' +
                '</div>' +
            '</div>';
        }
        if (this._isGreylistingManagementAllowed) {
            SpamFilterAndMailSettingsContent = SpamFilterAndMailSettingsContent +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('fieldGreylisting') + '</div>' +
                '<div class="field-value">' +
                    '<select id="mail-settings-greylisting">' + optionsHtml + '</select>' +
                '</div>' +
            '</div>';
        }

        var contentHtml = mailSettingsContent + mailListingContent + SpamFilterAndMailSettingsContent;
        return contentHtml;
    },
});

Smb.MailSettingsPopupManager.WebmailPopupForm = Class.create(Smb.MailSettingsPopupManager.MailSettingsPopupForm, {

    _getFormContent: function () {
        var domainSettings = Jsw.getComponent(this._listId).getSelectedItems().first();
        var webmails = domainSettings.availableWebmails;

        var webmailsOptionsHtml = '';
        $H(webmails).each(function (item) {
            webmailsOptionsHtml += '<option value="' + item.key.escapeHTML() + '">' + item.value.escapeHTML() + '</option>';
        });

        return '' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('fieldWebmail') + '</div>' +
                '<div class="field-value">' +
                    '<select id="mail-settings-webmail">' + webmailsOptionsHtml + '</select>' +
                '</div>' +
            '</div>';
    },

});

Smb.MailSettingsPopupManager.BounceSettingsPopupForm = Class.create(Smb.MailSettingsPopupManager.MailSettingsPopupForm, {

    _initConfiguration: function ($super, config) {
        $super(config);
        this._catchAllAddressCanBeSpecified = this._getConfigParam('catchAllAddressCanBeSpecified', false);
    },

    _getFormContent: function () {
        var domainSettings = Jsw.getComponent(this._listId).getSelectedItems().first();
        var availableOptions = domainSettings.availableNonExist;
        var rejectOptionHtml = '';
        var discardOptionHtml = '';
        var bounceOptionHtml = '';
        var catchAllOptionHtml = '';
        var catchIpOptionHtml = '';

        if (availableOptions.bounce) {
            bounceOptionHtml =
            '<div class="indent-box">' +
                '<input type="radio" value="bounce" id="nonExist-bounce" class="radio" name="nonExist">' +
                '<div class="indent-box-content">' +
                    '<label for="nonExist-bounce">' + this.lmsg('optionBounceWithMessage') + '</label>' +
                    '<p class="hint">' +
                    '<textarea name="bounceMessage" id="bounceMessage" rows="5" cols="50" disabled="" class="disabled">' + this.lmsg('bounceMessage') + '</textarea>' +
                    '</p>' +
                '</div>' +
            '</div>';
        }

        if (availableOptions.catch) {
            var catchAllHintHtml = (this._isOutgoingMailControlAllowed) ? '<span class=\"hint\">' + this.lmsg('optionBounceForwardDescription') + '</span>' : '';
            var catchAllCtrlDisabled = this._isOutgoingMailControlAllowed && !this._catchAllAddressCanBeSpecified && Jsw.getComponent(this._listId).getSelectedItems().length > 1;
            var catchAllCtrlDisabledHtml = catchAllCtrlDisabled ? 'disabled=""' : '';

            catchAllOptionHtml =
                '<div class="indent-box">' +
                    '<input type="radio" value="catch" id="nonExist-catch" class="radio" ' + catchAllCtrlDisabledHtml + ' name="nonExist">' +
                    '<div class="indent-box-content">' +
                        '<label for="nonExist-catch">' + this.lmsg('optionBounceForward') + '</label>' +
                        '<p class="hint">' +
                        '<input type="text" name="catchAddress" id="catchAddress" value="admin@example.com" class="input-text disabled" disabled="disabled">' +
                        '</p>' +
                        catchAllHintHtml +
                    '</div>' +
                '</div>';
        }

        if (availableOptions.external) {
            catchIpOptionHtml =
            '<div class="indent-box">' +
                '<input type="radio" value="external" id="nonExist-external" class="radio" name="nonExist">' +
                '<div class="indent-box-content">' +
                    '<label for="nonExist-external">' + this.lmsg('optionCatchIp') + '</label>' +
                    '<p class="hint">' +
                    '<input type="text" name="catchIp" id="catchIp" value="" class="input-text disabled" disabled="">' +
                    '</p>' +
                '</div>' +
            '</div>';
        }

        if (availableOptions.reject) {
            rejectOptionHtml =
            '<div class="indent-box">' +
                '<input type="radio" value="reject" id="nonExist-reject" class="radio" name="nonExist" checked="checked">' +
                '<div class="indent-box-content">' +
                    '<label for="nonExist-reject">' + this.lmsg('optionBounceReject') + '</label>' +
                '</div>' +
            '</div>';
        }

        if (availableOptions.discard) {
            discardOptionHtml =
            '<div class="indent-box">' +
                '<input type="radio" value="discard" id="nonExist-discard" class="radio" name="nonExist" checked="checked">' +
                '<div class="indent-box-content">' +
                '<label for="nonExist-discard">' + this.lmsg('optionBounceReject') + '</label>' +
                '</div>' +
            '</div>';
        }

        return '' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('fieldBounce') + '</div>' +
                '<div class="field-value">' +
                    '<div class="text-value">' +

                        bounceOptionHtml +
                        catchAllOptionHtml +
                        catchIpOptionHtml +
                        rejectOptionHtml +
                        discardOptionHtml +

                    '</div>' +
                '</div>' +
            '</div>';
    },

    _addEvents: function ($super) {
        $super();

        var multiOptions = $H({
            'nonExist-bounce': ['bounceMessage'],
            'nonExist-catch': ['catchAddress'],
            'nonExist-external': ['catchIp'],
            'nonExist-reject': null,
            'nonExist-discard': null,
        });

        var updateControlsState = function () {
            multiOptions.each(function (option) {
                if (option.value) {
                    option.value.each(function (control) {
                        if (!$(option.key)) {
                            return;
                        }
                        if ($(option.key).checked) {
                            $(control).enable();
                        } else {
                            $(control).disable();
                        }
                    });
                }
            });
        };

        multiOptions.each(function (option) {
            if ($(option.key)) {
                $(option.key).observe('click', updateControlsState);
            }
        });

        updateControlsState();
    },
});

Smb.MailSettingsPopupManager.DomainOutgoingSpamSettingsPopupForm = Class.create(Smb.MailSettingsPopupManager.MailSettingsPopupForm, {

    _getFormContent: function () {
        return '' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('fieldOutgoingMessages') + '</div>' +
                '<div class="field-value">' +
                    '<div class="field-limits">' +
                        '<div class="indent-box">' +
                            '<input type="radio" value="default" id="outgoing-messages-default" class="radio" name="radioOutgoingMessages" checked="checked">' +
                            '<div class="indent-box-content">' +
                                '<label for="outgoing-messages-default">' + this.lmsg('fieldOutgoingMessagesDefaultOption') + '</label>' +
                            '</div>' +
                        '</div>' +
                        '<div class="indent-box">' +
                            '<input type="radio" value="custom" id="outgoing-messages-custom" class="radio" name="radioOutgoingMessages">' +
                            '<div class="indent-box-content">' +
                                '<label for="outgoing-messages-custom">' + this.lmsg('fieldOutgoingMessagesCustomOption') + '</label><br>' +
                                '<input type="hidden" name="OutgoingMessages" id="outgoing-messages" value="default">' +
                                '<input type="text" name="customOutgoingMessagesValue" id="outgoing-messages-value"' +
                                    ' value="0" class="f-limits input-text disabled" disabled="disabled" maxlength="9"' +
                                '>' +
                                '&nbsp;' + this.lmsg('fieldOutgoingMessagesCustomOptionHint') +
                                '<span class="indent-box-unlimited">' +
                                    '<input type="checkbox" class="checkbox disabled" name="OutgoingMessagesUnlimited" id="outgoing-messages-unlimited" value="default">' +
                                    '<label for="outgoing-messages-unlimited">' + this.lmsg('fieldOutgoingMessagesCustomOptionUnlimited') + '</label>' +
                                '</span>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _addEvents: function ($super) {
        $super();

        var updateElementValueFunction = function (event) {
            var elementPrefix = 'outgoing-messages';
            if ($(elementPrefix + '-default') && $(elementPrefix + '-default').checked) {
                if ($(elementPrefix + '-unlimited')) {
                    $(elementPrefix + '-unlimited').disable();
                }
                $(elementPrefix + '-value').disable();
                $(elementPrefix).setValue($(elementPrefix + '-default').getValue());
                return;
            }

            if ($(elementPrefix + '-unlimited')) {
                $(elementPrefix + '-unlimited').enable();
            }
            $(elementPrefix + '-value').enable();
            if ($(elementPrefix + '-unlimited') && $(elementPrefix + '-unlimited').checked) {
                $(elementPrefix + '-value').disable();
                $(elementPrefix).setValue(-1);
            } else {
                $(elementPrefix + '-value').enable();
                if (event && Event.element(event).id === elementPrefix + '-unlimited') {
                    try {
                        $(elementPrefix + '-value').focus();
                    } catch (e) {}
                }
                $(elementPrefix).setValue($(elementPrefix + '-value').getValue());
            }
        };

        $('outgoing-messages-default').observe('change', updateElementValueFunction);
        $('outgoing-messages-custom').observe('change', updateElementValueFunction);
        $('outgoing-messages-value').observe('change', updateElementValueFunction);
        if ($('outgoing-messages-unlimited')) {
            $('outgoing-messages-unlimited').observe('click', updateElementValueFunction);
        }

        updateElementValueFunction();
    },

});

Smb.MailSettingsPopupManager.MailnameOutgoingSpamSettingsPopupForm = Class.create(Smb.MailSettingsPopupManager.DomainOutgoingSpamSettingsPopupForm, {

    _initComponentElement: function ($super) {
        $super();

        this._componentElement.down('div.content-area').update(
            '<div>' + this.lmsg(this._id + 'Hint') + '</div>' +
            '<hr>' +
            this._getFormContent() +
            '<hr>' +
            '<div class="form-row">' +
                '<div class="field-name">' + this.lmsg('selectedDomainsHint') + '</div>' +
                '<div class="field-value">' +
                    '<div class="text-value">' + this._getMailnameList() + '</div>' +
                '</div>' +
            '</div>'
        );
    },

    _getMailnameList: function () {
        var domainsListHtml = '';
        Jsw.getComponent(this._listId).getSelectedItems().each(function (item) {
            domainsListHtml += '<li>' + item.name.escapeHTML() + '@' + item.domainName.escapeHTML() + '</li>';
        });
        return '<ul>' + domainsListHtml + '</ul>';
    },
});
