// Copyright 1999-2017. Plesk International GmbH. All rights reserved.

Jsw.namespace('Smb.WebSitesAndDomains');

Smb.WebSitesAndDomains.UpdateProgressNotification = Class.create(Jsw.Component, {

    STATUS_WORKING: 'working',
    STATUS_DONE: 'done',
    STATUS_NOTWORKING: 'notworking',

    updateInterval: 5,

    _initConfiguration: function ($super, config) {
        $super(config);
        this._message = this._getConfigParam('message');
    },

    _addEvents: function () {
        this._processStatus({
            status: this.STATUS_WORKING,
            message: this._message,
        });
    },

    _scheduleUpdateStatus: function () {
        this._statusUpdate.bind(this).delay(this.updateInterval);
    },

    _statusUpdate: function () {
        new Ajax.Request('/admin/update/get-update-data', {
            method: 'get',
            onSuccess: this._onSuccess.bind(this),
            onFailure: this._onFailure.bind(this),
        });
    },


    _onSuccess: function (transport) {
        try {
            this._processStatus(transport.responseText.evalJSON());
        } catch (e) {
            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();
        }
    },


    _onFailure: function (transport) {
        // Schedule new update - control panel is probably down
        // or othewise unavailable due to updates installation.
        // Have no information to display.

        this._scheduleUpdateStatus();
    },


    _processStatus: function (response) {
        if (!response) {
            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();
        } else if (this.STATUS_WORKING === response.status) {
            this._componentElement.update(
                '<div class="msg-box msg-warning msg-update-running">' +
                    '<div><div><div><div><div>' +
                    '<div class="msg-content">' +
                    this._message +
                    '</div>' +
                    '</div></div></div></div></div>' +
                    '</div>'
            );

            this._scheduleUpdateStatus();
        } else if (this.STATUS_DONE === response.status || this.STATUS_NOTWORKING === response.status) {
            // Stop asynchronous notification and reload the whole page
            Jsw.redirect('/');
        }
    },

});

Smb.WebSitesAndDomains.UpdateResultNotification = Class.create(Jsw.Component, {

    _initConfiguration: function ($super, config) {
        $super(config);

        this._status = this._getConfigParam('status', 'info');
        this._message = this._getConfigParam('message', '');
    },

    _initComponentElement: function ($super) {
        $super();

        this._updateComponentElement(
            '<div class="msg-box msg-' + this._status + '"><div><div><div><div><div>' +
                '<div class="msg-content">' + this._message + '</div></div></div></div></div></div></div>'
        );
    },

});

Smb.WebSitesAndDomains.StickySidebar = Class.create({

    initialize: function (element, options) {
        this.element = $(element);
        this.options = options || {};
        this.offsetTop = this.options.offsetTop || 0;
        this.offsetBottom = this.options.offsetBottom || 0;

        Event.observe(window, 'resize', this.update.bindAsEventListener(this));
        Event.observe(window, 'scroll', this.update.bindAsEventListener(this));
        Event.observe(document, 'mousewheel', this.update.bindAsEventListener(this));
    },

    update: function () {
        var viewportTop = document.viewport.getScrollOffsets().top + this.offsetTop;
        var viewportBottom = viewportTop + document.viewport.getHeight() - this.offsetTop - this.offsetBottom;

        this.element.childElements().each(function (element) {
            var margin = parseInt(element.getStyle('marginTop'));
            var elementTop = element.cumulativeOffset().top;
            var elementBottom = elementTop + element.getHeight();
            var delta;
            var deltaTop;

            if (elementBottom < viewportBottom) {
                var deltaBottom = viewportBottom - elementBottom;
                delta = Math.min(deltaBottom, this.element.getHeight() - element.getHeight() - margin);
                if (elementTop < viewportTop) {
                    deltaTop = viewportTop - elementTop;
                    delta = Math.min(deltaTop, delta);
                } else {
                    delta = 0;
                }
                if (delta > 0) {
                    element.setStyle({
                        marginTop: (margin + delta) + 'px',
                        height: element.getHeight() + 'px',
                    });
                    if (delta === deltaBottom || delta === deltaTop) {
                        element.down().setStyle({
                            bottom: delta !== deltaTop && delta === deltaBottom ? this.offsetBottom + 'px' : null,
                            top: delta === deltaTop ? this.offsetTop + 'px' : null,
                            width: parseInt(element.down().getStyle('width')) + 'px',
                        }).addClassName('fixed');
                    } else {
                        delta = 0;
                    }
                }
            }

            if (elementTop > viewportTop) {
                deltaTop = elementTop - viewportTop;
                delta = Math.min(deltaTop, margin);
                if (delta > 0) {
                    element.setStyle({
                        marginTop: (margin - delta) + 'px',
                        height: element.getHeight() + 'px',
                    });
                    if (delta === deltaTop) {
                        element.down().setStyle({
                            top: this.offsetTop + 'px',
                            width: parseInt(element.down().getStyle('width')) + 'px',
                        }).addClassName('fixed');
                    } else {
                        delta = 0;
                    }
                }
            }

            if (!delta) {
                element.setStyle({
                    height: null,
                });
                element.down().removeClassName('fixed').setStyle({
                    width: null,
                    top: null,
                    bottom: null,
                });
            }
        }, this);
    },
});

Jsw.onReady(Jsw.priority.low, function () {
    var config = {
        offsetTop: $$('.page-content-wrapper').first().cumulativeOffset().top + ($('classic-mode-navigation')
            ? $('classic-mode-navigation').getHeight() + parseInt($('classic-mode-navigation').getStyle('marginBottom'))
            : parseInt($$('.heading').first().getStyle('marginBottom'))),
    };

    var stickySidebar = new Smb.WebSitesAndDomains.StickySidebar($$('.b-content-wrap').first(), config);
    $$('.active-list-details, .panel-content').each(function (element) {
        $w('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd').each(function (ev) {
            element.observe(ev, stickySidebar.update.bindAsEventListener(stickySidebar));
        });
    });
});

Jsw.onReady(function () {
    $$('.so-panel-version').each(function (element) {
        element.observe('click', function (event) {
            Event.stop(event);
            var position = element.positionedOffset();
            var popUp = $$('.pb-panel-version').first();
            var popUpWidth = popUp.down('.popup-wrapper').getWidth();
            var leftOffset = position[0] - popUpWidth + 'px';
            var topOffset = position[1] + 'px';
            popUp.setStyle({ left: leftOffset, top: topOffset });
            popUp.toggleClassName('collapsed');
        });

        $$('.pb-panel-version').each(function (popUp) {
            popUp.observe('click', function (event) {
                event = Event.extend(event);
                event.stopPropagation();
            });

            popUp.down('.close').observe('click', function () {
                $$('.pb-panel-version').first().addClassName('collapsed');
            });
        });

        $(document.body).observe('click', function () {
            this.down('.pb-panel-version').addClassName('collapsed');
        });
    });
});

