# Copyright 1999-2017. Plesk International GmbH. All rights reserved.
import os
import logging
import sys
import plesk_config
import traceback
from xml.dom import minidom

_logger = logging.getLogger("api.util.error_reporter")

mswindows = (sys.platform == "win32")


class ErrorReporter(object):

    @staticmethod
    def sendException(exception, stack_trace=None):
        if stack_trace is None:
            stack_trace = traceback.format_stack()
        ErrorReporter._sendMessage(str(exception) + " at " + "".join(stack_trace), str(type(exception)))

    @staticmethod
    def sendNonzeroExitException(exception):
        ErrorReporter.sendNonzeroExitError(
            exception.subprocess.get_cmd(),
            exception.exitcode,
            exception.subprocess.stdout,
            exception.subprocess.stderr,
            exception.stackTrace)

    @staticmethod
    def sendNonzeroExitError(cmd, code, stdout, stderr, stack_trace=None):
        if stack_trace is None:
            stack_trace = traceback.format_stack()
        message = """Command: %s
        Code: %d
        STDOUT: %s
        STDERR: %s
        Stacktrace: %s""" % (cmd, code, stdout, stderr, "".join(stack_trace))
        ErrorReporter._sendMessage(message, 'NonzeroExitError')


    @staticmethod
    def sendUnhandledException(exception):
        ErrorReporter._sendMessage(str(exception), 'UnhandledException')


    @staticmethod
    def _sendMessage(message, message_type):
        import subproc
        try:
            xml_doc = minidom.Document()
            error = xml_doc.createElement('error')
            module = xml_doc.createElement('module')
            module.appendChild(xml_doc.createTextNode('pmmcli'))
            operation = xml_doc.createElement('operation')
            operation.appendChild(xml_doc.createTextNode(message_type))
            message_element = xml_doc.createElement('message')
            message_element.appendChild(xml_doc.createTextNode(message))
            xml_doc.appendChild(error)
            error.appendChild(module)
            error.appendChild(operation)
            error.appendChild(message_element)

            util_path = os.path.join(plesk_config.get("PRODUCT_ROOT_D"), "admin", "bin", "send-error-report")
            if mswindows:
                util_path += ".exe"

            _logger.debug("send-error-report " + util_path + " " + xml_doc.toxml())

            cmd = subproc.CmdLine(util_path, None, [], os.environ, xml_doc.toxml())
            cmd.arg('send-error-report')
            cmd.arg('pmm')
            cmd.asyncSpawn()
        except Exception, e:
            _logger.debug(str(e))
