# Copyright 1999-2017. Plesk International GmbH. All rights reserved.
import logging

_logger = logging.getLogger("parts_cacher")


class PartNotFound(Exception):
    def __init__(self, id):
        self.id = id
    def __str__(self):
        return "MIME-part with Content-Id '%s' is not found" % self.id


class PartsCacher(object):
    """Reads the supplied backup file, caching non-requested backup parts."""

    def __init__(self, backup, cids=None):
        self.parts = backup.getContents()
        self.cached_parts = {}
        self.cids = cids

    def processPart(self, id, processor):
        _logger.info("Searching for MIME-part '%s'" % id)

        if id in self.cached_parts:
            _logger.debug("Removing '%s' from MIME-parts cache" % id)
            part = self.cached_parts[id] # do not use method .pop which is only in python 2.3 and later
            del self.cached_parts[id]
            _logger.info("Processing cached MIME-part '%s'" % id)
            return processor(part)

        _logger.debug("'%s' is not found in cache" % id)

        for (partId, part) in self.parts:
            _logger.debug("MIME-part '%s' found" % partId)

            if self.cids is not None:
                if not partId in self.cids:
                    _logger.debug("Skip MIME-part '%s'" % partId)
                    continue
                del self.cids[partId]

            if partId == id:
                _logger.info("Processing MIME-part '%s' from stream" % id)
                return processor(part)
            else:
                _logger.debug("Fixating MIME-part '%s'" % partId)
                part.fixate(partId)
                _logger.info("MIME-part '%s' added to cache" % partId)
                self.cached_parts[partId] = part

        _logger.info("MIME-part '%s' is not found in backup" % id)

        raise PartNotFound(id)

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
