<?xml version="1.0" encoding="utf-8"?>
<!-- Copyright 1999-2017. Plesk International GmbH. All rights reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" indent="yes"/>

    <!-- Add encryption-type attribute if possible. -->
    <xsl:template match="/migration-dump[@verification-string!='' and not(@encryption-type)]">
        <xsl:copy>
            <xsl:attribute name="encryption-type">
                <xsl:text>password</xsl:text>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()" />
        </xsl:copy>
    </xsl:template>

    <!-- Add iis_app_pool_addons template item. -->
    <xsl:template match="domain-template">
        <xsl:copy>
            <xsl:apply-templates select="@*" />
            <xsl:if test="count(/migration-dump/dump-info/os-description[@type = 'windows'])=1">
                <xsl:element name="template-item">
                    <xsl:attribute name="name">
                        <xsl:text>iis_app_pool_addons</xsl:text>
                    </xsl:attribute>
                    <xsl:text>false</xsl:text>
                </xsl:element>
            </xsl:if>
            <xsl:apply-templates select="node()" />
        </xsl:copy>
    </xsl:template>

    <!-- Add iis-application-pool section to sites. -->
    <xsl:template match="sites/site/phosting/preferences">
        <xsl:copy>
            <xsl:copy-of select="sysuser"/>
            <xsl:copy-of select="fpuser"/>
            <xsl:copy-of select="logrotation"/>
            <xsl:copy-of select="anonftp"/>
            <xsl:copy-of select="pdir"/>
            <xsl:copy-of select="webalizer"/>
            <xsl:copy-of select="sb-domain"/>
            <xsl:copy-of select="perfomance"/>
            <xsl:copy-of select="hotlink-protection"/>
            <xsl:copy-of select="shared-ssl"/>
            <xsl:copy-of select="web-stat"/>
            <xsl:if test="count(/migration-dump/dump-info/os-description[@type = 'windows'])=1">
                <xsl:element name="iis-application-pool">
                    <xsl:attribute name="turned-on">
                        <xsl:text>false</xsl:text>
                    </xsl:attribute>
                </xsl:element>
            </xsl:if>
            <xsl:copy-of select="dynamic-ip-security"/>
            <xsl:copy-of select="virtual-directories"/>
        </xsl:copy>
    </xsl:template>

    <!-- Add iis-application-pool section to subdomains. -->
    <xsl:template match="subdomains/subdomain/preferences">
        <xsl:copy>
            <xsl:copy-of select="hotlink-protection"/>
            <xsl:copy-of select="pdir"/>
            <xsl:copy-of select="shared-ssl"/>
            <xsl:copy-of select="perfomance"/>
            <xsl:if test="count(/migration-dump/dump-info/os-description[@type = 'windows'])=1">
                <xsl:element name="iis-application-pool">
                    <xsl:attribute name="turned-on">
                        <xsl:text>false</xsl:text>
                    </xsl:attribute>
                </xsl:element>
            </xsl:if>
            <xsl:copy-of select="virtual-directories"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="template-item[@name = 'manage_phosting']">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()" />
      </xsl:copy>
      <xsl:element name="template-item">
        <xsl:attribute name="name">manage_php_version</xsl:attribute>
        <xsl:value-of select="."/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="template-item[@name = 'allow_local_backups' or @name = 'allow_ftp_backups']">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()" />
      </xsl:copy>
      <xsl:element name="template-item">
        <xsl:attribute name="name"><xsl:value-of select="concat('allow_account_', substring-after(@name, 'allow_'))"/></xsl:attribute>
        <xsl:value-of select="."/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="limits-and-permissions/permission[@name = 'manage_phosting']">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()" />
      </xsl:copy>
      <xsl:element name="permission">
        <xsl:attribute name="name">manage_php_version</xsl:attribute>
        <xsl:attribute name="value"><xsl:value-of select="@value"/></xsl:attribute>
      </xsl:element>
    </xsl:template>

    <xsl:template match="limits-and-permissions/permission[@name = 'allow_local_backups' or @name = 'allow_ftp_backups']">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()" />
      </xsl:copy>
      <xsl:element name="permission">
        <xsl:attribute name="name"><xsl:value-of select="concat('allow_account_', substring-after(@name, 'allow_'))"/></xsl:attribute>
        <xsl:attribute name="value"><xsl:value-of select="@value"/></xsl:attribute>
      </xsl:element>
    </xsl:template>

    <!-- Add status attribute. -->
    <xsl:template match="mailuser/preferences/spamassassin[not(@status)]">
        <xsl:copy>
            <xsl:attribute name="status">
                <xsl:text>off</xsl:text>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()" />
        </xsl:copy>
    </xsl:template>

    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" />
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
