# Copyright 1999-2017. Plesk International GmbH. All rights reserved.
import cPickle
import logging
import os
import sqlite3
import urllib

import dirutil
import osutil
import pmm_config

_logger = logging.getLogger("pmm_upgrader")


def upgrade_tasks_storage_12_1_19():
    tasks_directory = pmm_config.tasks_dir()
    database = os.path.join(tasks_directory, 'tasks.db')
    if not os.path.exists(database):
        if not os.path.exists(tasks_directory):
            dirutil.mkdirs(tasks_directory, 0750)
        try:
            with sqlite3.connect(database) as connection:
                connection.execute('CREATE TABLE tasks (id INTEGER PRIMARY KEY AUTOINCREMENT, type TEXT, ownerGuid TEXT, topObjectId INTEGER, topObjectType TEXT, dump TEXT)')
                connection.execute('CREATE INDEX tasks_type ON tasks (type)')
                connection.execute('CREATE INDEX tasks_ownerGuid ON tasks (ownerGuid)')
                connection.execute('CREATE INDEX tasks_topObjectId ON tasks (topObjectId)')
                connection.execute('CREATE INDEX tasks_topObjectType ON tasks (topObjectType)')
        except:
            osutil.unlink_nothrow(database)
            raise

    with sqlite3.connect(database, isolation_level=None) as connection:
        connection.text_factory = str

        names = os.listdir(tasks_directory)
        for name in names:
            if 'tasks.db' == name:
                continue
            fullname = os.path.join(tasks_directory, name)
            if not os.path.isfile(fullname):
                continue
            try:
                with open(fullname, 'rb') as pickle_file:
                    pickle_str = pickle_file.read()
                    task = cPickle.loads(pickle_str)
                    cursor = connection.cursor()
                    cursor.execute('SELECT dump FROM tasks WHERE id=?', [name])
                    if None == cursor.fetchone():
                        connection.execute('INSERT INTO tasks VALUES (?,?,?,?,?,?)', [
                            name
                            , task.get_tasktype()
                            , task.get('owner_guid') if task.has_key('owner_guid') else None
                            , task.get('topobject_id') if task.has_key('topobject_id') else None
                            , task.get('topobject_type') if task.has_key('topobject_type') else None
                            , urllib.quote(pickle_str)
                        ])
                    osutil.unlink_nothrow(fullname)
            except Exception as e:
                _logger.debug('Unable to convert task #%s: %s' % (name, str(e)))
