# Copyright 1999-2017. Plesk International GmbH. All rights reserved.
import subproc

class PMMUtilityException(subproc.NonzeroExitException):
    def __init__(self, utility, nonzero_exit_exception):
        subproc.NonzeroExitException.__init__(
            self,
            nonzero_exit_exception.subprocess,
            nonzero_exit_exception.exitcode,
            nonzero_exit_exception.sendNotification,
            nonzero_exit_exception.stackTrace)
        nonzero_exit_exception.sendNotification = False
        self.__utility = utility

    def utility(self):
        return self.__utility

    def __str__(self):
        return self.__utility + subproc.NonzeroExitException.__str__(self)

class PMMException(Exception):
    def __init__(self, code, description, resolution):
        Exception.__init__(self, description)
        self.__code = code
        self.__resolution = resolution
        self.__description = description

    def getCode(self):
        return self.__code

    def getResolution(self):
        return self.__resolution

    def getDescription(self):
        return self.__description

def writeExecutionResult(out, ex):
    from execution_result import ExecutionResult, MessageType, DescriptionType

    er = ExecutionResult.factory()
    if ex is None:
        er.set_status('success')
    else:
        er.set_status('error')

        msg = MessageType.factory(severity='error', desription=DescriptionType(valueOf_=str(ex)))
        if ex.getResolution():
            msg.set_resolution(ex.getResolution())
        er.add_message(msg)

    out.write('<?xml version="1.0" encoding="UTF-8"?>\n')
    er.export(out, 0, '', 'execution-result')

class PMMHostNotAccessibleException(PMMException):
    def __init__(self, description):
        PMMException.__init__(self, 130, description)

class PMMAmbiguousObjectSpecifiedException(PMMException):
    def __init__(self, code, description):
        PMMException.__init__(self, code, description, None)

class PMMObjectAbsentException(PMMException):
    def __init__(self, code, description):
        PMMException.__init__(self, code, description, None)